@@ -1,6 +1,6 @@
 // _build.h
 //
-// $Id: _build.h,v 1.54 2006/02/03 06:46:38 sdennis Exp $
+// $Id: _build.h,v 1.60 2006/09/12 00:19:49 sdennis Exp $
 //
 // MUX 2.4
 // Copyright (C) 2005 Stephen Dennis.
@@ -17,10 +17,10 @@ extern char szBuildDate[];
 #define MUX_BUILD_DATE szBuildDate
 #endif // MUX_BUILD_DATE
 
-#define MUX_VERSION       "2.4.2.27"         // Version number
-#define MUX_RELEASE_DATE  "2006-FEB-02"      // Source release date
+#define MUX_VERSION       "2.4.3.31"         // Version number
+#define MUX_RELEASE_DATE  "2006-SEP-11"      // Source release date
 
 // Define if this release is qualified as ALPHA or BETA.
 //
 //#define ALPHA
-#define BETA
+//#define BETA
