@@ -1,10 +1,10 @@
 // cque.cpp -- commands and functions for manipulating the command queue.
 //
-// $Id: cque.cpp,v 1.9 2004/03/08 17:56:12 sdennis Exp $
+// $Id: cque.cpp,v 1.35 2006/01/07 23:46:32 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -18,11 +18,9 @@
 #include "interface.h"
 #include "powers.h"
 
-extern int  a_Queue(dbref, int);
-extern int  QueueMax(dbref);
 bool break_called = false;
 
-CLinearTimeDelta GetProcessorUsage(void)
+static CLinearTimeDelta GetProcessorUsage(void)
 {
     CLinearTimeDelta ltd;
 #ifdef WIN32
@@ -36,31 +34,6 @@ CLinearTimeDelta GetProcessorUsage(void)
         ltd.Set100ns(*(INT64 *)(&ftUser));
         return ltd;
     }
-
-    // Win9x - We can really only report the system time with a
-    // high-resolution timer. This doesn't separate the time from
-    // this process from other processes that are also running on
-    // the same machine, but it's the best we can do on Win9x.
-    //
-    // The scheduling in Win9x is totally screwed up, so even if
-    // Win9x did provide the information, it would certainly be
-    // wrong.
-    //
-    if (bQueryPerformanceAvailable)
-    {
-        // The QueryPerformanceFrequency call at game startup
-        // succeeded. The frequency number is the number of ticks
-        // per second.
-        //
-        INT64 li;
-        if (QueryPerformanceCounter((LARGE_INTEGER *)&li))
-        {
-            li = QP_A*li + (QP_B*li+QP_C)/QP_D;
-            ltd.Set100ns(li);
-            return ltd;
-        }
-        bQueryPerformanceAvailable = false;
-    }
 #endif
 
 #if !defined(WIN32) && defined(HAVE_GETRUSAGE)
@@ -110,12 +83,15 @@ static int add_to(dbref executor, int am, int attrnum)
 // This Task assumes that pEntry is already unlinked from any lists it may
 // have been related to.
 //
-void Task_RunQueueEntry(void *pEntry, int iUnused)
+static void Task_RunQueueEntry(void *pEntry, int iUnused)
 {
+    UNUSED_PARAMETER(iUnused);
+
     BQUE *point = (BQUE *)pEntry;
     dbref executor = point->executor;
 
-    if (Good_obj(executor) && !Going(executor))
+    if (  Good_obj(executor)
+       && !Going(executor))
     {
         giveto(executor, mudconf.waitcost);
         mudstate.curr_enactor = point->enactor;
@@ -142,42 +118,47 @@ void Task_RunQueueEntry(void *pEntry, int iUnused)
             }
 
             char *command = point->comm;
+
+            mux_assert(!mudstate.inpipe);
+            mux_assert(mudstate.pipe_nest_lev == 0);
+            mux_assert(mudstate.poutobj == NOTHING);
+            mux_assert(!mudstate.pout);
+
             break_called = false;
-            while (command && !break_called)
+            while (  command
+                  && !break_called)
             {
+                mux_assert(!mudstate.poutnew);
+                mux_assert(!mudstate.poutbufc);
+
                 char *cp = parse_to(&command, ';', 0);
-                if (cp && *cp)
+
+                if (  cp
+                   && *cp)
                 {
-                    int numpipes = 0;
-                    while (  command
-                          && (*command == '|')
-                          && (numpipes < mudconf.ntfy_nest_lim))
+                    // Will command be piped?
+                    //
+                    if (  command
+                       && *command == '|'
+                       && mudstate.pipe_nest_lev < mudconf.ntfy_nest_lim)
                     {
                         command++;
-                        numpipes++;
+                        mudstate.pipe_nest_lev++;
                         mudstate.inpipe = true;
-                        mudstate.poutnew = alloc_lbuf("process_command.pipe");
+
+                        mudstate.poutnew  = alloc_lbuf("process_command.pipe");
                         mudstate.poutbufc = mudstate.poutnew;
-                        mudstate.poutobj = executor;
-
-                        // No lag check on piped commands.
-                        //
-                        process_command(executor, point->caller, point->enactor,
-                            false, cp, point->env, point->nargs);
-                        if (mudstate.pout)
-                        {
-                            free_lbuf(mudstate.pout);
-                            mudstate.pout = NULL;
-                        }
-
-                        *mudstate.poutbufc = '\0';
-                        mudstate.pout = mudstate.poutnew;
-                        cp = parse_to(&command, ';', 0);
+                        mudstate.poutobj  = executor;
+                    }
+                    else
+                    {
+                        mudstate.inpipe = false;
+                        mudstate.poutobj = NOTHING;
                     }
-                    mudstate.inpipe = false;
 
                     CLinearTimeAbsolute ltaBegin;
                     ltaBegin.GetUTC();
+                    MuxAlarm.Set(mudconf.max_cmdsecs);
                     CLinearTimeDelta ltdUsageBegin = GetProcessorUsage();
 
                     char *log_cmdbuf = process_command(executor, point->caller,
@@ -185,19 +166,22 @@ void Task_RunQueueEntry(void *pEntry, int iUnused)
 
                     CLinearTimeAbsolute ltaEnd;
                     ltaEnd.GetUTC();
+                    if (MuxAlarm.bAlarmed)
+                    {
+                        notify(executor, "GAME: Expensive activity abbreviated.");
+                        s_Flags(point->enactor, FLAG_WORD1, Flags(point->enactor) | HALT);
+                        s_Flags(point->executor, FLAG_WORD1, Flags(point->executor) | HALT);
+                        halt_que(point->enactor, NOTHING);
+                        halt_que(executor, NOTHING);
+                    }
+                    MuxAlarm.Clear();
 
                     CLinearTimeDelta ltdUsageEnd = GetProcessorUsage();
                     CLinearTimeDelta ltd = ltdUsageEnd - ltdUsageBegin;
                     db[executor].cpu_time_used += ltd;
 
-                    if (mudstate.pout)
-                    {
-                        free_lbuf(mudstate.pout);
-                        mudstate.pout = NULL;
-                    }
-
                     ltd = ltaEnd - ltaBegin;
-                    if (ltd > mudconf.max_cmdsecs)
+                    if (ltd > mudconf.rpt_cmdsecs)
                     {
                         STARTLOG(LOG_PROBLEMS, "CMD", "CPU");
                         log_name_and_loc(executor);
@@ -210,7 +194,33 @@ void Task_RunQueueEntry(void *pEntry, int iUnused)
                         ENDLOG;
                     }
                 }
+
+                // Transition %| value.
+                //
+                if (mudstate.pout)
+                {
+                    free_lbuf(mudstate.pout);
+                    mudstate.pout = NULL;
+                }
+                if (mudstate.poutnew)
+                {
+                    *mudstate.poutbufc = '\0';
+                    mudstate.pout = mudstate.poutnew;
+                    mudstate.poutnew  = NULL;
+                    mudstate.poutbufc = NULL;
+                }
             }
+
+            // Clean up %| value.
+            //
+            if (mudstate.pout)
+            {
+                free_lbuf(mudstate.pout);
+                mudstate.pout = NULL;
+            }
+            mudstate.pipe_nest_lev = 0;
+            mudstate.inpipe = false;
+            mudstate.poutobj = NOTHING;
         }
         MEMFREE(point->text);
         point->text = NULL;
@@ -229,13 +239,19 @@ void Task_RunQueueEntry(void *pEntry, int iUnused)
 //
 static bool que_want(BQUE *entry, dbref ptarg, dbref otarg)
 {
-    if ((ptarg != NOTHING) && (ptarg != Owner(entry->executor)))
+    if (  ptarg != NOTHING
+       && ptarg != Owner(entry->executor))
+    {
         return false;
-    return ((otarg == NOTHING) || (otarg == entry->executor));
+    }
+    return (  otarg == NOTHING
+           || otarg == entry->executor);
 }
 
-void Task_SemaphoreTimeout(void *pExpired, int iUnused)
+static void Task_SemaphoreTimeout(void *pExpired, int iUnused)
 {
+    UNUSED_PARAMETER(iUnused);
+
     // A semaphore has timed out.
     //
     BQUE *point = (BQUE *)pExpired;
@@ -244,18 +260,31 @@ void Task_SemaphoreTimeout(void *pExpired, int iUnused)
     Task_RunQueueEntry(point, 0);
 }
 
-dbref Halt_Player_Target;
-dbref Halt_Object_Target;
-int   Halt_Entries;
-dbref Halt_Player_Run;
-dbref Halt_Entries_Run;
+#ifdef QUERY_SLAVE
+void Task_SQLTimeout(void *pExpired, int iUnused)
+{
+    // A SQL Query has timed out.
+    //
+    BQUE *point = (BQUE *)pExpired;
+    Task_RunQueueEntry(point, 0);
+}
+#endif // QUERY_SLAVE
 
-int CallBack_HaltQueue(PTASK_RECORD p)
+static dbref Halt_Player_Target;
+static dbref Halt_Object_Target;
+static int   Halt_Entries;
+static dbref Halt_Player_Run;
+static dbref Halt_Entries_Run;
+
+static int CallBack_HaltQueue(PTASK_RECORD p)
 {
     if (  p->fpTask == Task_RunQueueEntry
+#ifdef QUERY_SLAVE
+       || p->fpTask == Task_SQLTimeout
+#endif // QUERY_SLAVE
        || p->fpTask == Task_SemaphoreTimeout)
     {
-        // This is a @wait or timed semaphore task.
+        // This is a @wait, timed Semaphore Task, or timed SQL Query.
         //
         BQUE *point = (BQUE *)(p->arg_voidptr);
         if (que_want(point, Halt_Player_Target, Halt_Object_Target))
@@ -328,6 +357,9 @@ int halt_que(dbref executor, dbref object)
 //
 void do_halt(dbref executor, dbref caller, dbref enactor, int key, char *target)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref executor_targ, obj_targ;
 
     if ((key & HALT_ALL) && !Can_Halt(executor))
@@ -392,15 +424,15 @@ void do_halt(dbref executor, dbref caller, dbref enactor, int key, char *target)
     notify(Owner(executor), tprintf("%d queue entr%s removed.", numhalted, numhalted == 1 ? "y" : "ies"));
 }
 
-int Notify_Key;
-int Notify_Num_Done;
-int Notify_Num_Max;
-int Notify_Sem;
-int Notify_Attr;
+static int Notify_Key;
+static int Notify_Num_Done;
+static int Notify_Num_Max;
+static int Notify_Sem;
+static int Notify_Attr;
 
 // NFY_DRAIN or NFY_NFYALL
 //
-int CallBack_NotifySemaphoreDrainOrAll(PTASK_RECORD p)
+static int CallBack_NotifySemaphoreDrainOrAll(PTASK_RECORD p)
 {
     if (p->fpTask == Task_SemaphoreTimeout)
     {
@@ -436,7 +468,7 @@ int CallBack_NotifySemaphoreDrainOrAll(PTASK_RECORD p)
                 {
                     p->iPriority = PRIORITY_OBJECT;
                 }
-                p->ltaWhen = mudstate.now;
+                p->ltaWhen.GetUTC();
                 p->fpTask = Task_RunQueueEntry;
                 return IU_UPDATE_TASK;
             }
@@ -447,7 +479,7 @@ int CallBack_NotifySemaphoreDrainOrAll(PTASK_RECORD p)
 
 // NFY_NFY or NFY_QUIET
 //
-int CallBack_NotifySemaphoreFirstOrQuiet(PTASK_RECORD p)
+static int CallBack_NotifySemaphoreFirstOrQuiet(PTASK_RECORD p)
 {
     // If we've notified enough, exit.
     //
@@ -479,7 +511,7 @@ int CallBack_NotifySemaphoreFirstOrQuiet(PTASK_RECORD p)
             {
                 p->iPriority = PRIORITY_OBJECT;
             }
-            p->ltaWhen = mudstate.now;
+            p->ltaWhen.GetUTC();
             p->fpTask = Task_RunQueueEntry;
             return IU_UPDATE_TASK;
         }
@@ -548,8 +580,10 @@ void do_notify
     char *count
 )
 {
-    int loccount, attr = 0;
-    ATTR *ap;
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     char *obj = parse_to(&what, '/', 0);
     init_match(executor, obj, NOTYPE);
     match_everything(0);
@@ -565,35 +599,31 @@ void do_notify
     }
     else
     {
-        if (!what || !*what)
+        int atr = A_SEMAPHORE;
+        if (  what
+           && what[0] != '\0')
         {
-            ap = NULL;
-        }
-        else
-        {
-            ap = atr_str(what);
-        }
-
-        if (!ap)
-        {
-            attr = A_SEMAPHORE;
-        }
-        else
-        {
-            // Do they have permission to set this attribute?
-            //
-            if (bCanSetAttr(executor, thing, ap))
+            int i = mkattr(executor, what);
+            if (0 < i)
             {
-                attr = ap->number;
-            }
-            else
-            {
-                notify_quiet(executor, NOPERM_MESSAGE);
-                return;
+                atr = i;
+                if (atr != A_SEMAPHORE)
+                {
+                    // Do they have permission to set this attribute?
+                    //
+                    ATTR *ap = (ATTR *)anum_get(atr);
+                    if (!bCanSetAttr(executor, thing, ap))
+                    {
+                        notify_quiet(executor, NOPERM_MESSAGE);
+                        return;
+                    }
+                }
             }
         }
 
-        if (count && *count)
+        int loccount;
+        if (  count
+           && count[0] != '\0')
         {
             loccount = mux_atol(count);
         }
@@ -603,7 +633,7 @@ void do_notify
         }
         if (loccount > 0)
         {
-            nfy_que(thing, attr, key, loccount);
+            nfy_que(thing, atr, key, loccount);
             if (  (!(Quiet(executor) || Quiet(thing)))
                && key != NFY_QUIET)
             {
@@ -653,6 +683,8 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
     a = QueueMax(Owner(executor));
     if (a_Queue(Owner(executor), 1) > a)
     {
+        a_Queue(Owner(executor), -1);
+
         notify(Owner(executor),
             "Run away objects: too many commands queued.  Halted.");
         halt_que(Owner(executor), NOTHING);
@@ -708,7 +740,7 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
     tmp->comm = NULL;
 
     char *tptr = tmp->text = (char *)MEMALLOC(tlen);
-    (void)ISOUTOFMEMORY(tptr);
+    ISOUTOFMEMORY(tptr);
 
     if (command)
     {
@@ -834,6 +866,58 @@ void wait_que
     }
 }
 
+#ifdef QUERY_SLAVE
+// ---------------------------------------------------------------------------
+// sql_que: Add commands to the sql queue.
+//
+void sql_que
+(
+    dbref executor,
+    dbref caller,
+    dbref enactor,
+    bool bTimed,
+    CLinearTimeAbsolute &ltaWhen,
+    dbref thing,
+    int   attr,
+    char *command,
+    char *args[],
+    int   nargs,
+    char *sargs[]
+)
+{
+    if (!(mudconf.control_flags & CF_INTERP))
+    {
+        return;
+    }
+
+    BQUE *tmp = setup_que(executor, caller, enactor, command, args, nargs, sargs);
+    if (!tmp)
+    {
+        return;
+    }
+
+    tmp->IsTimed = bTimed;
+    tmp->waittime = ltaWhen;
+    tmp->sem = thing;
+    tmp->attr = attr;
+
+    int iPriority;
+    if (!tmp->IsTimed)
+    {
+        // In this case, the timeout task below will never run,
+        // but it allows us to manage all semaphores together in
+        // the same data structure.
+        //
+        iPriority = PRIORITY_SUSPEND;
+    }
+    else
+    {
+        iPriority = PRIORITY_OBJECT;
+    }
+    scheduler.DeferTask(tmp->waittime, iPriority, Task_SQLTimeout, tmp, 0);
+}
+#endif // QUERY_SLAVE
+
 // ---------------------------------------------------------------------------
 // do_wait: Command interface to wait_que
 //
@@ -890,7 +974,7 @@ void do_wait
     {
         // Get timeout, default 0.
         //
-        int attr = A_SEMAPHORE;
+        int atr = A_SEMAPHORE;
         bool bTimed = false;
         if (event && *event)
         {
@@ -913,17 +997,17 @@ void do_wait
                 ATTR *ap = atr_str(event);
                 if (!ap)
                 {
-                    attr = mkattr(event);
-                    if (attr <= 0)
+                    atr = mkattr(executor, event);
+                    if (atr <= 0)
                     {
                         notify_quiet(executor, "Invalid attribute.");
                         return;
                     }
-                    ap = atr_num(attr);
+                    ap = atr_num(atr);
                 }
                 else
                 {
-                    attr = ap->number;
+                    atr = ap->number;
                 }
                 if (!bCanSetAttr(executor, thing, ap))
                 {
@@ -933,7 +1017,7 @@ void do_wait
             }
         }
 
-        int num = add_to(thing, 1, attr);
+        int num = add_to(thing, 1, atr);
         if (num <= 0)
         {
             // Thing over-notified, run the command immediately.
@@ -941,35 +1025,85 @@ void do_wait
             thing = NOTHING;
             bTimed = false;
         }
-        wait_que(executor, caller, enactor, bTimed, ltaWhen, thing, attr,
+        wait_que(executor, caller, enactor, bTimed, ltaWhen, thing, atr,
             cmd, cargs, ncargs, mudstate.global_regs);
     }
 }
 
-CLinearTimeAbsolute Show_lsaNow;
-int Total_SystemTasks;
-int Total_RunQueueEntry;
-int Shown_RunQueueEntry;
-int Total_SemaphoreTimeout;
-int Shown_SemaphoreTimeout;
-dbref Show_Player_Target;
-dbref Show_Object_Target;
-int Show_Key;
-dbref Show_Player;
-int Show_bFirstLine;
+#ifdef QUERY_SLAVE
+// ---------------------------------------------------------------------------
+// do_query: Command interface to sql_que
+//
+void do_query
+(
+    dbref executor,
+    dbref caller,
+    dbref enactor,
+    int   key,
+    char *dbref_attr,
+    char *dbname_query,
+    char *cargs[],
+    int   ncargs
+)
+{
+    if (key & QUERY_SQL)
+    {
+        // SQL Query.
+        //
+        dbref thing;
+        ATTR *pattr;
 
-#ifdef WIN32
-void Task_FreeDescriptor(void *arg_voidptr, int arg_Integer);
-#endif
-void dispatch_DatabaseDump(void *pUnused, int iUnused);
-void dispatch_FreeListReconstruction(void *pUnused, int iUnused);
-void dispatch_IdleCheck(void *pUnused, int iUnused);
-void dispatch_CheckEvents(void *pUnused, int iUnused);
-#ifndef MEMORY_BASED
-void dispatch_CacheTick(void *pUnused, int iUnused);
-#endif
+        if (!( parse_attrib(executor, dbref_attr, &thing, &pattr)
+            && pattr))
+        {
+            notify_quiet(executor, "No match.");
+            return;
+        }
 
-int CallBack_ShowDispatches(PTASK_RECORD p)
+        if (!Controls(executor, thing))
+        {
+            notify_quiet(executor, NOPERM_MESSAGE);
+            return;
+        }
+
+        char *pQuery = dbname_query;
+        char *pDBName = parse_to(&pQuery, '/', 0);
+
+        if (NULL == pQuery)
+        {
+            notify(executor, "QUERY: No Query.");
+            return;
+        }
+
+        STARTLOG(LOG_ALWAYS, "CMD", "QUERY");
+        Log.tinyprintf("Thing=#%d, Attr=%s, dbname=%s, query=%s", thing, pattr->name, pDBName, pQuery);
+        ENDLOG;
+    }
+    else
+    {
+        notify_quiet(executor, "At least one query option is required.");
+    }
+}
+#endif // QUERY_SLAVE
+
+static CLinearTimeAbsolute Show_lsaNow;
+static int Total_SystemTasks;
+static int Total_RunQueueEntry;
+static int Shown_RunQueueEntry;
+static int Total_SemaphoreTimeout;
+static int Shown_SemaphoreTimeout;
+static dbref Show_Player_Target;
+static dbref Show_Object_Target;
+static int Show_Key;
+static dbref Show_Player;
+static int Show_bFirstLine;
+
+#ifdef QUERY_SLAVE
+int Total_SQLTimeout;
+int Shown_SQLTimeout;
+#endif // QUERY_SLAVE
+
+static int CallBack_ShowDispatches(PTASK_RECORD p)
 {
     Total_SystemTasks++;
     CLinearTimeDelta ltd = p->ltaWhen - Show_lsaNow;
@@ -1004,6 +1138,10 @@ int CallBack_ShowDispatches(PTASK_RECORD p)
     {
         notify(Show_Player, tprintf("[%d]Delayed descriptor deallocation", ltd.ReturnSeconds()));
     }
+    else if (p->fpTask == Task_DeferredClose)
+    {
+        notify(Show_Player, tprintf("[%d]Delayed socket close", ltd.ReturnSeconds()));
+    }
 #endif
     else
     {
@@ -1012,7 +1150,7 @@ int CallBack_ShowDispatches(PTASK_RECORD p)
     return IU_NEXT_TASK;
 }
 
-void ShowPsLine(BQUE *tmp)
+static void ShowPsLine(BQUE *tmp)
 {
     char *bufp = unparse_object(Show_Player, tmp->executor, false);
     if (tmp->IsTimed && (Good_obj(tmp->sem)))
@@ -1055,7 +1193,7 @@ void ShowPsLine(BQUE *tmp)
     free_lbuf(bufp);
 }
 
-int CallBack_ShowWait(PTASK_RECORD p)
+static int CallBack_ShowWait(PTASK_RECORD p)
 {
     if (p->fpTask != Task_RunQueueEntry)
     {
@@ -1081,7 +1219,7 @@ int CallBack_ShowWait(PTASK_RECORD p)
     return IU_NEXT_TASK;
 }
 
-int CallBack_ShowSemaphore(PTASK_RECORD p)
+static int CallBack_ShowSemaphore(PTASK_RECORD p)
 {
     if (p->fpTask != Task_SemaphoreTimeout)
     {
@@ -1107,11 +1245,42 @@ int CallBack_ShowSemaphore(PTASK_RECORD p)
     return IU_NEXT_TASK;
 }
 
+#ifdef QUERY_SLAVE
+int CallBack_ShowSQLQueries(PTASK_RECORD p)
+{
+    if (p->fpTask != Task_SQLTimeout)
+    {
+        return IU_NEXT_TASK;
+    }
+
+    Total_SQLTimeout++;
+    BQUE *tmp = (BQUE *)(p->arg_voidptr);
+    if (que_want(tmp, Show_Player_Target, Show_Object_Target))
+    {
+        Shown_SQLTimeout++;
+        if (Show_Key == PS_SUMM)
+        {
+            return IU_NEXT_TASK;
+        }
+        if (Show_bFirstLine)
+        {
+            notify(Show_Player, "----- SQL Queries -----");
+            Show_bFirstLine = false;
+        }
+        ShowPsLine(tmp);
+    }
+    return IU_NEXT_TASK;
+}
+#endif
+
 // ---------------------------------------------------------------------------
 // do_ps: tell executor what commands they have pending in the queue
 //
 void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     char *bufp;
     dbref executor_targ, obj_targ;
 
@@ -1185,6 +1354,10 @@ void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
     scheduler.TraverseOrdered(CallBack_ShowWait);
     Show_bFirstLine = true;
     scheduler.TraverseOrdered(CallBack_ShowSemaphore);
+#ifdef QUERY_SLAVE
+    Show_bFirstLine = true;
+    scheduler.TraverseOrdered(CallBack_ShowSQLQueries);
+#endif // QUERY_SLAVE
     if (Wizard(executor))
     {
         notify(executor, "----- System Queue -----");
@@ -1194,9 +1367,16 @@ void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
     // Display stats.
     //
     bufp = alloc_mbuf("do_ps");
+#ifdef QUERY_SLAVE
+    sprintf(bufp, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d  SQL %d/%d",
+        Shown_RunQueueEntry, Total_RunQueueEntry,
+        Shown_SemaphoreTimeout, Total_SemaphoreTimeout,
+        Shown_SQLTimeout, Total_SQLTimeout);
+#else
     sprintf(bufp, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d",
         Shown_RunQueueEntry, Total_RunQueueEntry,
         Shown_SemaphoreTimeout, Total_SemaphoreTimeout);
+#endif // QUERY_SLAVE
     notify(executor, bufp);
     if (Wizard(executor))
     {
@@ -1206,10 +1386,14 @@ void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
     free_mbuf(bufp);
 }
 
-CLinearTimeDelta ltdWarp;
-int CallBack_Warp(PTASK_RECORD p)
+static CLinearTimeDelta ltdWarp;
+static int CallBack_Warp(PTASK_RECORD p)
 {
-    if (p->fpTask == Task_RunQueueEntry || p->fpTask == Task_SemaphoreTimeout)
+    if (  p->fpTask == Task_RunQueueEntry
+#ifdef QUERY_SLAVE
+       || p->fpTask == Task_SQLTimeout
+#endif // QUERY_SLAVE
+       || p->fpTask == Task_SemaphoreTimeout)
     {
         BQUE *point = (BQUE *)(p->arg_voidptr);
         if (point->IsTimed)
@@ -1226,6 +1410,9 @@ int CallBack_Warp(PTASK_RECORD p)
 //
 void do_queue(dbref executor, dbref caller, dbref enactor, int key, char *arg)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     if (key == QUEUE_KICK)
     {
         int i = mux_atol(arg);
