@@ -1,6 +1,6 @@
 // create.cpp -- Commands that create new objects.
 //
-// $Id: create.cpp,v 1.8 2004/09/21 06:33:03 sdennis Exp $
+// $Id: create.cpp,v 1.20 2006/01/31 00:17:26 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -66,7 +66,7 @@ static void open_exit(dbref player, dbref loc, char *direction, char *linkto)
         if(!(Open_ok(loc) && could_doit(player, loc, A_LOPEN)))
         {
             notify_quiet(player, NOPERM_MESSAGE);
-            return;         
+            return;
         }
     }
     dbref exit = create_obj(player, TYPE_EXIT, direction, 0);
@@ -80,6 +80,7 @@ static void open_exit(dbref player, dbref loc, char *direction, char *linkto)
     s_Exits(exit, loc);
     s_Next(exit, Exits(loc));
     s_Exits(loc, exit);
+    local_data_create(exit);
 
     // and we're done
     //
@@ -122,6 +123,9 @@ static void open_exit(dbref player, dbref loc, char *direction, char *linkto)
 void do_open(dbref executor, dbref caller, dbref enactor, int key,
              char *direction, char *links[], int nlinks)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     char *dest;
 
     // Create the exit and link to the destination, if there is one
@@ -232,6 +236,8 @@ void do_link
     char *where
 )
 {
+    UNUSED_PARAMETER(nargs);
+
     // Find the thing to link
     //
     init_match(executor, what, TYPE_EXIT);
@@ -304,19 +310,19 @@ void do_link
             s_Home(thing, nHomeNew);
             if (!Quiet(executor))
             {
-                char *buff = alloc_lbuf("do_link.notify");
-                char *bp = buff;
+                char *buff1 = alloc_lbuf("do_link.notify");
+                char *bp = buff1;
 
                 char *p;
                 p = tprintf("Home of %s(#%d) changed from ", Name(thing), thing);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 p = tprintf("%s(#%d) to ", Name(nHomeOrig), nHomeOrig);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 p = tprintf("%s(#%d).", Name(nHomeNew), nHomeNew);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 *bp = '\0';
-                notify_quiet(executor, buff);
-                free_lbuf(buff);
+                notify_quiet(executor, buff1);
+                free_lbuf(buff1);
             }
         }
         break;
@@ -356,19 +362,19 @@ void do_link
             s_Dropto(thing, room);
             if (!Quiet(executor))
             {
-                char *buff = alloc_lbuf("do_link2.notify");
-                char *bp = buff;
+                char *buff1 = alloc_lbuf("do_link2.notify");
+                char *bp = buff1;
 
                 char *p;
                 p = tprintf("Dropto of %s(#%d) changed from ", Name(thing), thing);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 p = tprintf("%s(#%d) to ", Name(nDroptoOrig), nDroptoOrig);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 p = tprintf("%s(#%d).", Name(nDroptoNew), nDroptoNew);
-                safe_str(p, buff, &bp);
+                safe_str(p, buff1, &bp);
                 *bp = '\0';
-                notify_quiet(executor, buff);
-                free_lbuf(buff);
+                notify_quiet(executor, buff1);
+                free_lbuf(buff1);
             }
         }
         break;
@@ -404,6 +410,11 @@ void do_parent
     char *pname
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     dbref thing, parent, curr;
     int lev;
 
@@ -478,6 +489,8 @@ void do_parent
 void do_dig(dbref executor, dbref caller, dbref enactor, int key, char *name,
             char *args[], int nargs)
 {
+    UNUSED_PARAMETER(caller);
+
     // we don't need to know player's location!  hooray!
     //
     if (!name || !*name)
@@ -491,6 +504,7 @@ void do_dig(dbref executor, dbref caller, dbref enactor, int key, char *name,
         return;
     }
 
+    local_data_create(room);
     notify(executor, tprintf("%s created as room #%d.", name, room));
 
     char *buff = alloc_sbuf("do_dig");
@@ -529,6 +543,10 @@ void do_create
     char *coststr
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     int cost = 0;
     if (!name || !*name)
     {
@@ -553,6 +571,8 @@ void do_create
     {
         notify(executor, tprintf("%s created as object #%d", Name(thing), thing));
     }
+
+    local_data_create(thing);
 }
 
 
@@ -570,6 +590,10 @@ void do_clone
     char *arg2
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     dbref clone, thing, new_owner, loc;
     FLAG rmv_flags;
     int cost;
@@ -608,12 +632,12 @@ void do_clone
     //
     if (  !Controls(executor, thing)
        && !Parent_ok(thing)
-       && (key & CLONE_PARENT))
+       && (key & CLONE_FROM_PARENT))
     {
         notify_quiet(executor,
               tprintf("You don't control %s, ignoring /parent.",
                   Name(thing)));
-        key &= ~CLONE_PARENT;
+        key &= ~CLONE_FROM_PARENT;
     }
 
     // Determine the cost of cloning
@@ -673,13 +697,16 @@ void do_clone
         return;
     }
 
-    // Wipe out any old attributes and copy in the new data.
+    // Copy in the new data.
     //
-    atr_free(clone);
-    if (key & CLONE_PARENT)
+    if (key & CLONE_FROM_PARENT)
+    {
         s_Parent(clone, thing);
+    }
     else
-        atr_cpy(clone, thing);
+    {
+        atr_cpy(clone, thing, false);
+    }
 
     // Reset the name, since we cleared the attributes.
     //
@@ -752,7 +779,7 @@ void do_clone
     //
     if (new_owner == Owner(thing))
     {
-        if (!(key & CLONE_PARENT))
+        if (!(key & CLONE_FROM_PARENT))
         {
             s_Parent(clone, Parent(thing));
         }
@@ -761,13 +788,15 @@ void do_clone
     }
     else
     {
-        if (  !(key & CLONE_PARENT)
+        if (  !(key & CLONE_FROM_PARENT)
            && (Controls(executor, thing) || Parent_ok(thing)))
         {
             s_Parent(clone, Parent(thing));
         }
         s_Halted(clone);
     }
+
+    local_data_clone(clone, thing);
 }
 
 // ---------------------------------------------------------------------------
@@ -784,6 +813,10 @@ void do_pcreate
     char *pass
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     const char *pmsg;
     bool isrobot = (key == PCRE_ROBOT);
     dbref newplayer = create_player(name, pass, executor, isrobot, &pmsg);
@@ -863,6 +896,9 @@ static bool can_destroy_player(dbref player, dbref victim)
 
 void do_destroy(dbref executor, dbref caller, dbref enactor, int key, char *what)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     // You can destroy anything you control.
     //
     dbref thing = match_controlled_quiet(executor, what);
@@ -986,70 +1022,68 @@ void do_destroy(dbref executor, dbref caller, dbref enactor, int key, char *what
         {
             notify_quiet(ThingOwner,
                 tprintf("You will be rewarded shortly for %s(#%d).",
-                Name(thing), thing));
+                Moniker(thing), thing));
         }
     }
-
+    free_sbuf(NameOfType);
 
     // Imperative Destruction emits.
     //
-    if (  (  !bInstant
-          || isPlayer(thing))
-       && !Quiet(executor))
+    if (!Quiet(executor))
     {
-        if (  Good_owner(ThingOwner)
-           && Owner(executor) != ThingOwner)
+        if (Good_owner(ThingOwner))
         {
-            if (ThingOwner == thing)
+            if (ThingOwner == Owner(executor))
             {
-                notify(executor, tprintf("Destroyed. %s(#%d)",
-                    Name(thing), thing));
+                if (!Quiet(thing))
+                {
+                    notify(executor, tprintf("Destroyed %s(#%d).",
+                        Moniker(thing), thing));
+                }
+            }
+            else if (ThingOwner == thing)
+            {
+                notify(executor, tprintf("Destroyed %s(#%d).",
+                    Moniker(thing), thing));
             }
             else
             {
                 char *tname = alloc_lbuf("destroy_obj");
                 strcpy(tname, Moniker(ThingOwner));
-                notify(executor, tprintf("Destroyed. %s's %s(#%d)",
-                    tname, Name(thing), thing));
+                notify(executor, tprintf("Destroyed %s's %s(#%d).",
+                    tname, Moniker(thing), thing));
                 free_lbuf(tname);
             }
         }
-        else if (!Quiet(thing))
+    }
+
+    if (bInstant)
+    {
+        // Instant destruction by type.
+        //
+        switch (Typeof(thing))
         {
-            notify(executor, "Destroyed.");
+        case TYPE_EXIT:
+            destroy_exit(thing);
+            break;
+
+        case TYPE_PLAYER:
+            destroy_player(executor, thing);
+            break;
+
+        case TYPE_ROOM:
+            empty_obj(thing);
+            destroy_obj(thing);
+            break;
+
+        case TYPE_THING:
+            destroy_thing(thing);
+            break;
+
+        default:
+            notify(executor, "Weird object type cannot be destroyed.");
+            return;
         }
     }
-    free_sbuf(NameOfType);
     s_Going(thing);
-
-    if (!bInstant)
-    {
-        return;
-    }
-
-    // Instant destruction by type.
-    //
-    switch (Typeof(thing))
-    {
-    case TYPE_EXIT:
-        destroy_exit(thing);
-        break;
-
-    case TYPE_PLAYER:
-        destroy_player(executor, thing);
-        break;
-
-    case TYPE_ROOM:
-        empty_obj(thing);
-        destroy_obj(thing);
-        break;
-
-    case TYPE_THING:
-        destroy_thing(thing);
-        break;
-
-    default:
-        notify(executor, "Weird object type cannot be destroyed.");
-        break;
-    }
 }
