@@ -1,6 +1,6 @@
 // db.cpp
 //
-// $Id: db.cpp,v 1.78 2006/01/31 00:17:26 sdennis Exp $
+// $Id: db.cpp,v 1.81 2006/09/05 23:42:17 sdennis Exp $
 //
 // MUX 2.4
 // Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
@@ -17,6 +17,7 @@
 #include "interface.h"
 #include "powers.h"
 #include "vattr.h"
+#include "ansi.h"
 
 #ifndef O_ACCMODE
 #define O_ACCMODE   (O_RDONLY|O_WRONLY|O_RDWR)
@@ -1290,6 +1291,8 @@ int mkattr(dbref executor, char *buff)
     return -1;
 }
 
+#ifndef MEMORY_BASED
+
 /* ---------------------------------------------------------------------------
  * al_decode: Fetch an attribute number from an alist.
  */
@@ -1344,6 +1347,8 @@ static char *al_code(char *ap, int atrnum)
     return ap;
 }
 
+#endif // MEMORY_BASED
+
 /* ---------------------------------------------------------------------------
  * Commer: check if an object has any $-commands in its attributes.
  */
@@ -1735,41 +1740,44 @@ void atr_clr(dbref thing, int atr)
 {
 #ifdef MEMORY_BASED
 
-    if (!db[thing].at_count || !db[thing].ahead)
+    if (  !db[thing].nALUsed
+       || !db[thing].pALHead)
     {
         return;
     }
 
-    mux_assert(0 <= db[thing].at_count);
+    mux_assert(0 <= db[thing].nALUsed);
 
     // Binary search for the attribute.
     //
     int lo = 0;
     int mid;
-    int hi = db[thing].at_count - 1;
-    ATRLIST *list = db[thing].ahead;
+    int hi = db[thing].nALUsed - 1;
+    ATRLIST *list = db[thing].pALHead;
     while (lo <= hi)
     {
         mid = ((hi - lo) >> 1) + lo;
-        if (list[mid].number == atr)
-        {
-            MEMFREE(list[mid].data);
-            list[mid].data = NULL;
-            db[thing].at_count -= 1;
-            if (mid != db[thing].at_count)
-            {
-                memmove(list+mid, list+mid+1, (db[thing].at_count - mid) * sizeof(ATRLIST));
-            }
-            break;
-        }
-        else if (list[mid].number > atr)
+        if (list[mid].number > atr)
         {
             hi = mid - 1;
         }
-        else
+        else if (list[mid].number < atr)
         {
             lo = mid + 1;
         }
+        else // (list[mid].number == atr)
+        {
+            MEMFREE(list[mid].data);
+            list[mid].data = NULL;
+            db[thing].nALUsed--;
+            if (mid != db[thing].nALUsed)
+            {
+                memmove( list + mid,
+                         list + mid + 1,
+                         (db[thing].nALUsed - mid) * sizeof(ATRLIST));
+            }
+            break;
+        }
     }
 #else // MEMORY_BASED
     Aname okey;
@@ -1834,7 +1842,8 @@ void atr_clr(dbref thing, int atr)
 
 void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
 {
-    if (!szValue || szValue[0] == '\0')
+    if (  !szValue
+       || '\0' == szValue[0])
     {
         atr_clr(thing, atr);
         return;
@@ -1846,15 +1855,16 @@ void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
     int hi, lo, mid;
     char *text = StringCloneLen(szValue, nValue);
 
-    if (!db[thing].ahead)
+    if (!db[thing].pALHead)
     {
-        list = (ATRLIST *)MEMALLOC(sizeof(ATRLIST));
+        db[thing].nALAlloc = INITIAL_ATRLIST_SIZE;
+        list = (ATRLIST *)MEMALLOC(db[thing].nALAlloc*sizeof(ATRLIST));
         ISOUTOFMEMORY(list);
-        db[thing].ahead = list;
-        db[thing].at_count = 1;
+        db[thing].pALHead  = list;
+        db[thing].nALUsed  = 1;
         list[0].number = atr;
         list[0].data = text;
-        list[0].size = nValue+1;
+        list[0].size = nValue + 1;
         found = true;
     }
     else
@@ -1862,60 +1872,72 @@ void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
         // Binary search for the attribute
         //
         lo = 0;
-        hi = db[thing].at_count - 1;
+        hi = db[thing].nALUsed - 1;
 
-        list = db[thing].ahead;
+        list = db[thing].pALHead;
         while (lo <= hi)
         {
             mid = ((hi - lo) >> 1) + lo;
-            if (list[mid].number == atr)
-            {
-                MEMFREE(list[mid].data);
-                list[mid].data = text;
-                list[mid].size = nValue+1;
-                found = true;
-                break;
-            }
-            else if (list[mid].number > atr)
+            if (list[mid].number > atr)
             {
                 hi = mid - 1;
             }
-            else
+            else if (list[mid].number < atr)
             {
                 lo = mid + 1;
             }
+            else // if (list[mid].number == atr)
+            {
+                MEMFREE(list[mid].data);
+                list[mid].data = text;
+                list[mid].size = nValue + 1;
+                found = true;
+                break;
+            }
         }
 
-
         if (!found)
         {
-            // If we got here, we didn't find it, so lo = hi + 1,
-            // and the attribute should be inserted between them.
+            // We didn't find it, and lo == hi + 1.  The attribute should be
+            // inserted between (0,hi) and (lo,nALUsed-1).
             //
-            list = (ATRLIST *)MEMALLOC((db[thing].at_count + 1) * sizeof(ATRLIST));
-            ISOUTOFMEMORY(list);
-
-            // Copy bottom part.
-            //
-            if (lo > 0)
+            if (db[thing].nALUsed < db[thing].nALAlloc)
             {
-                memcpy(list, db[thing].ahead, lo * sizeof(ATRLIST));
+                memmove( list + lo + 1,
+                         list + lo,
+                         (db[thing].nALUsed - lo) * sizeof(ATRLIST));
             }
-
-            // Copy top part.
-            //
-            if (lo < db[thing].at_count)
+            else
             {
-                memcpy(list+lo+1, db[thing].ahead+lo, (db[thing].at_count - lo) * sizeof(ATRLIST));
-            }
-            MEMFREE(db[thing].ahead);
-            db[thing].ahead = NULL;
+                // Expand the list.
+                //
+                db[thing].nALAlloc += ATRLIST_CHUNK;
+                list = (ATRLIST *)MEMALLOC(db[thing].nALAlloc
+                     * sizeof(ATRLIST));
+                ISOUTOFMEMORY(list);
 
+                // Copy bottom part.
+                //
+                if (lo > 0)
+                {
+                    memcpy(list, db[thing].pALHead, lo * sizeof(ATRLIST));
+                }
+    
+                // Copy top part.
+                //
+                if (lo < db[thing].nALUsed)
+                {
+                    memcpy( list + lo + 1,
+                            db[thing].pALHead + lo,
+                            (db[thing].nALUsed - lo) * sizeof(ATRLIST));
+                }
+                MEMFREE(db[thing].pALHead);
+                db[thing].pALHead = list;
+            }
+            db[thing].nALUsed++;
             list[lo].data = text;
             list[lo].number = atr;
-            list[lo].size = nValue+1;
-            db[thing].at_count++;
-            db[thing].ahead = list;
+            list[lo].size = nValue + 1;
         }
     }
 
@@ -2057,31 +2079,31 @@ const char *atr_get_raw_LEN(dbref thing, int atr, size_t *pLen)
 
     // Binary search for the attribute.
     //
-    ATRLIST *list = db[thing].ahead;
+    ATRLIST *list = db[thing].pALHead;
     if (!list)
     {
         return NULL;
     }
 
     int lo = 0;
-    int hi = db[thing].at_count - 1;
+    int hi = db[thing].nALUsed - 1;
     int mid;
     while (lo <= hi)
     {
         mid = ((hi - lo) >> 1) + lo;
-        if (list[mid].number == atr)
-        {
-            *pLen = list[mid].size - 1;
-            return list[mid].data;
-        }
-        else if (list[mid].number > atr)
+        if (list[mid].number > atr)
         {
             hi = mid - 1;
         }
-        else
+        else if (list[mid].number < atr)
         {
             lo = mid + 1;
         }
+        else // if (list[mid].number == atr)
+        {
+            *pLen = list[mid].size - 1;
+            return list[mid].data;
+        }
     }
     *pLen = 0;
     return NULL;
@@ -2253,12 +2275,13 @@ bool atr_pget_info(dbref thing, int atr, dbref *owner, int *flags)
 void atr_free(dbref thing)
 {
 #ifdef MEMORY_BASED
-    if (db[thing].ahead)
+    if (db[thing].pALHead)
     {
-        MEMFREE(db[thing].ahead);
+        MEMFREE(db[thing].pALHead);
     }
-    db[thing].ahead = NULL;
-    db[thing].at_count = 0;
+    db[thing].pALHead  = NULL;
+    db[thing].nALAlloc = 0;
+    db[thing].nALUsed  = 0;
 #else // MEMORY_BASED
     char *as;
     atr_push();
@@ -2370,14 +2393,14 @@ int atr_next(char **attrp)
     else
     {
         atr = (ATRCOUNT *) * attrp;
-        if (atr->count >= db[atr->thing].at_count)
+        if (atr->count >= db[atr->thing].nALUsed)
         {
             MEMFREE(atr);
             atr = NULL;
             return 0;
         }
         atr->count++;
-        return db[atr->thing].ahead[atr->count - 1].number;
+        return db[atr->thing].pALHead[atr->count - 1].number;
     }
 
 #else // MEMORY_BASED
@@ -2444,14 +2467,14 @@ void atr_pop(void)
 int atr_head(dbref thing, char **attrp)
 {
 #ifdef MEMORY_BASED
-    if (db[thing].at_count)
+    if (db[thing].nALUsed)
     {
         ATRCOUNT *atr = (ATRCOUNT *) MEMALLOC(sizeof(ATRCOUNT));
         ISOUTOFMEMORY(atr);
         atr->thing = thing;
         atr->count = 1;
         *attrp = (char *)atr;
-        return db[thing].ahead[0].number;
+        return db[thing].pALHead[0].number;
     }
     return 0;
 #else // MEMORY_BASED
@@ -2517,8 +2540,9 @@ static void initialize_objects(dbref first, dbref last)
         s_ThMail(thing, 0);
 
 #ifdef MEMORY_BASED
-        db[thing].ahead = NULL;
-        db[thing].at_count = 0;
+        db[thing].pALHead  = NULL;
+        db[thing].nALAlloc = 0;
+        db[thing].nALUsed  = 0;
 #else
         db[thing].name = NULL;
 #endif // MEMORY_BASED
@@ -2722,17 +2746,21 @@ void putref(FILE *f, dbref ref)
 // Code 1 - NUL  (0x00)
 // Code 2 - '"'  (0x22)
 // Code 3 - '\\' (0x5C)
+// Code 4 - 'e'  (0x65) or 'E' (0x45)
+// Code 5 - 'n'  (0x6E) or 'N' (0x4E)
+// Code 6 - 'r'  (0x72) or 'R' (0x52)
+// Code 7 - 't'  (0x74) or 'T' (0x54)
 //
-static const unsigned char xlat_table[256] =
+static const unsigned char decode_table[256] =
 {
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
+    0, 0, 6, 0, 7, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
+    0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0,
+    0, 0, 6, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -2752,13 +2780,17 @@ static const unsigned char xlat_table[256] =
 // Action 2 - Emit X. Move to START state.
 // Action 3 - Terminate parse.
 // Action 4 - Move to ESC state.
+// Action 5 - Emit ESC (0x1B). Move to START state.
+// Action 6 - Emit LF  (0x0A). Move to START state.
+// Action 7 - Emit CR  (0x0D). Move to START state.
+// Action 8 - Emit TAB (0x09). Move to START state.
 //
 
-static const int action_table[2][4] =
+static const int action_table[2][8] =
 {
-//   Any '\0' "   backslash
-    { 0,  1,  3,  4 }, // STATE_START
-    { 2,  1,  2,  2 }  // STATE_ESC
+//   Any  '\0' '"'  '\\' 'e'  'n'  'r'  't'
+    { 0,   1,   3,   4,   0,   0,   0,   0}, // STATE_START
+    { 2,   1,   2,   2,   5,   6,   7,   8}  // STATE_HAVE_ESC
 };
 
 char *getstring_noalloc(FILE *f, int new_strings)
@@ -2793,20 +2825,20 @@ char *getstring_noalloc(FILE *f, int new_strings)
                 char ch = *pInput++;
                 if (iState == STATE_START)
                 {
-                    if (xlat_table[(unsigned char)ch] == 0)
+                    if (decode_table[(unsigned char)ch] == 0)
                     {
-                        // As long as xlat_table[*p] is 0, just keep copying the characters.
+                        // As long as decode_table[*p] is 0, just keep copying the characters.
                         //
                         char *p = pOutput;
                         do
                         {
                             *pOutput++ = ch;
                             ch = *pInput++;
-                        } while (xlat_table[(unsigned char)ch] == 0);
+                        } while (decode_table[(unsigned char)ch] == 0);
                         nOutput = pOutput - p;
                     }
                 }
-                int iAction = action_table[iState][xlat_table[(unsigned char)ch]];
+                int iAction = action_table[iState][decode_table[(unsigned char)ch]];
                 if (iAction <= 2)
                 {
                     if (iAction == 1)
@@ -2832,13 +2864,37 @@ char *getstring_noalloc(FILE *f, int new_strings)
                     *pOutput = 0;
                     return buf;
                 }
-                else
+                else if (iAction == 4)
                 {
-                    // iAction == 4
                     // Move to ESC state.
                     //
                     iState = STATE_HAVE_ESC;
                 }
+                else if (iAction == 5)
+                {
+                    *pOutput++ = ESC_CHAR;
+                    nOutput++;
+                    iState = STATE_START;
+                }
+                else if (iAction == 6)
+                {
+                    *pOutput++ = '\n';
+                    nOutput++;
+                    iState = STATE_START;
+                }
+                else if (iAction == 7)
+                {
+                    *pOutput++ = '\r';
+                    nOutput++;
+                    iState = STATE_START;
+                }
+                else
+                {
+                    // if (iAction == 8)
+                    *pOutput++ = '\t';
+                    nOutput++;
+                    iState = STATE_START;
+                }
             }
 
             nBufferLeft -= nOutput;
@@ -2892,6 +2948,31 @@ char *getstring_noalloc(FILE *f, int new_strings)
     }
 }
 
+// Code 0 - Any byte.
+// Code 1 - NUL (0x00)
+// Code 2 - '"' (0x22)
+// Code 3 - '\\' (0x5C)
+//
+static const unsigned char encode_table[256] =
+{
+    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
 void putstring(FILE *f, const char *pRaw)
 {
     static char aBuffer[2*LBUF_SIZE+4];
@@ -2907,7 +2988,7 @@ void putstring(FILE *f, const char *pRaw)
         for (;;)
         {
             char ch;
-            while ((ch = xlat_table[(unsigned char)*pRaw]) == 0)
+            while ((ch = encode_table[(unsigned char)*pRaw]) == 0)
             {
                 *pBuffer++ = *pRaw++;
             }
