@@ -1,6 +1,6 @@
 // externs.h -- Prototypes for externs not defined elsewhere.
 //
-// $Id: externs.h,v 1.24 2005/08/25 15:43:47 sdennis Exp $
+// $Id: externs.h,v 1.65 2006/01/31 00:17:26 sdennis Exp $
 //
 
 #ifndef EXTERNS_H
@@ -11,26 +11,84 @@
 #include "mudconf.h"
 #include "svdrand.h"
 
-/* From bsd.cpp */
+// From bsd.cpp.
+//
+void boot_slave(dbref executor, dbref caller, dbref enactor, int key);
+void close_sockets(bool emergency, char *message);
 void CleanUpSlaveSocket(void);
 void CleanUpSlaveProcess(void);
+#ifdef QUERY_SLAVE
+void CleanUpSQLSlaveSocket(void);
+void CleanUpSQLSlaveProcess(void);
+#endif // QUERY_SLAVE
+#ifdef WIN32
+extern HANDLE CompletionPort;    // IOs are queued up on this port
+extern CRITICAL_SECTION csDescriptorList;
+#endif // WIN32
 
-/* From conf.cpp */
-extern void cf_log_notfound(dbref, char *, const char *, char *);
-extern int  cf_modify_bits(int *, char *, void *, UINT32, dbref, char *);
+#ifdef QUERY_SLAVE
+void boot_sqlslave(dbref executor, dbref caller, dbref enactor, int key);
+#endif // QUERY_SLAVE
 
-/* From mail.cpp */
-extern void load_mail(FILE *);
-extern int  dump_mail(FILE *);
-extern struct mail *mail_fetch(dbref, int);
+extern NAMETAB sigactions_nametab[];
 
-/* From netcommon.cpp */
-extern void DCL_CDECL raw_broadcast(int, char *, ...);
+// From conf.cpp
+//
+void cf_log_notfound(dbref, char *, const char *, char *);
+int  cf_modify_bits(int *, char *, void *, UINT32, dbref, char *);
+void DCL_CDECL cf_log_syntax(dbref player, char *cmd, const char *fmt, ...);
+void ValidateConfigurationDbrefs(void);
+int  cf_read(void);
+void cf_init(void);
+void cf_list(dbref, char *, char **);
+void cf_display(dbref, char *, char *, char **);
+void list_cf_access(dbref);
+int cf_set(char *, char *, dbref);
+CF_HAND(cf_cf_access);
+CF_HAND(cf_access);
+CF_HAND(cf_cmd_alias);
+CF_HAND(cf_acmd_access);
+CF_HAND(cf_attr_access);
+CF_HAND(cf_func_access);
+CF_HAND(cf_flag_access);
+CF_HAND(cf_flag_name);
+CF_HAND(cf_art_rule);
+
+// From local.cpp.
+//
+void local_startup(void);
+void local_presync_database(void);
+void local_presync_database_sigsegv(void);
+void local_dump_database(int);
+void local_dump_complete_signal(void);
+void local_shutdown(void);
+void local_dbck(void);
+void local_connect(dbref, int, int);
+void local_disconnect(dbref, int);
+void local_data_create(dbref);
+void local_data_clone(dbref, dbref);
+void local_data_free(dbref);
+
+// From mail.cpp.
+//
+void load_mail(FILE *);
+int  dump_mail(FILE *);
+struct mail *mail_fetch(dbref, int);
+
+// From netcommon.cpp.
+//
+void DCL_CDECL raw_broadcast(int, char *, ...);
+void list_siteinfo(dbref);
+void logged_out0(dbref executor, dbref caller, dbref enactor, int key);
+void logged_out1(dbref executor, dbref caller, dbref enactor, int key, char *arg);
+void init_logout_cmdtab(void);
+void desc_reload(dbref);
+void make_portlist(dbref, dbref, char *, char **);
 
 /* From cque.cpp */
-extern int  nfy_que(dbref, int, int, int);
-extern int  halt_que(dbref, dbref);
-extern void wait_que(dbref executor, dbref caller, dbref enactor, bool,
+int  nfy_que(dbref, int, int, int);
+int  halt_que(dbref, dbref);
+void wait_que(dbref executor, dbref caller, dbref enactor, bool,
     CLinearTimeAbsolute&, dbref, int, char *, char *[],int, char *[]);
 
 #ifdef WIN32
@@ -38,6 +96,7 @@ extern void wait_que(dbref executor, dbref caller, dbref enactor, bool,
 #else // WIN32
 extern "C" char *crypt(const char *inptr, const char *inkey);
 #endif // WIN32
+extern bool break_called;
 
 /* From eval.cpp */
 void tcache_init(void);
@@ -48,14 +107,15 @@ int get_gender(dbref);
 void mux_exec(char *buff, char **bufc, dbref executor, dbref caller,
               dbref enactor, int eval, char **dstr, char *cargs[],
               int ncargs);
-extern void save_global_regs(const char *, char *[], int []);
-extern void save_and_clear_global_regs(const char *, char *[], int[]);
-extern void restore_global_regs(const char *, char *[], int []);
-extern char **PushPointers(int nNeeded);
-extern void PopPointers(char **p, int nNeeded);
-extern int *PushIntegers(int nNeeded);
-extern void PopIntegers(int *pi, int nNeeded);
+void save_global_regs(const char *, char *[], int []);
+void save_and_clear_global_regs(const char *, char *[], int[]);
+void restore_global_regs(const char *, char *[], int []);
+char **PushPointers(int nNeeded);
+void PopPointers(char **p, int nNeeded);
+int *PushIntegers(int nNeeded);
+void PopIntegers(int *pi, int nNeeded);
 extern const signed char mux_RegisterSet[256];
+extern const char *ColorTable[256];
 
 /* From game.cpp */
 #define notify(p,m)                         notify_check(p,p,m, MSG_PUP_ALWAYS|MSG_ME_ALL|MSG_F_DOWN)
@@ -74,16 +134,43 @@ extern const signed char mux_RegisterSet[256];
 #define notify_all_from_inside_html(p,c,m)  notify_check(p,c,m, MSG_ME_ALL|MSG_NBR_EXITS_A|MSG_F_UP|MSG_F_CONTENTS|MSG_S_INSIDE|MSG_HTML)
 //#define notify_all_from_outside(p,c,m)      notify_check(p,c,m, MSG_ME_ALL|MSG_NBR_EXITS|MSG_F_UP|MSG_F_CONTENTS|MSG_S_OUTSIDE)
 
-extern void notify_except(dbref, dbref, dbref, const char *, int key);
-extern void notify_except2(dbref, dbref, dbref, dbref, const char *);
+void notify_except(dbref, dbref, dbref, const char *, int key);
+void notify_except2(dbref, dbref, dbref, dbref, const char *);
 
-extern void notify_check(dbref, dbref, const char *, int);
+void notify_check(dbref, dbref, const char *, int);
 
-extern bool Hearer(dbref);
-extern void report(void);
-extern bool atr_match(dbref, dbref, char, char *, bool);
-extern bool list_check(dbref, dbref, char, char *, bool);
-extern bool html_escape(const char *src, char *dest, char **destp);
+bool Hearer(dbref);
+void report(void);
+
+bool atr_match
+(
+    dbref thing,
+    dbref player,
+    char  type,
+    char  *str,
+    char  *raw_str,
+    bool  check_parents
+);
+
+bool regexp_match
+(
+    char *pattern,
+    char *str,
+    int case_opt,
+    char *args[],
+    int nargs
+);
+
+bool list_check
+(
+    dbref thing,
+    dbref player,
+    char  type,
+    char  *str,
+    char  *raw_str,
+    bool  check_parent
+);
+bool html_escape(const char *src, char *dest, char **destp);
 
 #define DUMP_I_NORMAL    0  // OUTPUT to the outdb through a temporary file.
 #define DUMP_I_PANIC     1  // UNLOAD to a crashdb
@@ -91,15 +178,17 @@ extern bool html_escape(const char *src, char *dest, char **destp);
 #define DUMP_I_FLAT      3  // UNLOAD to a .FLAT file
 #define DUMP_I_SIGNAL    4  // UNLOAD to a .FLAT file from signal.
 #define NUM_DUMP_TYPES   5
-extern void dump_database_internal(int);
+void dump_database_internal(int);
+void fork_and_dump(int key);
 
 /* From help.cpp */
-extern void helpindex_clean(int);
-extern void helpindex_load(dbref);
-extern void helpindex_init(void);
+void helpindex_clean(int);
+void helpindex_load(dbref);
+void helpindex_init(void);
+void help_helper(dbref executor, int iHelpfile, char *topic_arg, char *buff, char **bufc);
 
 /* From htab.cpp */
-extern int  cf_ntab_access(int *, char *, void *, UINT32, dbref, char *);
+int  cf_ntab_access(int *, char *, void *, UINT32, dbref, char *);
 
 /* From log.cpp */
 #ifdef WIN32
@@ -107,106 +196,107 @@ extern int  cf_ntab_access(int *, char *, void *, UINT32, dbref, char *);
 #else // WIN32
 #define ENDLINE "\n"
 #endif // WIN32
-extern bool start_log(const char *primary, const char *secondary);
-extern void end_log(void);
-extern void log_perror(const char *, const char *,const char *,
+bool start_log(const char *primary, const char *secondary);
+void end_log(void);
+void log_perror(const char *, const char *,const char *,
             const char *);
-extern void log_text(const char *);
-extern void log_number(int);
-extern void log_name(dbref);
-extern void log_name_and_loc(dbref);
-extern void log_type_and_name(dbref);
+void log_text(const char *);
+void log_number(int);
+void DCL_CDECL log_printf(const char *fmt, ...);
+void log_name(dbref);
+void log_name_and_loc(dbref);
+void log_type_and_name(dbref);
 
 /* From look.cpp */
-extern void look_in(dbref,dbref, int);
-extern void show_vrml_url(dbref, dbref);
+void look_in(dbref,dbref, int);
+void show_vrml_url(dbref, dbref);
 size_t decode_attr_flags(int aflags, char *buff);
 
 /* From move.cpp */
-extern void move_object(dbref, dbref);
-extern void move_via_generic(dbref, dbref, dbref, int);
-extern bool move_via_teleport(dbref, dbref, dbref, int);
-extern void move_exit(dbref, dbref, bool, const char *, int);
-extern void do_enter_internal(dbref, dbref, bool);
+void move_object(dbref, dbref);
+void move_via_generic(dbref, dbref, dbref, int);
+bool move_via_teleport(dbref, dbref, dbref, int);
+void move_exit(dbref, dbref, bool, const char *, int);
+void do_enter_internal(dbref, dbref, bool);
 
 /* From object.cpp */
-extern dbref start_home(void);
-extern dbref default_home(void);
-extern bool  can_set_home(dbref, dbref, dbref);
-extern dbref new_home(dbref);
-extern dbref clone_home(dbref, dbref);
-extern void  divest_object(dbref);
-extern dbref create_obj(dbref, int, const char *, int);
-extern void  destroy_obj(dbref);
-extern void  empty_obj(dbref);
+dbref start_home(void);
+dbref default_home(void);
+bool  can_set_home(dbref, dbref, dbref);
+dbref new_home(dbref);
+dbref clone_home(dbref, dbref);
+void  divest_object(dbref);
+dbref create_obj(dbref, int, const char *, int);
+void  destroy_obj(dbref);
+void  empty_obj(dbref);
 
 /* From player.cpp */
-extern dbref create_player(char *name, char *pass, dbref executor, bool isrobot, const char **pmsg);
-extern void AddToPublicChannel(dbref player);
-extern bool add_player_name(dbref, const char *);
-extern bool delete_player_name(dbref, const char *);
-extern dbref lookup_player(dbref, char *, bool);
-extern void load_player_names(void);
-extern void badname_add(char *);
-extern void badname_remove(char *);
-extern bool badname_check(char *);
-extern void badname_list(dbref, const char *);
-extern void ChangePassword(dbref player, const char *szPassword);
-extern const char *mux_crypt(const char *szPassword, const char *szSalt, int *piType);
+dbref create_player(char *name, char *pass, dbref executor, bool isrobot, const char **pmsg);
+void AddToPublicChannel(dbref player);
+bool add_player_name(dbref, const char *);
+bool delete_player_name(dbref, const char *);
+dbref lookup_player(dbref, char *, bool);
+void load_player_names(void);
+void badname_add(char *);
+void badname_remove(char *);
+bool badname_check(char *);
+void badname_list(dbref, const char *);
+void ChangePassword(dbref player, const char *szPassword);
+const char *mux_crypt(const char *szPassword, const char *szSalt, int *piType);
+int  QueueMax(dbref);
+int  a_Queue(dbref, int);
+void pcache_reload(dbref);
+void pcache_init(void);
 
 /* From predicates.cpp */
-extern char * DCL_CDECL tprintf(const char *, ...);
-extern void DCL_CDECL safe_tprintf_str(char *, char **, const char *, ...);
-extern dbref insert_first(dbref, dbref);
-extern dbref remove_first(dbref, dbref);
-extern dbref reverse_list(dbref);
-extern bool member(dbref, dbref);
-extern bool could_doit(dbref, dbref, int);
-extern bool can_see(dbref, dbref, bool);
-extern void add_quota(dbref, int);
-extern bool canpayfees(dbref, dbref, int, int);
-extern void giveto(dbref,int);
-extern bool payfor(dbref,int);
-extern char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid);
-extern char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid);
-extern bool ValidatePlayerName(const char *pName);
-extern bool ok_password(const char *szPassword, const char **pmsg);
-extern void handle_ears(dbref, bool, bool);
-extern dbref match_possessed(dbref, dbref, char *, dbref, bool);
-extern void parse_range(char **, dbref *, dbref *);
-extern bool parse_thing_slash(dbref, char *, char **, dbref *);
-extern bool get_obj_and_lock(dbref, char *, dbref *, ATTR **, char *, char **);
-extern dbref where_is(dbref);
-extern dbref where_room(dbref);
-extern bool locatable(dbref, dbref, dbref);
-extern bool nearby(dbref, dbref);
-extern bool exit_visible(dbref, dbref, int);
-extern bool exit_displayable(dbref, dbref, int);
-extern void did_it(dbref, dbref, int, const char *, int, const char *, int, char *[], int);
-extern bool bCanReadAttr(dbref executor, dbref target, ATTR *tattr, bool bParentCheck);
-extern bool bCanSetAttr(dbref executor, dbref target, ATTR *tattr);
-extern bool bCanLockAttr(dbref executor, dbref target, ATTR *tattr);
+char * DCL_CDECL tprintf(const char *, ...);
+void DCL_CDECL safe_tprintf_str(char *, char **, const char *, ...);
+dbref insert_first(dbref, dbref);
+dbref remove_first(dbref, dbref);
+dbref reverse_list(dbref);
+bool member(dbref, dbref);
+bool could_doit(dbref, dbref, int);
+bool can_see(dbref, dbref, bool);
+void add_quota(dbref, int);
+bool canpayfees(dbref, dbref, int, int);
+void giveto(dbref,int);
+bool payfor(dbref,int);
+char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid);
+char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid);
+bool ValidatePlayerName(const char *pName);
+bool ok_password(const char *szPassword, const char **pmsg);
+void handle_ears(dbref, bool, bool);
+dbref match_possessed(dbref, dbref, char *, dbref, bool);
+void parse_range(char **, dbref *, dbref *);
+bool parse_thing_slash(dbref, char *, char **, dbref *);
+bool get_obj_and_lock(dbref, char *, dbref *, ATTR **, char *, char **);
+dbref where_is(dbref);
+dbref where_room(dbref);
+bool locatable(dbref, dbref, dbref);
+bool nearby(dbref, dbref);
+bool exit_visible(dbref, dbref, int);
+bool exit_displayable(dbref, dbref, int);
+void did_it(dbref, dbref, int, const char *, int, const char *, int, char *[], int);
+bool bCanReadAttr(dbref executor, dbref target, ATTR *tattr, bool bParentCheck);
+bool bCanSetAttr(dbref executor, dbref target, ATTR *tattr);
+bool bCanLockAttr(dbref executor, dbref target, ATTR *tattr);
 
 /* From set.cpp */
-extern bool parse_attrib(dbref, char *, dbref *, ATTR **);
-extern bool parse_attrib_wild(dbref, char *, dbref *, bool, bool, bool);
-extern void edit_string(char *, char **, char *, char *);
-extern dbref match_controlled_handler(dbref player, const char *name, bool bQuiet);
+bool parse_attrib(dbref, char *, dbref *, ATTR **);
+bool parse_attrib_wild(dbref, char *, dbref *, bool, bool, bool);
+void edit_string(char *, char **, char *, char *);
+dbref match_controlled_handler(dbref player, const char *name, bool bQuiet);
 #define match_controlled(player,name)       match_controlled_handler(player, name, false)
 #define match_controlled_quiet(player,name) match_controlled_handler(player, name, true)
-extern void set_modified(dbref thing);
+void set_modified(dbref thing);
 
 /* From boolexp.cpp */
-extern bool eval_boolexp(dbref, dbref, dbref, BOOLEXP *);
-extern BOOLEXP *parse_boolexp(dbref, const char *, bool);
-extern bool eval_boolexp_atr(dbref, dbref, dbref, char *);
+bool eval_boolexp(dbref, dbref, dbref, BOOLEXP *);
+BOOLEXP *parse_boolexp(dbref, const char *, bool);
+bool eval_boolexp_atr(dbref, dbref, dbref, char *);
 
 /* From functions.cpp */
-extern bool xlate(char *);
-extern char *trim_space_sep(char *, char);
-extern char *trim_space_sep_LEN(char *str, int nStr, char sep, int *nTrim);
-extern char *next_token(char *str, char sep);
-extern char *split_token(char **sp, char sep);
+bool xlate(char *);
 
 #define IEEE_MAKE_NAN  1
 #define IEEE_MAKE_IND  2
@@ -216,28 +306,28 @@ extern char *split_token(char **sp, char sep);
 double MakeSpecialFloat(int iWhich);
 
 /* From unparse.cpp */
-extern char *unparse_boolexp(dbref, BOOLEXP *);
-extern char *unparse_boolexp_quiet(dbref, BOOLEXP *);
-extern char *unparse_boolexp_decompile(dbref, BOOLEXP *);
-extern char *unparse_boolexp_function(dbref, BOOLEXP *);
+char *unparse_boolexp(dbref, BOOLEXP *);
+char *unparse_boolexp_quiet(dbref, BOOLEXP *);
+char *unparse_boolexp_decompile(dbref, BOOLEXP *);
+char *unparse_boolexp_function(dbref, BOOLEXP *);
 
 /* From walkdb.cpp */
 int chown_all(dbref from_player, dbref to_player, dbref acting_player, int key);
-extern void olist_push(void);
-extern void olist_pop(void);
-extern void olist_add(dbref);
-extern dbref olist_first(void);
-extern dbref olist_next(void);
+void olist_push(void);
+void olist_pop(void);
+void olist_add(dbref);
+dbref olist_first(void);
+dbref olist_next(void);
 
 /* From wild.cpp */
-extern bool wild(char *, char *, char *[], int);
-extern bool wild_match(char *, const char *);
-extern bool quick_wild(const char *, const char *);
+bool wild(char *, char *, char *[], int);
+bool wild_match(char *, const char *);
+bool quick_wild(const char *, const char *);
 
 /* From command.cpp */
-extern bool check_access(dbref player, int mask);
-extern void set_prefix_cmds(void);
-extern char *process_command(dbref executor, dbref caller, dbref enactor, bool,
+bool check_access(dbref player, int mask);
+void set_prefix_cmds(void);
+char *process_command(dbref executor, dbref caller, dbref enactor, bool,
     char *, char *[], int);
 
 #define Protect(f) (cmdp->perms & f)
@@ -248,49 +338,51 @@ extern char *process_command(dbref executor, dbref caller, dbref enactor, bool,
  (Protect(CA_PLAYER) && !isPlayer(x)))
 
 /* from db.cpp */
-extern bool Commer(dbref);
-extern void s_Pass(dbref, const char *);
-extern void s_Name(dbref, const char *);
-extern void s_Moniker(dbref thing, const char *s);
-extern const char *Name(dbref thing);
-extern const char *PureName(dbref thing);
-extern const char *Moniker(dbref thing);
-extern int  fwdlist_load(FWDLIST *, dbref, char *);
-extern void fwdlist_set(dbref, FWDLIST *);
-extern void fwdlist_clr(dbref);
-extern int  fwdlist_rewrite(FWDLIST *, char *);
-extern FWDLIST *fwdlist_get(dbref);
-extern void atr_push(void);
-extern void atr_pop(void);
-extern int  atr_head(dbref, char **);
-extern int  atr_next(char **);
-extern int  init_dbfile(char *game_dir_file, char *game_pag_file, int nCachePages);
-extern void atr_cpy(dbref dest, dbref source);
-extern void atr_chown(dbref);
-extern void atr_clr(dbref, int);
-extern void atr_add_raw_LEN(dbref, int, const char *, int);
-extern void atr_add_raw(dbref, int, const char *);
-extern void atr_add(dbref, int, char *, dbref, int);
-extern void atr_set_flags(dbref, int, int);
-extern const char *atr_get_raw_LEN(dbref, int, size_t *);
-extern const char *atr_get_raw(dbref, int);
-extern char *atr_get_LEN(dbref, int, dbref *, int *, size_t *);
-extern char *atr_get_real(dbref, int, dbref *, int *, const char *, const int);
+bool Commer(dbref);
+void s_Pass(dbref, const char *);
+void s_Name(dbref, const char *);
+void s_Moniker(dbref thing, const char *s);
+const char *Name(dbref thing);
+const char *PureName(dbref thing);
+const char *Moniker(dbref thing);
+int  fwdlist_load(FWDLIST *, dbref, char *);
+void fwdlist_set(dbref, FWDLIST *);
+void fwdlist_clr(dbref);
+int  fwdlist_rewrite(FWDLIST *, char *);
+FWDLIST *fwdlist_get(dbref);
+void atr_push(void);
+void atr_pop(void);
+int  atr_head(dbref, char **);
+int  atr_next(char **);
+int  init_dbfile(char *game_dir_file, char *game_pag_file, int nCachePages);
+void atr_cpy(dbref dest, dbref source, bool bInternal);
+void atr_chown(dbref);
+void atr_clr(dbref, int);
+void atr_add_raw_LEN(dbref, int, const char *, int);
+void atr_add_raw(dbref, int, const char *);
+void atr_add(dbref, int, char *, dbref, int);
+void atr_set_flags(dbref, int, int);
+const char *atr_get_raw_LEN(dbref, int, size_t *);
+const char *atr_get_raw(dbref, int);
+char *atr_get_LEN(dbref, int, dbref *, int *, size_t *);
+char *atr_get_real(dbref, int, dbref *, int *, const char *, const int);
 #define atr_get(t,a,o,f) atr_get_real(t,a,o,f, __FILE__, __LINE__)
-extern char *atr_pget_LEN(dbref, int, dbref *, int *, size_t *);
-extern char *atr_pget_real(dbref, int, dbref *, int *, const char *, const int);
+char *atr_pget_LEN(dbref, int, dbref *, int *, size_t *);
+char *atr_pget_real(dbref, int, dbref *, int *, const char *, const int);
 #define atr_pget(t,a,o,f) atr_pget_real(t,a,o,f, __FILE__, __LINE__)
-extern char *atr_get_str_LEN(char *s, dbref, int, dbref *, int *, size_t *);
-extern char *atr_get_str(char *, dbref, int, dbref *, int *);
-extern char *atr_pget_str_LEN(char *, dbref, int, dbref *, int *, size_t *);
-extern char *atr_pget_str(char *, dbref, int, dbref *, int *);
-extern bool atr_get_info(dbref, int, dbref *, int *);
-extern bool atr_pget_info(dbref, int, dbref *, int *);
-extern void atr_free(dbref);
-extern bool check_zone_handler(dbref player, dbref thing, bool bPlayerCheck);
+char *atr_get_str_LEN(char *s, dbref, int, dbref *, int *, size_t *);
+char *atr_get_str(char *, dbref, int, dbref *, int *);
+char *atr_pget_str_LEN(char *, dbref, int, dbref *, int *, size_t *);
+char *atr_pget_str(char *, dbref, int, dbref *, int *);
+bool atr_get_info(dbref, int, dbref *, int *);
+bool atr_pget_info(dbref, int, dbref *, int *);
+void atr_free(dbref);
+bool check_zone_handler(dbref player, dbref thing, bool bPlayerCheck);
 #define check_zone(player, thing) check_zone_handler(player, thing, false)
-extern void ReleaseAllResources(dbref obj);
-extern bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext);
+void ReleaseAllResources(dbref obj);
+bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext);
+
+extern int anum_alc_top;
 
 /* Command handler keys */
 
@@ -314,7 +406,7 @@ extern bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext);
 #define CLONE_SET_COST  8   /* ARG2 is cost of cloned object */
 #define CLONE_SET_LOC   16  /* ARG2 is location of cloned object */
 #define CLONE_SET_NAME  32  /* ARG2 is alternate name of cloned object */
-#define CLONE_PARENT    64  /* Set parent on obj instd of cloning attrs */
+#define CLONE_FROM_PARENT 64 /* Set parent on obj instd of cloning attrs */
 #define CBOOT_QUIET     1   // Cboot without a message
 #define COMTITLE_ON     1   // Turn Comtitles on.
 #define COMTITLE_OFF    2   // Turn Comtitles off.
@@ -473,6 +565,7 @@ extern bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext);
 #define PS_LONG         1   /* Long PS report */
 #define PS_SUMM         2   /* Queue counts only */
 #define PS_ALL          4   /* List entire queue */
+#define QUERY_SQL       1   /* SQL query */
 #define QUEUE_KICK      1   /* Process commands from queue */
 #define QUEUE_WARP      2   /* Advance or set back wait queue clock */
 #define QUOTA_SET       1   /* Set a quota */
@@ -532,6 +625,7 @@ extern bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext);
 #define SWEEP_VERBOSE   256 /* Display what pattern matches */
 #define TELEPORT_DEFAULT 1  /* Emit all messages */
 #define TELEPORT_QUIET  2   /* Teleport in quietly */
+#define TELEPORT_LIST   4   /* Teleport a list of items */
 #define TIMECHK_RESET   1   /* Reset all counters to zero */
 #define TIMECHK_SCREEN  2   /* Write info to screen */
 #define TIMECHK_LOG     4   /* Write info to log */
@@ -648,10 +742,10 @@ extern const char *FUNC_NOPERM_MESSAGE;
 #define safe_ambiguous(b,p) safe_copy_buf(FUNC_AMBIGUOUS,13,(b),(p))
 #define safe_notfound(b,p)  safe_copy_buf(FUNC_NOT_FOUND,13,(b),(p))
 
-extern int  ReplaceFile(char *old_name, char *new_name);
-extern void RemoveFile(char *name);
-extern void destroy_player(dbref agent, dbref victim);
-extern void do_pemit_list
+int  ReplaceFile(char *old_name, char *new_name);
+void RemoveFile(char *name);
+void destroy_player(dbref agent, dbref victim);
+void do_pemit_list
 (
     dbref player,
     int key,
@@ -661,7 +755,7 @@ extern void do_pemit_list
     int chPoseType,
     char *message
 );
-extern void do_pemit_single
+void do_pemit_single
 (
     dbref player,
     int key,
@@ -671,24 +765,30 @@ extern void do_pemit_single
     int chPoseType,
     char *message
 );
-extern void do_say(dbref executor, dbref caller, dbref enactor, int key,
+void do_say(dbref executor, dbref caller, dbref enactor, int key,
                    char *message);
 
-extern int  boot_off(dbref player, const char *message);
-extern void do_mail_clear(dbref player, char *msglist);
-extern void do_mail_purge(dbref player);
-extern void raw_notify_html(dbref player, const char *msg);
-extern void do_lock(dbref executor, dbref caller, dbref enactor, int key,
+int  boot_off(dbref player, const char *message);
+void do_mail_clear(dbref player, char *msglist);
+void do_mail_purge(dbref player);
+void malias_cleanup(dbref player);
+void count_mail(dbref player, int folder, int *rcount, int *ucount, int *ccount);
+void check_mail_expiration(void);
+void check_mail(dbref player, int folder, bool silent);
+const char *mail_fetch_message(dbref player, int num);
+int  mail_fetch_from(dbref player, int num);
+void raw_notify_html(dbref player, const char *msg);
+void do_lock(dbref executor, dbref caller, dbref enactor, int key,
                     int nargs, char *name, char *keytext);
-extern void check_events(void);
-extern void list_system_resources(dbref player);
+void check_events(void);
+void list_system_resources(dbref player);
 
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
 
 #define ACTION_IS_STATIONARY    0
 #define ACTION_IS_MOVING        1
 #define ACTION_IS_TALKING       2
-#define NUMBER_OF_ACTIONS       3  // A count, n, of the number of possible actions 0...n
+#define NUMBER_OF_ACTIONS       3  // A count, n, of the number of possible actions 0...n-1
 
 #define REALM_DO_NORMALLY_SEEN        1
 #define REALM_DO_HIDDEN_FROM_YOU      2
@@ -698,9 +798,8 @@ extern void list_system_resources(dbref player);
 #define REALM_DO_SHOW_MATRIXDESC      6
 #define REALM_DO_SHOW_FAEDESC         7
 #define REALM_DO_MASK                 7
-
 #define REALM_DISABLE_ADESC           0x00000008L
-extern int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state);
+int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state);
 #endif // WOD_REALMS
 
 typedef struct
@@ -735,16 +834,25 @@ typedef BOOL __stdcall FGETPROCESSTIMES(HANDLE hProcess,
     LPFILETIME pftUser);
 extern FCANCELIO *fpCancelIo;
 extern FGETPROCESSTIMES *fpGetProcessTimes;
-extern bool bQueryPerformanceAvailable;
-extern INT64 QP_A;
-extern INT64 QP_B;
-extern INT64 QP_C;
-extern INT64 QP_D;
 #else // WIN32
 extern pid_t game_pid;
 #endif // WIN32
 
-extern void init_timer(void);
+// From timer.cpp
+//
+void init_timer(void);
+#ifdef WIN32
+void Task_FreeDescriptor(void *arg_voidptr, int arg_Integer);
+void Task_DeferredClose(void *arg_voidptr, int arg_Integer);
+#endif
+void dispatch_DatabaseDump(void *pUnused, int iUnused);
+void dispatch_FreeListReconstruction(void *pUnused, int iUnused);
+void dispatch_IdleCheck(void *pUnused, int iUnused);
+void dispatch_CheckEvents(void *pUnused, int iUnused);
+#ifndef MEMORY_BASED
+void dispatch_CacheTick(void *pUnused, int iUnused);
+#endif
+
 
 // Using a heap as the data structure for representing this priority
 // has some attributes which we depend on:
@@ -844,7 +952,7 @@ public:
 
 extern CScheduler scheduler;
 
-extern int fetch_cmds(dbref target);
+int fetch_cmds(dbref target);
 void fetch_ConnectionInfoFields(dbref target, long anFields[4]);
 long fetch_ConnectionInfoField(dbref target, int iField);
 void put_ConnectionInfoFields
@@ -876,10 +984,42 @@ CLinearTimeAbsolute fetch_logouttime(dbref target);
 
 // From strtod.cpp
 //
-extern void FLOAT_Initialize(void);
+void FLOAT_Initialize(void);
+void mux_FPInit();
+void mux_FPSet();
+void mux_FPRestore();
 double ulp(double);
 double mux_strtod(const char *s00, char **se);
 char *mux_dtoa(double d, int mode, int ndigits, int *decpt, int *sign,
              char **rve);
 
+// From wiz.cpp
+//
+extern NAMETAB enable_names[];
+
+// From version.cpp
+//
+void build_version(void);
+void init_version(void);
+
+// From player_c.cpp
+//
+void pcache_sync(void);
+void pcache_trim(void);
+
+// From attrcache.cpp
+//
+void cache_redirect(void);
+void cache_pass2(void);
+extern CLinearTimeAbsolute cs_ltime;
+
+// From speech.cpp
+//
+char *modSpeech(dbref player, char *message, bool bWhich, char *command);
+
+// From funceval.cpp
+//
+void stack_clr(dbref obj);
+bool parse_and_get_attrib(dbref, char *[], char **, dbref *, char *, char **);
+
 #endif // EXTERNS_H
