@@ -1,6 +1,6 @@
 // flags.cpp -- Flag manipulation routines.
 //
-// $Id: flags.cpp,v 1.16 2004/04/01 22:00:42 sdennis Exp $
+// $Id: flags.cpp,v 1.24 2006/01/07 23:01:36 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -15,7 +15,7 @@
  * fh_any: set or clear indicated bit, no security checking
  */
 
-bool fh_any(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_any(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     // Never let God drop his/her own wizbit.
     //
@@ -45,7 +45,7 @@ bool fh_any(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * fh_god: only GOD may set or clear the bit
  */
 
-bool fh_god(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_god(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (!God(player))
     {
@@ -59,7 +59,7 @@ bool fh_god(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * fh_wiz: only WIZARDS (or GOD) may set or clear the bit
  */
 
-bool fh_wiz(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_wiz(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (!Wizard(player))
     {
@@ -73,7 +73,7 @@ bool fh_wiz(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * fh_wizroy: only WIZARDS, ROYALTY, (or GOD) may set or clear the bit
  */
 
-bool fh_wizroy(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_wizroy(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (!WizRoy(player))
     {
@@ -88,7 +88,7 @@ bool fh_wizroy(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * this on players, but ordinary players can set it on other types
  * * of objects.
  */
-bool fh_restrict_player
+static bool fh_restrict_player
 (
     dbref target,
     dbref player,
@@ -102,7 +102,7 @@ bool fh_restrict_player
     {
         return false;
     }
-    return fh_any(target, player, flag, fflags, reset);
+    return (fh_any(target, player, flag, fflags, reset));
 }
 
 /* ---------------------------------------------------------------------------
@@ -110,7 +110,7 @@ bool fh_restrict_player
  * yourself have this flag and are a player who owns themselves (i.e.,
  * no robots). Only God can set this on a player.
  */
-bool fh_privileged
+static bool fh_privileged
 (
     dbref target,
     dbref player,
@@ -129,7 +129,7 @@ bool fh_privileged
             return false;
         }
     }
-    return fh_any(target, player, flag, fflags, reset);
+    return (fh_any(target, player, flag, fflags, reset));
 }
 
 /*
@@ -137,7 +137,7 @@ bool fh_privileged
  * * fh_inherit: only players may set or clear this bit.
  */
 
-bool fh_inherit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_inherit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (!Inherits(player))
     {
@@ -151,7 +151,7 @@ bool fh_inherit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * fh_dark_bit: manipulate the dark bit. Nonwizards may not set on players.
  */
 
-bool fh_dark_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_dark_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (  !reset
        && isPlayer(target)
@@ -161,7 +161,7 @@ bool fh_dark_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
     {
         return false;
     }
-    return fh_any(target, player, flag, fflags, reset);
+    return (fh_any(target, player, flag, fflags, reset));
 }
 
 /*
@@ -169,14 +169,14 @@ bool fh_dark_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * fh_going_bit: manipulate the going bit.  Non-gods may only clear.
  */
 
-bool fh_going_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_going_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (  Going(target)
        && reset
        && (Typeof(target) != TYPE_GARBAGE))
     {
         notify(player, "Your object has been spared from destruction.");
-        return fh_any(target, player, flag, fflags, reset);
+        return (fh_any(target, player, flag, fflags, reset));
     }
     if (!God(player))
     {
@@ -195,7 +195,7 @@ bool fh_going_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
     {
         return false;
     }
-    return fh_any(target, player, flag, fflags, reset);
+    return (fh_any(target, player, flag, fflags, reset));
 }
 
 /*
@@ -203,13 +203,13 @@ bool fh_going_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
  * * fh_hear_bit: set or clear bits that affect hearing.
  */
 
-bool fh_hear_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
+static bool fh_hear_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
 {
     if (isPlayer(target) && (flag & MONITOR))
     {
         if (Can_Monitor(player))
         {
-            fh_any(target, player, flag, fflags, reset);
+            return (fh_any(target, player, flag, fflags, reset));
         }
         else
         {
@@ -218,15 +218,16 @@ bool fh_hear_bit(dbref target, dbref player, FLAG flag, int fflags, bool reset)
     }
 
     bool could_hear = Hearer(target);
-    fh_any(target, player, flag, fflags, reset);
+    bool result = fh_any(target, player, flag, fflags, reset);
     handle_ears(target, could_hear, Hearer(target));
-    return true;
+    return result;
 }
 
+
 /* ---------------------------------------------------------------------------
  * fh_player_bit: Can set and reset this on everything but players.
  */
-bool fh_player_bit
+static bool fh_player_bit
 (
     dbref target,
     dbref player,
@@ -239,14 +240,14 @@ bool fh_player_bit
     {
         return false;
     }
-    return fh_any(target, player, flag, fflags, reset);
+    return (fh_any(target, player, flag, fflags, reset));
 }
 
 /* ---------------------------------------------------------------------------
  * fh_staff: only STAFF, WIZARDS, ROYALTY, (or GOD) may set or clear
  * the bit.
  */
-bool fh_staff
+static bool fh_staff
 (
     dbref target,
     dbref player,
@@ -467,7 +468,10 @@ void init_flagtab(void)
         strncpy(nbuf, fp->pOrigName, SBUF_SIZE);
         nbuf[SBUF_SIZE-1] = '\0';
         mux_strlwr(nbuf);
-        hashaddLEN(nbuf, strlen(nbuf), fp, &mudstate.flags_htab);
+        if (!hashfindLEN(nbuf, strlen(nbuf), &mudstate.flags_htab))
+        {
+            hashaddLEN(nbuf, strlen(nbuf), fp, &mudstate.flags_htab);
+        }
     }
     free_sbuf(nbuf);
 }
@@ -544,7 +548,7 @@ char *MakeCanonicalFlagName
     }
 }
 
-FLAGNAMEENT *find_flag(char *flagname)
+static FLAGNAMEENT *find_flag(char *flagname)
 {
     // Convert flagname to canonical lowercase format.
     //
@@ -915,6 +919,10 @@ char *unparse_object(dbref player, dbref target, bool obey_myopic)
 
 CF_HAND(cf_flag_access)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, str);
     mux_strtok_ctl(&tts, " \t=,");
@@ -996,11 +1004,10 @@ bool convert_flags(dbref player, char *flaglist, FLAGSET *fset, FLAG *p_type)
 {
     FLAG type = NOTYPE;
     FLAGSET flagmask;
+    flagmask.word[FLAG_WORD1] = 0;
+    flagmask.word[FLAG_WORD2] = 0;
+    flagmask.word[FLAG_WORD3] = 0;
     int i;
-    for (i = FLAG_WORD1; i <= FLAG_WORD3; i++)
-    {
-        flagmask.word[i] = 0;
-    }
 
     char *s;
     bool handled;
@@ -1118,7 +1125,7 @@ void decompile_flags(dbref player, dbref thing, char *thingname)
 // do_flag: Rename flags or remove flag aliases.
 // Based on RhostMUSH code.
 //
-bool flag_rename(char *alias, char *newname)
+static bool flag_rename(char *alias, char *newname)
 {
     int nAlias;
     bool bValidAlias;
@@ -1171,6 +1178,9 @@ bool flag_rename(char *alias, char *newname)
 void do_flag(dbref executor, dbref caller, dbref enactor, int key, int nargs,
              char *flag1, char *flag2)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     if (key & FLAG_REMOVE)
     {
         if (nargs == 2)
@@ -1225,13 +1235,19 @@ void do_flag(dbref executor, dbref caller, dbref enactor, int key, int nargs,
 
 CF_HAND(cf_flag_name)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, str);
     mux_strtok_ctl(&tts, " \t=,");
     char *flagstr = mux_strtok_parse(&tts);
     char *namestr = mux_strtok_parse(&tts);
 
-    if (  !flagstr 
+    if (  !flagstr
        || !*flagstr
        || !namestr
        || !*namestr)
