@@ -1,6 +1,6 @@
 // flags.h -- Object flags.
 //
-// $Id: flags.h,v 1.4 2003/02/05 06:20:58 jake Exp $
+// $Id: flags.h,v 1.10 2006/01/07 21:48:00 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -85,7 +85,7 @@
 
 // Third word of flags
 //
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
 #define OBF          0x00000001      // Obfuscate Flag
 #define HSS          0x00000002      // Auspex/Heightened Senses Flag
 #define UMBRA        0x00000004      // Umbra, UMBRADESC
@@ -98,6 +98,7 @@
 #define PEERING      0x00000200      // Means the a looker is seeing a
                                      // different realm than they are in.
 #endif // WOD_REALMS
+
 #define SITEMON      0x00000400      // Sitemonitor Flag
 #define CMDCHECK     0x00000800      // Has @icmd set
 #define MARK_0       0x00400000      // User-defined flags.
@@ -120,7 +121,7 @@ typedef struct flag_bit_entry
     char flaglett;          // Flag letter for listing.
     int  flagflag;          // Ctrl flags for this flag.
     int  listperm;          // Who sees this flag when set.
-    bool (*handler)(dbref target, dbref player, FLAG flag, int fflags, 
+    bool (*handler)(dbref target, dbref player, FLAG flag, int fflags,
         bool reset);         // Handler for setting/clearing this flag.
 } FLAGBITENT;
 
@@ -132,6 +133,8 @@ typedef struct flag_name_entry
     char *flagname;         // Name of the flag.
 } FLAGNAMEENT;
 
+extern FLAGNAMEENT gen_flag_names[];
+
 /* ---------------------------------------------------------------------------
  * OBJENT: Fundamental object types
  */
@@ -316,10 +319,9 @@ extern char *MakeCanonicalFlagName
 #define s_Going(x)          s_Flags((x), FLAG_WORD1, Flags(x) | GOING)
 #define s_Connected(x)      s_Flags((x), FLAG_WORD2, Flags2(x) | CONNECTED)
 #define c_Connected(x)      s_Flags((x), FLAG_WORD2, Flags2(x) & ~CONNECTED)
-
 #define SiteMon(x)          ((Flags3(x) & SITEMON) != 0)
 #define CmdCheck(x)         ((Flags3(x) & CMDCHECK) != 0)
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
 #define isObfuscate(x)        ((Flags3(x) & OBF) != 0)
 #define isHeightenedSenses(x) ((Flags3(x) & HSS) != 0)
 #define isUmbra(x)            ((Flags3(x) & UMBRA) != 0)
@@ -332,6 +334,7 @@ extern char *MakeCanonicalFlagName
 #define isPeering(x)          ((Flags3(x) & PEERING) != 0)
 #endif // WOD_REALMS
 
+
 #define Parentable(p,x)     (Controls(p,x) || \
                             (Parent_ok(x) && could_doit(p,x,A_LPARENT)))
 
@@ -374,4 +377,5 @@ extern char *MakeCanonicalFlagName
 #define Has_power(p,x)      (check_access((p),powers_nametab[x].flag))
 #define Html(x)             ((Flags2(x) & HTML) != 0)
 #define s_Html(x)           s_Flags((x), FLAG_WORD2, Flags2(x) | HTML)
+
 #endif // !__FLAGS_H
