@@ -1,10 +1,10 @@
 // look.cpp -- Commands which look at things.
 //
-// $Id: look.cpp,v 1.20 2004/04/01 22:00:42 sdennis Exp $
+// $Id: look.cpp,v 1.45 2006/01/09 16:02:17 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -17,8 +17,11 @@
 #include "interface.h"
 #include "powers.h"
 
-#ifdef WOD_REALMS
+#ifdef REALITY_LVLS
+#include "levels.h"
+#endif /* REALITY_LVLS */
 
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
 #define NORMAL_REALM  0
 #define UMBRA_REALM   1
 #define SHROUD_REALM  2
@@ -29,7 +32,7 @@
 #define STAFF_REALM   7
 #define NUMBER_OF_REALMS 8
 
-int RealmActions[NUMBER_OF_REALMS] =
+static int RealmActions[NUMBER_OF_REALMS] =
 {
     REALM_DO_NORMALLY_SEEN,
     REALM_DO_SHOW_UMBRADESC,
@@ -53,7 +56,7 @@ int RealmActions[NUMBER_OF_REALMS] =
 #define MAP_NO_ADESC 2 // Don't trigger DESC actions on that.
 #define MAP_MEDIUM   4 // Hide this from that unless that is a medium and this is moving or talking.
 
-int RealmHiddenMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
+static int RealmHiddenMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
 {
     /* NORMAL  LOOKER */ {     MAP_SEEN, MAP_HIDE, MAP_MEDIUM, MAP_HIDE,     MAP_SEEN, MAP_HIDE, MAP_NO_ADESC, MAP_SEEN},
     /* UMBRA   LOOKER */ {     MAP_HIDE, MAP_SEEN, MAP_MEDIUM, MAP_HIDE,     MAP_HIDE, MAP_HIDE,     MAP_HIDE, MAP_SEEN},
@@ -65,7 +68,7 @@ int RealmHiddenMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
     /* STAFF   LOOKER */ {     MAP_SEEN, MAP_SEEN,   MAP_SEEN, MAP_SEEN,     MAP_SEEN, MAP_SEEN,     MAP_SEEN, MAP_SEEN}
 };
 
-int RealmExitsMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
+static int RealmExitsMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
 {
     /* NORMAL  LOOKER */ { MAP_SEEN, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE},
     /* UMBRA   LOOKER */ { MAP_SEEN, MAP_SEEN, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE, MAP_HIDE},
@@ -77,7 +80,7 @@ int RealmExitsMap[NUMBER_OF_REALMS][NUMBER_OF_REALMS] =
     /* STAFF   LOOKER */ { MAP_SEEN, MAP_SEEN, MAP_SEEN, MAP_SEEN, MAP_SEEN, MAP_SEEN, MAP_SEEN, MAP_SEEN}
 };
 
-int WhichRealm(dbref what, bool bPeering)
+static int WhichRealm(dbref what, bool bPeering)
 {
     int realm = NORMAL_REALM;
     if (isMatrix(what))       realm = MATRIX_REALM;
@@ -112,7 +115,7 @@ int WhichRealm(dbref what, bool bPeering)
     return realm;
 }
 
-int HandleObfuscation(dbref looker, dbref lookee, int threshhold)
+static int HandleObfuscation(dbref looker, dbref lookee, int threshhold)
 {
     int iReturn = REALM_DO_NORMALLY_SEEN;
     if (isObfuscate(lookee))
@@ -163,7 +166,6 @@ int HandleObfuscation(dbref looker, dbref lookee, int threshhold)
     }
     return iReturn;
 }
-
 int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state)
 {
     // If the looker is a room, then there is some contents/recursion stuff
@@ -194,7 +196,6 @@ int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state)
             }
         }
     }
-
     int realmLooker = WhichRealm(looker, isPeering(looker));
     int realmLookee = WhichRealm(lookee, false);
 
@@ -300,7 +301,6 @@ int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state)
         }
         return iReturn;
     }
-
     int iObfReturn = HandleObfuscation(looker, lookee, threshhold);
     switch (iObfReturn)
     {
@@ -321,7 +321,6 @@ int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state)
             bDisableADESC = true;
         }
     }
-
     if (bDisableADESC)
     {
         iReturn |= REALM_DISABLE_ADESC;
@@ -329,7 +328,7 @@ int DoThingToThingVisibility(dbref looker, dbref lookee, int action_state)
     return iReturn;
 }
 
-void LetDescriptionsDefault(dbref thing, int *piDESC, int *piADESC, int RealmDirective)
+static void LetDescriptionsDefault(dbref thing, int *piDESC, int *piADESC, int RealmDirective)
 {
     int   iDesc = 0;
     dbref owner;
@@ -383,13 +382,6 @@ void LetDescriptionsDefault(dbref thing, int *piDESC, int *piADESC, int RealmDir
 
 static void look_exits(dbref player, dbref loc, const char *exit_name)
 {
-#ifdef WOD_REALMS
-    if (isMatrix(player))
-    {
-        return;
-    }
-#endif
-
     // Make sure location has exits.
     //
     if (  !Good_obj(loc)
@@ -408,7 +400,11 @@ static void look_exits(dbref player, dbref loc, const char *exit_name)
     bool bFoundAny = false;
     int key = 0;
     int lev;
+#ifdef REALITY_LVLS
+    if (Dark(loc) || !IsReal(player, loc))
+#else
     if (Dark(loc))
+#endif /* REALITY_LVLS */
     {
         key |= VE_BASE_DARK;
     }
@@ -606,7 +602,11 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
 
     // Check to see if he can see the location.
     //
-    bool can_see_loc = (  !Dark(loc)
+#ifdef REALITY_LVLS
+     bool can_see_loc = ( !Dark(loc) && IsReal(player, loc)
+#else
+     bool can_see_loc = (  !Dark(loc)
+#endif /* REALITY_LVLS */
                        || (mudconf.see_own_dark && Examinable(player, loc)));
 
     dbref aowner;
@@ -625,7 +625,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
 
         DOLIST(thing, Contents(loc))
         {
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
             if (  can_see(player, thing, can_see_loc)
                && (REALM_DO_HIDDEN_FROM_YOU != DoThingToThingVisibility(player,
                                                 thing, ACTION_IS_STATIONARY)) )
@@ -683,7 +683,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
     //
     DOLIST(thing, Contents(loc))
     {
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
         if (  can_see(player, thing, can_see_loc)
            && (REALM_DO_HIDDEN_FROM_YOU != DoThingToThingVisibility(player, thing, ACTION_IS_STATIONARY)))
 #else
@@ -695,7 +695,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
             notify(player, contents_name);
             DOLIST(thing, Contents(loc))
             {
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
                 if (  can_see(player, thing, can_see_loc)
                    && (REALM_DO_HIDDEN_FROM_YOU != DoThingToThingVisibility(player, thing, ACTION_IS_STATIONARY)))
 #else
@@ -751,7 +751,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
 typedef struct
 {
     int mask;
-    int letter;
+    char letter;
 } ATTR_DECODE_ENTRY, *PATTR_DECODE_ENTRY;
 
 static ATTR_DECODE_ENTRY attr_decode_table[] =
@@ -761,6 +761,7 @@ static ATTR_DECODE_ENTRY attr_decode_table[] =
     { AF_CASE,    'C' },
     { AF_HTML,    'H' },
     { AF_PRIVATE, 'I' },
+    { AF_NOPARSE, 'P' },
     { AF_REGEXP,  'R' },
     { AF_VISUAL,  'V' },
     { AF_MDARK,   'M' },
@@ -806,7 +807,8 @@ static void view_atr
     // If we don't control the object or own the attribute, hide the
     // attr owner and flag info.
     //
-    if (!Controls(player, thing) && (Owner(player) != aowner))
+    if (  !Controls(player, thing)
+       && Owner(player) != aowner)
     {
         if (  skip_tag
            && ap->number == A_DESC)
@@ -823,7 +825,7 @@ static void view_atr
 
     // Generate flags.
     //
-    char xbuf[10];
+    char xbuf[11];
     decode_attr_flags(aflags, xbuf);
 
     if (  aowner != Owner(thing)
@@ -858,51 +860,95 @@ static void look_atrs1
     bool  hash_insert
 )
 {
-    dbref aowner;
-    int ca, aflags;
-    ATTR *attr;
-    char *as, *buf;
+    bool bFoundCommands = false;
+    bool bFoundListens  = false;
 
-    ATTR cattr;
-    for (ca = atr_head(thing, &as); ca; ca = atr_next(&as))
+    char *as;
+    for (int ca = atr_head(thing, &as); ca; ca = atr_next(&as))
     {
         if (  ca == A_DESC
            || ca == A_LOCK)
         {
             continue;
         }
-        attr = atr_num(ca);
-        if (!attr)
+
+        ATTR *pattr = atr_num(ca);
+        if (!pattr)
         {
             continue;
         }
 
-        memcpy(&cattr, attr, sizeof(ATTR));
+        ATTR cattr;
+        memcpy(&cattr, pattr, sizeof(ATTR));
 
         // Should we exclude this attr?
         //
         if (  check_exclude
-           && (  (attr->flags & AF_PRIVATE)
+           && (  (pattr->flags & AF_PRIVATE)
               || hashfindLEN(&ca, sizeof(ca), &mudstate.parent_htab)))
         {
             continue;
         }
 
-        buf = atr_get(thing, ca, &aowner, &aflags);
+        int   aflags;
+        dbref aowner;
+        char *buf = atr_get(thing, ca, &aowner, &aflags);
+
+        if (!(aflags & AF_NOPROG))
+        {
+            if (  AMATCH_CMD    == buf[0]
+               || AMATCH_LISTEN == buf[0])
+            {
+                char *s = strchr(buf+1, ':');
+                if (s)
+                {
+                    if (AMATCH_CMD == buf[0])
+                    {
+                        bFoundCommands = true;
+                    }
+                    else
+                    {
+                        bFoundListens = true;
+                    }
+                }
+            }
+        }
+
         if (bCanReadAttr(player, othing, &cattr, false))
         {
             if (!(check_exclude && (aflags & AF_PRIVATE)))
             {
                 if (hash_insert)
                 {
-                    hashaddLEN(&ca, sizeof(ca), attr,
-                        &mudstate.parent_htab);
+                    hashaddLEN(&ca, sizeof(ca), pattr, &mudstate.parent_htab);
                 }
                 view_atr(player, thing, &cattr, buf, aowner, aflags, false);
             }
         }
         free_lbuf(buf);
     }
+
+    if (bFoundCommands)
+    {
+        mudstate.bfNoCommands.Clear(thing);
+        mudstate.bfCommands.Set(thing);
+    }
+    else
+    {
+        mudstate.bfCommands.Clear(thing);
+        mudstate.bfNoCommands.Set(thing);
+    }
+
+    if (bFoundListens)
+    {
+        mudstate.bfNoListens.Clear(thing);
+        mudstate.bfListens.Set(thing);
+    }
+    else
+    {
+        mudstate.bfListens.Clear(thing);
+        mudstate.bfNoListens.Set(thing);
+    }
 }
 
 static void look_atrs(dbref player, dbref thing, bool check_parents)
@@ -936,7 +982,7 @@ static bool show_a_desc(dbref player, dbref loc)
 {
     int iDescDefault = A_DESC;
     int iADescDefault = A_ADESC;
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
     int iRealmDirective = DoThingToThingVisibility(player, loc, ACTION_IS_STATIONARY);
     if (REALM_DO_HIDDEN_FROM_YOU == iRealmDirective)
     {
@@ -963,7 +1009,7 @@ static bool show_a_desc(dbref player, dbref loc)
         char *tbuf1 = atr_pget(loc, iDescDefault, &aowner, &aflags);
         char *str = tbuf1;
         char *temp = alloc_lbuf("look_description.ET");
-        char *bp = temp; 
+        char *bp = temp;
         mux_exec(temp, &bp, loc, player, player,
                EV_FCHECK | EV_EVAL | EV_TOP,
                &str, (char **)NULL, 0);
@@ -980,11 +1026,15 @@ static bool show_a_desc(dbref player, dbref loc)
         mux_exec(FormatOutput, &tPtr, loc, player, player,
                 EV_FCHECK | EV_EVAL | EV_TOP,
                 &DescFormat, ParameterList, 2);
-        
-        notify(player, FormatOutput); 
-        did_it(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);  
 
-        free_lbuf(tbuf1); 
+        notify(player, FormatOutput);
+#ifdef REALITY_LVLS
+        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#else
+        did_it(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#endif /* REALITY_LVLS */
+
+        free_lbuf(tbuf1);
         free_lbuf(attrname);
         free_lbuf(FormatOutput);
         free_lbuf(temp);
@@ -999,7 +1049,11 @@ static bool show_a_desc(dbref player, dbref loc)
             got = atr_pget(loc, A_HTDESC, &aowner, &aflags);
             if (*got)
             {
+#ifdef REALITY_LVLS
+                did_it_rlevel(player, loc, A_HTDESC, NULL, A_ODESC, NULL, A_ADESC, (char **) NULL, 0);
+#else
                 did_it(player, loc, A_HTDESC, NULL, A_ODESC, NULL, A_ADESC, (char **) NULL, 0);
+#endif /* REALITY_LVLS */
                 ret = true;
             }
             else
@@ -1012,7 +1066,11 @@ static bool show_a_desc(dbref player, dbref loc)
                     {
                         raw_notify_newline(player);
                     }
+#ifdef REALITY_LVLS
+                    did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#else
                     did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#endif /* REALITY_LVLS */
                     if (indent)
                     {
                         raw_notify_newline(player);
@@ -1027,7 +1085,11 @@ static bool show_a_desc(dbref player, dbref loc)
             {
                 raw_notify_newline(player);
             }
+#ifdef REALITY_LVLS
+            did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#else
             did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+#endif /* REALITY_LVLS */
             if (indent)
             {
                 raw_notify_newline(player);
@@ -1049,7 +1111,7 @@ static void look_simple(dbref player, dbref thing, bool obey_terse)
         return;
     }
 
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
     int iRealmDirective = DoThingToThingVisibility(player, thing, ACTION_IS_STATIONARY);
     if (REALM_DO_HIDDEN_FROM_YOU == iRealmDirective)
     {
@@ -1070,15 +1132,21 @@ static void look_simple(dbref player, dbref thing, bool obey_terse)
     int iDescDefault = A_DESC;
     int iADescDefault = A_ADESC;
 
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
     LetDescriptionsDefault(thing, &iDescDefault, &iADescDefault, iRealmDirective);
 #endif
 
     int pattr = (obey_terse && Terse(player)) ? 0 : iDescDefault;
-    if (!show_a_desc(player,thing))
+    if (!show_a_desc(player, thing))
     {
         notify(player, "You see nothing special.");
-        did_it(player, thing, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);	
+#ifdef REALITY_LVLS
+        did_it_rlevel(player, thing, 0, NULL, A_ODESC, NULL, iADescDefault,
+            (char **)NULL, 0);
+#else
+        did_it(player, thing, pattr, NULL, A_ODESC, NULL, iADescDefault,
+            (char **)NULL, 0);
+#endif /* REALITY_LVLS */
     }
 
     if (  !mudconf.quiet_look
@@ -1098,14 +1166,22 @@ static void show_desc(dbref player, dbref loc, int key)
     if (  (key & LK_OBEYTERSE)
        && Terse(player))
     {
+#ifdef REALITY_LVLS
+        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+#else
         did_it(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+#endif /* REALITY_LVLS */
     }
     else if (  !isRoom(loc)
             && (key & LK_IDESC))
     {
         if (*(got = atr_pget(loc, A_IDESC, &aowner, &aflags)))
         {
+#ifdef REALITY_LVLS
+           did_it_rlevel(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+#else
             did_it(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+#endif /* REALITY_LVLS */
         }
         else
         {
@@ -1237,18 +1313,7 @@ void look_in(dbref player, dbref loc, int key)
     if (  !is_terse
        || mudconf.terse_contents)
     {
-#ifdef WOD_REALMS
-        if (isMatrix(player))
-        {
-            look_contents(player, loc, "|Icons|", CONTENTS_LOCAL);
-        }
-        else
-        {
-            look_contents(player, loc, "Contents:", CONTENTS_LOCAL);
-        }
-#else
         look_contents(player, loc, "Contents:", CONTENTS_LOCAL);
-#endif
     }
     if (  (key & LK_SHOWEXIT)
        && (  !is_terse
@@ -1260,6 +1325,9 @@ void look_in(dbref player, dbref loc, int key)
 
 void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     int look_key = LK_SHOWATTR | LK_SHOWEXIT;
     if (!mudconf.terse_look)
     {
@@ -1317,6 +1385,10 @@ void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
     //
     if (Good_obj(thing))
     {
+#ifdef REALITY_LVLS
+        if (!IsReal(executor, thing))
+            return;
+#endif /* REALITY_LVLS */
         switch (Typeof(thing))
         {
         case TYPE_ROOM:
@@ -1361,7 +1433,7 @@ static void debug_examine(dbref player, dbref thing)
     char *buf;
     int aflags, ca;
     BOOLEXP *pBoolExp;
-    ATTR *attr;
+    ATTR *pattr;
     char *as, *cp;
 
     notify(player, tprintf("Number  = %d", thing));
@@ -1382,9 +1454,17 @@ static void debug_examine(dbref player, dbref thing)
     buf = flag_description(player, thing);
     notify(player, tprintf("Flags   = %s", buf));
     free_mbuf(buf);
-    buf = power_description(player, thing);
+    buf = powers_list(player, thing);
     notify(player, tprintf("Powers  = %s", buf));
-    free_mbuf(buf);
+    free_lbuf(buf);
+#ifdef REALITY_LVLS
+    buf = rxlevel_description(player, thing);
+    notify(player, tprintf("RxLevel = %s", buf));
+    free_lbuf(buf);
+    buf = txlevel_description(player, thing);
+    notify(player, tprintf("TxLevel = %s", buf));
+    free_lbuf(buf);
+#endif /* REALITY_LVLS */
     buf = atr_get(thing, A_LOCK, &aowner, &aflags);
     pBoolExp = parse_boolexp(player, buf, true);
     free_lbuf(buf);
@@ -1397,20 +1477,20 @@ static void debug_examine(dbref player, dbref thing)
 
     for (ca = atr_head(thing, &as); ca; ca = atr_next(&as))
     {
-        attr = atr_num(ca);
-        if (!attr)
+        pattr = atr_num(ca);
+        if (!pattr)
         {
             continue;
         }
 
         atr_get_info(thing, ca, &aowner, &aflags);
-        if (bCanReadAttr(player, thing, attr, false))
+        if (bCanReadAttr(player, thing, pattr, false))
         {
-            if (attr)
+            if (pattr)
             {
                 // Valid attr.
                 //
-                safe_str(attr->name, buf, &cp);
+                safe_str(pattr->name, buf, &cp);
                 safe_chr(' ', buf, &cp);
             }
             else
@@ -1425,16 +1505,16 @@ static void debug_examine(dbref player, dbref thing)
 
     for (ca = atr_head(thing, &as); ca; ca = atr_next(&as))
     {
-        attr = atr_num(ca);
-        if (!attr)
+        pattr = atr_num(ca);
+        if (!pattr)
         {
             continue;
         }
 
         buf = atr_get(thing, ca, &aowner, &aflags);
-        if (bCanReadAttr(player, thing, attr, false))
+        if (bCanReadAttr(player, thing, pattr, false))
         {
-            view_atr(player, thing, attr, buf, aowner, aflags, 0);
+            view_atr(player, thing, pattr, buf, aowner, aflags, 0);
         }
         free_lbuf(buf);
     }
@@ -1514,6 +1594,9 @@ static void exam_wildattrs
 
 void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     // This command is pointless if the player can't hear.
     //
     if (!Hearer(executor))
@@ -1563,7 +1646,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         }
     }
 
-#ifdef WOD_REALMS
+#if defined(WOD_REALMS) || defined(REALITY_LVLS)
     if (REALM_DO_HIDDEN_FROM_YOU == DoThingToThingVisibility(executor, thing, ACTION_IS_STATIONARY))
     {
         notify(executor, NOMATCH_MESSAGE);
@@ -1681,9 +1764,19 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
             notify(executor, tprintf("Parent: %s", buf2));
             free_lbuf(buf2);
         }
-        buf2 = power_description(executor, thing);
-        notify(executor, buf2);
+        buf2 = powers_list(executor, thing);
+        notify(executor, tprintf("Powers: %s", buf2));
+        free_lbuf(buf2);
+#ifdef REALITY_LVLS
+        /* Show Rx and Tx levels */
+
+        buf2 = rxlevel_description(executor, thing);
+        notify(executor, tprintf("TxLevel: %s", buf2));
         free_mbuf(buf2);
+        buf2 = txlevel_description(executor, thing);
+        notify(executor, tprintf("RxLevel: %s", buf2));
+        free_mbuf(buf2);
+#endif /* REALITY_LVLS */
     }
     if (!(key & EXAM_BRIEF))
     {
@@ -1698,18 +1791,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         //
         if (Contents(thing) != NOTHING)
         {
-#ifdef WOD_REALMS
-            if (isMatrix(executor))
-            {
-                notify(executor, "|Icons|");
-            }
-            else
-            {
-                notify(executor, "Contents:");
-            }
-#else
             notify(executor, "Contents:");
-#endif
             DOLIST(content, Contents(thing))
             {
                 buf2 = unparse_object(executor, content, false);
@@ -1823,18 +1905,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
     {
         if (Has_contents(thing))
         {
-#ifdef WOD_REALMS
-            if (isMatrix(executor))
-            {
-                look_contents(executor, thing, "|Icons|", CONTENTS_REMOTE);
-            }
-            else
-            {
-                look_contents(executor, thing, "Contents:", CONTENTS_REMOTE);
-            }
-#else
             look_contents(executor, thing, "Contents:", CONTENTS_REMOTE);
-#endif
         }
         if (!isExit(thing))
         {
@@ -1861,12 +1932,20 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
 
 void do_score(dbref executor, dbref caller, dbref enactor, int key)
 {
-    notify(executor, tprintf("You have %d %s.", Pennies(executor),
-        (Pennies(executor) == 1) ?  mudconf.one_coin : mudconf.many_coins));
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
+    int nPennies = Pennies(executor);
+    notify(executor, tprintf("You have %d %s.", nPennies,
+        (1 == nPennies) ?  mudconf.one_coin : mudconf.many_coins));
 }
 
 void do_inventory(dbref executor, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref thing;
     char *buff, *e;
     const char *s;
@@ -1911,6 +1990,10 @@ void do_inventory(dbref executor, dbref caller, dbref enactor, int key)
 
 void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref thing, i, j;
     char *exit, *message;
     int control_thing, count, low_bound, high_bound;
@@ -2031,10 +2114,12 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
 //
 static void sweep_check(dbref player, dbref what, int key, bool is_loc)
 {
-    dbref aowner, parent;
-    int attr, aflags, lev;
-    char *buf, *buf2, *bp, *as, *buff, *s;
-    ATTR *ap;
+    if (  Can_Hide(what)
+       && Hidden(what)
+       && !See_Hidden(player))
+    {
+        return;
+    }
 
     bool canhear    = false;
     bool cancom     = false;
@@ -2043,70 +2128,91 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
     bool isconnected = false;
     bool is_parent  = false;
 
-    if (  (key & SWEEP_LISTEN)
-       && (  (  isExit(what)
-             || is_loc)
-          && Audible(what)))
+    if (key & SWEEP_LISTEN)
     {
-        canhear = true;
-    }
-    else if (key & SWEEP_LISTEN)
-    {
-        buff = NULL;
-        if (Monitor(what))
+        if (  (  (  isExit(what)
+                 || is_loc)
+              && Audible(what))
+           || H_Listen(what)
+           || mudstate.bfListens.IsSet(what))
         {
-            buff = alloc_lbuf("Hearer");
+            canhear = true;
         }
-
-        for (attr = atr_head(what, &as); attr; attr = atr_next(&as))
+        else if (  !mudstate.bfNoListens.IsSet(what)
+                && Monitor(what))
         {
-            if (attr == A_LISTEN)
+            bool bFoundCommands = false;
+
+            char *as;
+            char *buff = alloc_lbuf("sweep_check.Hearer");
+            for (int atr = atr_head(what, &as); atr; atr = atr_next(&as))
             {
-                canhear = true;
-                break;
-            }
-            if (Monitor(what))
-            {
-                ap = atr_num(attr);
+                ATTR *ap = atr_num(atr);
                 if (  !ap
                    || (ap->flags & AF_NOPROG))
                 {
                     continue;
                 }
 
-                atr_get_str(buff, what, attr, &aowner, &aflags);
+                int   aflags;
+                dbref aowner;
+                atr_get_str(buff, what, atr, &aowner, &aflags);
 
-                // Make sure we can execute it.
-                //
-                if (  (buff[0] != AMATCH_LISTEN)
-                   || (aflags & AF_NOPROG))
+                if (aflags & AF_NOPROG)
                 {
                     continue;
                 }
 
-                // Make sure there's a : in it.
-                //
-                for (s = buff + 1; *s && (*s != ':'); s++)
+                char *s = NULL;
+                if (  AMATCH_CMD    == buff[0]
+                   || AMATCH_LISTEN == buff[0])
                 {
-                    ; // Nothing.
-                }
-                if (s)
-                {
-                    canhear = true;
-                    break;
+                    s = strchr(buff+1, ':');
+                    if (s)
+                    {
+                        if (AMATCH_CMD == buff[0])
+                        {
+                            bFoundCommands = true;
+                        }
+                        else
+                        {
+                            canhear = true;
+                            break;
+                        }
+                    }
                 }
             }
-        }
-        if (buff)
-        {
             free_lbuf(buff);
+
+            if (canhear)
+            {
+                mudstate.bfListens.Set(what);
+            }
+            else
+            {
+                mudstate.bfNoListens.Set(what);
+            }
+
+            if (bFoundCommands)
+            {
+                mudstate.bfNoCommands.Clear(what);
+                mudstate.bfCommands.Set(what);
+            }
+            else
+            {
+                mudstate.bfCommands.Clear(what);
+                mudstate.bfNoCommands.Set(what);
+            }
         }
     }
+
     if (  (key & SWEEP_COMMANDS)
        && !isExit(what))
     {
         // Look for commands on the object and parents too.
         //
+        dbref parent;
+        int lev;
         ITER_PARENTS(what, parent, lev)
         {
             if (Commer(parent))
@@ -2120,6 +2226,7 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
             }
         }
     }
+
     if (key & SWEEP_CONNECT)
     {
         if (  Connected(what)
@@ -2133,6 +2240,7 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
             isconnected = true;
         }
     }
+
     if (  (key & SWEEP_PLAYER)
        || isconnected)
     {
@@ -2145,42 +2253,49 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
             ispuppet = true;
         }
     }
+
     if (  canhear
        || cancom
        || isplayer
        || ispuppet
        || isconnected)
     {
-        buf = alloc_lbuf("sweep_check.types");
-        bp = buf;
+        char *buf = alloc_lbuf("sweep_check.types");
+        char *bp = buf;
 
         if (cancom)
         {
             safe_str("commands ", buf, &bp);
         }
+
         if (canhear)
         {
             safe_str("messages ", buf, &bp);
         }
+
         if (isplayer)
         {
             safe_str("player ", buf, &bp);
         }
+
         if (ispuppet)
         {
             safe_str("puppet(", buf, &bp);
             safe_str(Name(Owner(what)), buf, &bp);
             safe_str(") ", buf, &bp);
         }
+
         if (isconnected)
         {
             safe_str("connected ", buf, &bp);
         }
+
         if (is_parent)
         {
             safe_str("parent ", buf, &bp);
         }
         bp[-1] = '\0';
+
         if (!isExit(what))
         {
             notify(player, tprintf("  %s is listening. [%s]",
@@ -2188,9 +2303,12 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
         }
         else
         {
-            buf2 = alloc_lbuf("sweep_check.name");
+            char *buf2 = alloc_lbuf("sweep_check.name");
             strcpy(buf2, Name(what));
-            for (bp = buf2; *bp && (*bp != ';'); bp++) ;
+            for (bp = buf2; *bp && (*bp != ';'); bp++)
+            {
+                ; // Nothing.
+            }
             *bp = '\0';
             notify(player, tprintf("  %s is listening. [%s]", buf2, buf));
             free_lbuf(buf2);
@@ -2201,6 +2319,9 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
 
 void do_sweep(dbref executor, dbref caller, dbref enactor, int key, char *where)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref here, sweeploc;
     int where_key, what_key;
 
@@ -2221,11 +2342,17 @@ void do_sweep(dbref executor, dbref caller, dbref enactor, int key, char *where)
     }
 
     if (!where_key)
+    {
         where_key = -1;
+    }
     if (!what_key)
+    {
         what_key = -1;
+    }
     else if (what_key == SWEEP_VERBOSE)
+    {
         what_key = SWEEP_VERBOSE | SWEEP_COMMANDS;
+    }
 
     // Check my location.  If I have none or it is dark, check just me.
     //
@@ -2302,8 +2429,6 @@ void do_sweep(dbref executor, dbref caller, dbref enactor, int key, char *where)
  * will almost certainly vary.  (i.e. different flags, etc.)
  */
 
-extern NAMETAB indiv_attraccess_nametab[];
-
 void do_decomp
 (
     dbref executor,
@@ -2315,11 +2440,15 @@ void do_decomp
     char *qual
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     BOOLEXP *pBoolExp;
     char *got, *thingname, *as, *ltext, *buff;
     dbref aowner, thing;
     int val, aflags, ca;
-    ATTR *attr;
+    ATTR *pattr;
     NAMETAB *np;
     bool wild_decomp;
 
@@ -2445,31 +2574,31 @@ void do_decomp
         {
             continue;
         }
-        attr = atr_num(ca);
-        if (!attr)
+        pattr = atr_num(ca);
+        if (!pattr)
         {
             continue;
         }
-        if (  (attr->flags & AF_NOCMD)
-           && !(attr->flags & AF_IS_LOCK))
+        if (  (pattr->flags & AF_NOCMD)
+           && !(pattr->flags & AF_IS_LOCK))
         {
             continue;
         }
 
         got = atr_get(thing, ca, &aowner, &aflags);
-        if (bCanReadAttr(executor, thing, attr, false))
+        if (bCanReadAttr(executor, thing, pattr, false))
         {
-            if (attr->flags & AF_IS_LOCK)
+            if (pattr->flags & AF_IS_LOCK)
             {
                 pBoolExp = parse_boolexp(executor, got, true);
                 ltext = unparse_boolexp_decompile(executor, pBoolExp);
                 free_boolexp(pBoolExp);
-                notify(executor, tprintf("@lock/%s %s=%s", attr->name,
+                notify(executor, tprintf("@lock/%s %s=%s", pattr->name,
                     thingname, ltext));
             }
             else
             {
-                strcpy(buff, attr->name);
+                strcpy(buff, pattr->name);
                 notify(executor, tprintf("%c%s %s=%s", ((ca < A_USER_START) ?
                     '@' : '&'), buff, thingname, got));
                 for (np = indiv_attraccess_nametab; np->name; np++)
@@ -2497,6 +2626,9 @@ void do_decomp
     {
         decompile_flags(executor, thing, thingname);
         decompile_powers(executor, thing, thingname);
+#ifdef REALITY_LVLS
+        decompile_rlevels(executor, thing, thingname);
+#endif /* REALITY_LVLS */
     }
 
     // If the object has a parent, report it.
