@@ -1,6 +1,6 @@
 // move.cpp -- Routines for moving about.
 //
-// $Id: move.cpp,v 1.4 2003/08/19 02:01:01 jake Exp $
+// $Id: move.cpp,v 1.7 2006/01/07 18:29:14 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -9,11 +9,15 @@
 #include "externs.h"
 
 #include "attrs.h"
+#include "command.h"
 #include "interface.h"
 #include "powers.h"
+#ifdef REALITY_LVLS
+#include "levels.h"
+#endif /* REALITY_LVLS */
 
 /* ---------------------------------------------------------------------------
- * process_leave_loc: Generate messages and actions resulting from leaving a 
+ * process_leave_loc: Generate messages and actions resulting from leaving a
  * place.
  */
 
@@ -45,7 +49,11 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
     // EXCEPT if we were called with the HUSH_LEAVE key.
     //
 
+#ifdef REALITY_LVLS
+    bool quiet = (  (hush & HUSH_LEAVE) || !IsReal(loc, thing)
+#else
     bool quiet = (  (hush & HUSH_LEAVE)
+#endif /* REALITY_LVLS */
                  || (  !Wizard(loc)
                     && (  Dark(thing)
                        || Dark(loc))
@@ -70,7 +78,7 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
     // criteria:
     //
     //   - Neither the current room nor the moving object are dark.
-    //   - The object can hear and is not a dark wizard. 
+    //   - The object can hear and is not a dark wizard.
     //
     if (  !quiet
        && !Blind(thing)
@@ -81,7 +89,11 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
            || (  canhear
               && !(Wizard(thing) && Dark(thing))))
         {
+#ifdef REALITY_LVLS
+            notify_except2_rlevel(loc, thing, thing, cause,
+#else
             notify_except2(loc, thing, thing, cause,
+#endif /* REALITY_LVLS */
                        tprintf("%s has left.", Name(thing)));
         }
     }
@@ -112,7 +124,11 @@ static void process_enter_loc(dbref thing, dbref src, dbref cause, bool canhear,
     //
     // EXCEPT if we were called with the HUSH_ENTER key.
     //
+#ifdef REALITY_LVLS
+    bool quiet = (  (hush & HUSH_ENTER) || !IsReal(loc, thing)
+#else
     bool quiet = (  (hush & HUSH_ENTER)
+#endif /* REALITY_LVLS */
                  || (  !Wizard(loc)
                     && (  Dark(thing)
                        || Dark(loc))
@@ -147,7 +163,11 @@ static void process_enter_loc(dbref thing, dbref src, dbref cause, bool canhear,
        && !Blind(loc)
        && !(Dark(thing) && Wizard(thing)))
     {
+#ifdef REALITY_LVLS
+        notify_except2_rlevel(loc, thing, thing, cause,
+#else
         notify_except2(loc, thing, thing, cause,
+#endif /* REALITY_LVLS */
                    tprintf("%s has arrived.", Name(thing)));
     }
 }
@@ -161,21 +181,21 @@ void move_object(dbref thing, dbref dest)
 {
     dbref src = Location(thing);
 
-    // Remove from the source location 
+    // Remove from the source location
     //
     if (src != NOTHING)
     {
         s_Contents(src, remove_first(Contents(src), thing));
     }
 
-    // Special check for HOME 
+    // Special check for HOME
     //
     if (dest == HOME)
     {
         dest = Home(thing);
     }
 
-    // Add to destination location 
+    // Add to destination location
     //
     if (dest != NOTHING)
     {
@@ -187,7 +207,7 @@ void move_object(dbref thing, dbref dest)
     }
     s_Location(thing, dest);
 
-    // Look around and do the penny check 
+    // Look around and do the penny check
     //
     look_in(thing, dest, (LK_SHOWEXIT | LK_OBEYTERSE));
     if (  isPlayer(thing)
@@ -203,7 +223,7 @@ void move_object(dbref thing, dbref dest)
 
 // move_the_exit: Move an exit silently from its location to its destination
 //
-void move_the_exit(dbref thing, dbref dest)
+static void move_the_exit(dbref thing, dbref dest)
 {
     dbref exitloc = Exits(thing);
     s_Exits(exitloc, remove_first(Exits(exitloc), thing));
@@ -216,7 +236,7 @@ void move_the_exit(dbref thing, dbref dest)
  * process_sacrifice_dropto: Check for and process droptos.
  */
 
-// send_dropto: Send an object through the dropto of a room 
+// send_dropto: Send an object through the dropto of a room
 //
 static void send_dropto(dbref thing, dbref player)
 {
@@ -238,48 +258,48 @@ static void process_sticky_dropto(dbref loc, dbref player)
 {
     dbref dropto, thing, next;
 
-    // Do nothing if checking anything but a sticky room 
+    // Do nothing if checking anything but a sticky room
     //
     if (!Good_obj(loc) || !Has_dropto(loc) || !Sticky(loc))
         return;
 
-    // Make sure dropto loc is valid 
+    // Make sure dropto loc is valid
     //
     dropto = Dropto(loc);
     if ((dropto == NOTHING) || (dropto == loc))
         return;
 
-    // Make sure no players hanging out 
+    // Make sure no players hanging out
     //
-    DOLIST(thing, Contents(loc)) 
+    DOLIST(thing, Contents(loc))
     {
         if ((Connected(Owner(thing)) && Hearer(thing)))
             return;
     }
 
-    // Send everything through the dropto 
+    // Send everything through the dropto
     //
     s_Contents(loc, reverse_list(Contents(loc)));
-    SAFE_DOLIST(thing, next, Contents(loc)) 
+    SAFE_DOLIST(thing, next, Contents(loc))
     {
         send_dropto(thing, player);
     }
 }
 
-// process_dropped_dropto: Check what to do when someone drops an object. 
+// process_dropped_dropto: Check what to do when someone drops an object.
 //
 static void process_dropped_dropto(dbref thing, dbref player)
 {
-    // If STICKY, send home 
+    // If STICKY, send home
     //
-    if (Sticky(thing)) 
+    if (Sticky(thing))
     {
         move_via_generic(thing, HOME, player, 0);
         divest_object(thing);
         return;
     }
 
-    // Process the dropto if location is a room and is not STICKY 
+    // Process the dropto if location is a room and is not STICKY
     //
     dbref loc = Location(thing);
     if (Has_dropto(loc) && (Dropto(loc) != NOTHING) && !Sticky(loc))
@@ -311,7 +331,7 @@ void move_via_generic(dbref thing, dbref dest, dbref cause, int hush)
  * move_via_exit: Exit move routine, generic + exit messages + dropto check.
  */
 
-void move_via_exit(dbref thing, dbref dest, dbref cause, dbref exit, int hush)
+static void move_via_exit(dbref thing, dbref dest, dbref cause, dbref exit, int hush)
 {
     if (dest == HOME)
     {
@@ -421,7 +441,7 @@ bool move_via_teleport(dbref thing, dbref dest, dbref cause, int hush)
  * move_exit: Try to move a player through an exit.
  */
 
-dbref get_exit_dest(dbref executor, dbref exit)
+static dbref get_exit_dest(dbref executor, dbref exit)
 {
     dbref aowner;
     int   aflags;
@@ -558,16 +578,19 @@ void move_exit(dbref player, dbref exit, bool divest, const char *failmsg, int h
 
 void do_move(dbref executor, dbref caller, dbref enactor, int key, char *direction)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref exit, loc;
     int i, quiet;
 
-    if (!string_compare(direction, "home")) 
-    {   
+    if (!string_compare(direction, "home"))
+    {
         // Go home w/o stuff.
         //
         if (  (  Fixed(executor)
               || Fixed(Owner(executor)))
-           && !(WizRoy(executor))) 
+           && !(WizRoy(executor)))
         {
             notify(executor, mudconf.fixed_home_msg);
             return;
@@ -575,13 +598,13 @@ void do_move(dbref executor, dbref caller, dbref enactor, int key, char *directi
 
         if (  (loc = Location(executor)) != NOTHING
            && !Dark(executor)
-           && !Dark(loc)) 
+           && !Dark(loc))
         {
-            // Tell all 
+            // Tell all
             //
             notify_except(loc, executor, executor, tprintf("%s goes home.", Name(executor)), 0);
         }
-        // Give the player the messages 
+        // Give the player the messages
         //
         for (i = 0; i < 3; i++)
             notify(executor, "There's no place like home...");
@@ -617,6 +640,9 @@ void do_move(dbref executor, dbref caller, dbref enactor, int key, char *directi
 
 void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref playerloc;
     if (  !Has_location(executor)
        || !Good_obj(playerloc = Location(executor)))
@@ -772,6 +798,9 @@ void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
 
 void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref loc = Location(executor);
     if (!Good_obj(loc))
         return;
@@ -785,7 +814,7 @@ void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
     match_possession();
     match_carried_exit();
 
-    switch (thing = match_result()) 
+    switch (thing = match_result())
     {
     case NOTHING:
         notify(executor, "You don't have that!");
@@ -795,7 +824,7 @@ void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
         return;
     }
 
-    switch (Typeof(thing)) 
+    switch (Typeof(thing))
     {
     case TYPE_THING:
     case TYPE_PLAYER:
@@ -835,12 +864,12 @@ void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
 
         // You have to be carrying it.
         //
-        if ((Exits(thing) != executor) && !Wizard(executor)) 
+        if ((Exits(thing) != executor) && !Wizard(executor))
         {
             notify(executor, "You can't drop that.");
             return;
         }
-        if (!Controls(executor, loc)) 
+        if (!Controls(executor, loc))
         {
             notify(executor, NOPERM_MESSAGE);
             return;
@@ -899,6 +928,9 @@ void do_enter_internal(dbref player, dbref thing, bool quiet)
 
 void do_enter(dbref executor, dbref caller, dbref enactor, int key, char *what)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     init_match(executor, what, TYPE_THING);
     match_neighbor();
     if (Long_Fingers(executor))
@@ -910,7 +942,7 @@ void do_enter(dbref executor, dbref caller, dbref enactor, int key, char *what)
     if (thing == NOTHING)
         return;
 
-    switch (Typeof(thing)) 
+    switch (Typeof(thing))
     {
     case TYPE_PLAYER:
     case TYPE_THING:
@@ -926,6 +958,9 @@ void do_enter(dbref executor, dbref caller, dbref enactor, int key, char *what)
 
 void do_leave(dbref executor, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref loc = Location(executor);
     dbref newLoc = loc;
 
