@@ -1,6 +1,6 @@
 // netcommon.cpp
 //
-// $Id: netcommon.cpp,v 1.32 2004/09/21 06:33:03 sdennis Exp $
+// $Id: netcommon.cpp,v 1.62 2006/01/07 18:17:49 sdennis Exp $
 //
 // This file contains routines used by the networking code that do not
 // depend on the implementation of the networking code.  The network-specific
@@ -23,15 +23,10 @@
 #include "mguests.h"
 #include "powers.h"
 #include "svdreport.h"
+#ifdef REALITY_LVLS
+#include "levels.h"
+#endif /* REALITY_LVLS */
 
-extern void handle_prog(DESC *, char *);
-
-#ifdef WIN32
-extern HANDLE CompletionPort;    // IOs are queued up on this port
-extern OVERLAPPED lpo_aborted; // special to indicate a player has finished TCP IOs
-extern OVERLAPPED lpo_aborted_final; // Actually free the descriptor.
-extern OVERLAPPED lpo_shutdown; // special to indicate a player should do a shutdown
-#endif
 
 /* ---------------------------------------------------------------------------
  * make_portlist: Make a list of ports for PORTS().
@@ -39,6 +34,8 @@ extern OVERLAPPED lpo_shutdown; // special to indicate a player should do a shut
 
 void make_portlist(dbref player, dbref target, char *buff, char **bufc)
 {
+    UNUSED_PARAMETER(player);
+
     ITL itl;
     ItemToList_Init(&itl, buff, bufc);
 
@@ -93,17 +90,21 @@ void make_port_ulist(dbref player, char *buff, char **bufc)
  * update_quotas: Update timeslice quotas
  */
 
-CLinearTimeAbsolute update_quotas(const CLinearTimeAbsolute& ltaLast, const CLinearTimeAbsolute& ltaCurrent)
+void update_quotas(CLinearTimeAbsolute& ltaLast, const CLinearTimeAbsolute& ltaCurrent)
 {
     if (ltaCurrent < ltaLast)
     {
-        return ltaCurrent;
+        ltaLast = ltaCurrent;
+        return;
     }
 
     CLinearTimeDelta ltdDiff = ltaCurrent - ltaLast;
-    CLinearTimeDelta ltdTimeSlice;
-    ltdTimeSlice.SetMilliseconds(mudconf.timeslice);
-    int nSlices = ltdDiff / ltdTimeSlice;
+    if (ltdDiff < mudconf.timeslice)
+    {
+        return;
+    }
+
+    int nSlices = ltdDiff / mudconf.timeslice;
     int nExtraQuota = mudconf.cmd_quota_incr * nSlices;
 
     if (nExtraQuota > 0)
@@ -118,7 +119,7 @@ CLinearTimeAbsolute update_quotas(const CLinearTimeAbsolute& ltaLast, const CLin
             }
         }
     }
-    return ltaLast + ltdTimeSlice * nSlices;
+    ltaLast += mudconf.timeslice * nSlices;
 }
 
 /* raw_notify_html() -- raw_notify() without the newline */
@@ -161,7 +162,8 @@ void raw_notify(dbref player, const char *msg)
         return;
     }
 
-    if (mudstate.inpipe && (player == mudstate.poutobj))
+    if (  mudstate.inpipe
+       && player == mudstate.poutobj)
     {
         safe_str(msg, mudstate.poutnew, &mudstate.poutbufc);
         safe_str("\r\n", mudstate.poutnew, &mudstate.poutbufc);
@@ -182,7 +184,8 @@ void raw_notify(dbref player, const char *msg)
 
 void raw_notify_newline(dbref player)
 {
-    if (mudstate.inpipe && (player == mudstate.poutobj))
+    if (  mudstate.inpipe
+       && player == mudstate.poutobj)
     {
         safe_str("\r\n", mudstate.poutnew, &mudstate.poutbufc);
         return;
@@ -247,7 +250,7 @@ void clearstrings(DESC *d)
     }
 }
 
-void add_to_output_queue(DESC *d, const char *b, int n)
+static void add_to_output_queue(DESC *d, const char *b, int n)
 {
     TBLOCK *tp;
     int left;
@@ -257,7 +260,7 @@ void add_to_output_queue(DESC *d, const char *b, int n)
     if (d->output_head == NULL)
     {
         tp = (TBLOCK *)MEMALLOC(OUTPUT_BLOCK_SIZE);
-        (void)ISOUTOFMEMORY(tp);
+        ISOUTOFMEMORY(tp);
         tp->hdr.nxt = NULL;
         tp->hdr.start = tp->data;
         tp->hdr.end = tp->data;
@@ -299,7 +302,7 @@ void add_to_output_queue(DESC *d, const char *b, int n)
                 n -= left;
             }
             tp = (TBLOCK *)MEMALLOC(OUTPUT_BLOCK_SIZE);
-            (void)ISOUTOFMEMORY(tp);
+            ISOUTOFMEMORY(tp);
             tp->hdr.nxt = NULL;
             tp->hdr.start = tp->data;
             tp->hdr.end = tp->data;
@@ -340,7 +343,7 @@ void queue_write_LEN(DESC *d, const char *b, int n)
         {
             STARTLOG(LOG_NET, "NET", "WRITE");
             char *buf = alloc_lbuf("queue_write.LOG");
-            sprintf(buf, "[%d/%s] Output buffer overflow, %d chars discarded by ", d->descriptor, d->addr, tp->hdr.nchars);
+            sprintf(buf, "[%u/%s] Output buffer overflow, %d chars discarded by ", d->descriptor, d->addr, tp->hdr.nchars);
             log_text(buf);
             free_lbuf(buf);
             if (d->flags & DS_CONNECTED)
@@ -379,6 +382,52 @@ void queue_write(DESC *d, const char *b)
     queue_write_LEN(d, b, strlen(b));
 }
 
+static const char *encode_iac(const char *szString)
+{
+    static char Buffer[2*LBUF_SIZE];
+    char *pBuffer = Buffer;
+
+    const char *pString = szString;
+    if (pString)
+    {
+        while (*pString)
+        {
+            const char *p = strchr(pString, NVT_IAC);
+            if (!p)
+            {
+                // NVT_IAC does not appear in the buffer. This is by far the most-common case.
+                //
+                if (pString == szString)
+                {
+                    // Avoid copying to the static buffer, and just return the original buffer.
+                    //
+                    return szString;
+                }
+                else
+                {
+                    strcpy(pBuffer, pString);
+                    return Buffer;
+                }
+            }
+            else
+            {
+                // Copy up to and including the IAC.
+                //
+                size_t n = p - pString + 1;
+                memcpy(pBuffer, pString, n);
+                pBuffer += n;
+                pString += n;
+
+                // Add another IAC.
+                //
+                *pBuffer++ = NVT_IAC;
+            }
+        }
+    }
+    *pBuffer = '\0';
+    return Buffer;
+}
+
 void queue_string(DESC *d, const char *s)
 {
     const char *p = s;
@@ -408,6 +457,7 @@ void queue_string(DESC *d, const char *s)
         }
         p = strip_accents(p);
     }
+    p = encode_iac(p);
     queue_write(d, p);
 }
 
@@ -438,9 +488,22 @@ void freeqs(DESC *d)
     d->input_tail = NULL;
 
     if (d->raw_input)
+    {
         free_lbuf(d->raw_input);
+    }
     d->raw_input = NULL;
+
     d->raw_input_at = NULL;
+    d->nOption = 0;
+    d->raw_input_state    = NVT_IS_NORMAL;
+    d->nvt_sga_him_state  = OPTION_NO;
+    d->nvt_sga_us_state   = OPTION_NO;
+    d->nvt_eor_him_state  = OPTION_NO;
+    d->nvt_eor_us_state   = OPTION_NO;
+    d->nvt_naws_him_state = OPTION_NO;
+    d->nvt_naws_us_state  = OPTION_NO;
+    d->height = 24;
+    d->width = 78;
 }
 
 /* ---------------------------------------------------------------------------
@@ -727,7 +790,7 @@ static void announce_connect(dbref player, DESC *d)
         if (  Suspect(player)
            || (d->host_info & H_SUSPECT))
         {
-            raw_broadcast(WIZARD, "[Suspect] %s has connected.", Name(player));
+            raw_broadcast(WIZARD, "[Suspect] %s has connected.", Moniker(player));
         }
     }
     else
@@ -737,11 +800,11 @@ static void announce_connect(dbref player, DESC *d)
         if (  Suspect(player)
            || (d->host_info & H_SUSPECT))
         {
-            raw_broadcast(WIZARD, "[Suspect] %s has reconnected.", Name(player));
+            raw_broadcast(WIZARD, "[Suspect] %s has reconnected.", Moniker(player));
         }
     }
-    sprintf(buf, pRoomAnnounceFmt, Name(player));
-    raw_broadcast(MONITOR, pMonitorAnnounceFmt, Name(player));
+    sprintf(buf, pRoomAnnounceFmt, Moniker(player));
+    raw_broadcast(MONITOR, pMonitorAnnounceFmt, Moniker(player));
 
     int key = MSG_INV;
     if (  loc != NOTHING
@@ -753,7 +816,14 @@ static void announce_connect(dbref player, DESC *d)
 
     dbref temp = mudstate.curr_enactor;
     mudstate.curr_enactor = player;
+#ifdef REALITY_LVLS
+    if(loc == NOTHING)
+        notify_check(player, player, buf, key);
+    else
+        notify_except_rlevel(loc, player, player, buf, 0);
+#else
     notify_check(player, player, buf, key);
+#endif /* REALITY_LVLS */
     atr_pget_str_LEN(buf, player, A_ACONNECT, &aowner, &aflags, &nLen);
     CLinearTimeAbsolute lta;
     dbref zone, obj;
@@ -828,6 +898,10 @@ static void announce_connect(dbref player, DESC *d)
 
     record_login(player, true, time_str, d->addr, d->username,
         inet_ntoa((d->address).sin_addr));
+    if (mudconf.have_mailer)
+    {
+        check_mail(player, 0, false);
+    }
     look_in(player, Location(player), (LK_SHOWEXIT|LK_OBEYTERSE|LK_SHOWVRML));
     mudstate.curr_enactor = temp;
     if (Guest(player))
@@ -854,11 +928,11 @@ void announce_disconnect(dbref player, DESC *d, const char *reason)
         if (  Suspect(player)
            || (d->host_info & H_SUSPECT))
         {
-            raw_broadcast(WIZARD, "[Suspect] %s has disconnected.", Name(player));
+            raw_broadcast(WIZARD, "[Suspect] %s has disconnected.", Moniker(player));
         }
         char *buf = alloc_lbuf("announce_disconnect.only");
 
-        sprintf(buf, "%s has disconnected.", Name(player));
+        sprintf(buf, "%s has disconnected.", Moniker(player));
         key = MSG_INV;
         if (  loc != NOTHING
            && !(  Hidden(player)
@@ -866,14 +940,21 @@ void announce_disconnect(dbref player, DESC *d, const char *reason)
         {
             key |= (MSG_NBR | MSG_NBR_EXITS | MSG_LOC | MSG_FWDLIST);
         }
+#ifdef REALITY_LVLS
+        if(loc == NOTHING)
+            notify_check(player, player, buf, key);
+        else
+            notify_except_rlevel(loc, player, player, buf, 0);
+#else
         notify_check(player, player, buf, key);
+#endif /* REALITY_LVLS */
 
         if (mudconf.have_mailer)
         {
             do_mail_purge(player);
         }
 
-        raw_broadcast(MONITOR, "GAME: %s has disconnected. <%s>", Name(player), reason);
+        raw_broadcast(MONITOR, "GAME: %s has disconnected. <%s>", Moniker(player), reason);
 
         c_Connected(player);
 
@@ -957,20 +1038,7 @@ void announce_disconnect(dbref player, DESC *d, const char *reason)
         if (d->flags & DS_AUTODARK)
         {
             d->flags &= ~DS_AUTODARK;
-
-            // Don't clear the DARK flag on the player unless there are
-            // no other sessions.
-            //
-            DESC *d1;
-            int num = 0;
-            DESC_ITER_PLAYER(player, d1)
-            {
-                num++;
-            }
-            if (num <= 1)
-            {
-                db[player].fs.word[FLAG_WORD1] &= ~DARK;
-            }
+            db[player].fs.word[FLAG_WORD1] &= ~DARK;
         }
 
         if (Guest(player))
@@ -984,10 +1052,10 @@ void announce_disconnect(dbref player, DESC *d, const char *reason)
         if (  Suspect(player)
            || (d->host_info & H_SUSPECT))
         {
-            raw_broadcast(WIZARD, "[Suspect] %s has partially disconnected.", Name(player));
+            raw_broadcast(WIZARD, "[Suspect] %s has partially disconnected.", Moniker(player));
         }
         char *mbuf = alloc_mbuf("announce_disconnect.partial");
-        sprintf(mbuf, "%s has partially disconnected.", Name(player));
+        sprintf(mbuf, "%s has partially disconnected.", Moniker(player));
         key = MSG_INV;
         if (  loc != NOTHING
            && !(  Hidden(player)
@@ -995,14 +1063,23 @@ void announce_disconnect(dbref player, DESC *d, const char *reason)
         {
             key |= (MSG_NBR | MSG_NBR_EXITS | MSG_LOC | MSG_FWDLIST);
         }
+#ifdef REALITY_LVLS
+        if(loc == NOTHING)
+            notify_check(player, player, mbuf, key);
+        else
+            notify_except_rlevel(loc, player, player, mbuf, 0);
+#else
         notify_check(player, player, mbuf, key);
+#endif /* REALITY_LVLS */
         raw_broadcast(MONITOR, "GAME: %s has partially disconnected.",
-            Name(player));
+            Moniker(player));
         free_mbuf(mbuf);
     }
 
     mudstate.curr_enactor = temp;
     desc_delhash(d);
+
+    local_disconnect(player, num);
 }
 
 int boot_off(dbref player, const char *message)
@@ -1064,7 +1141,9 @@ void desc_reload(dbref player)
         {
             d->timeout = mux_atol(buf);
             if (d->timeout <= 0)
+            {
                 d->timeout = mudconf.idle_timeout;
+            }
         }
         free_lbuf(buf);
     }
@@ -1084,30 +1163,57 @@ int fetch_session(dbref target)
     return nCount;
 }
 
+static DESC *find_least_idle(dbref target)
+{
+    CLinearTimeAbsolute ltaNewestLastTime;
+
+    DESC *d;
+    DESC *dLeastIdle = NULL;
+    DESC_ITER_PLAYER(target, d)
+    {
+        if (  NULL == dLeastIdle
+           || ltaNewestLastTime < d->last_time)
+        {
+            dLeastIdle = d;
+            ltaNewestLastTime = d->last_time;
+        }
+    }
+    return dLeastIdle;
+}
+
+int fetch_height(dbref target)
+{
+    DESC *d = find_least_idle(target);
+    if (NULL != d)
+    {
+        return d->height;
+    }
+    return 24;
+}
+
+int fetch_width(dbref target)
+{
+    DESC *d = find_least_idle(target);
+    if (NULL != d)
+    {
+        return d->width;
+    }
+    return 78;
+}
+
 // ---------------------------------------------------------------------------
 // fetch_idle: Return smallest idle time for a player (or -1 if not logged in).
 //
 int fetch_idle(dbref target)
 {
     CLinearTimeAbsolute ltaNow;
-    CLinearTimeAbsolute ltaNewestLastTime;
     ltaNow.GetUTC();
 
-    DESC *d;
-    bool bFound = false;
-    DESC_ITER_PLAYER(target, d)
-    {
-        if (  !bFound
-           || ltaNewestLastTime < d->last_time)
-        {
-            bFound = true;
-            ltaNewestLastTime = d->last_time;
-        }
-    }
-    if (bFound)
+    DESC *d = find_least_idle(target);
+    if (NULL != d)
     {
         CLinearTimeDelta ltdResult;
-        ltdResult = ltaNow - ltaNewestLastTime;
+        ltdResult = ltaNow - d->last_time;
         return ltdResult.ReturnSeconds();
     }
     else
@@ -1185,7 +1291,8 @@ void check_idle(void)
             // Send a Telnet NOP code - creates traffic to keep NAT routers
             // happy.  Hopefully this only runs once a minute.
             //
-            queue_write_LEN(d, "\377\361", 2);
+            const char aNOP[2] = { NVT_IAC, NVT_NOP };
+            queue_write_LEN(d, aNOP, sizeof(aNOP));
         }
         if (d->flags & DS_AUTODARK)
         {
@@ -1193,6 +1300,15 @@ void check_idle(void)
         }
         if (d->flags & DS_CONNECTED)
         {
+            if (mudconf.idle_timeout <= 0)
+            {
+                // Idle timeout checking on connected players is effectively disabled.
+                // PennMUSH uses idle_timeout == 0. Rhost uses idel_timeout == -1.
+                // We will be disabled for either setting.
+                //
+                continue;
+            }
+
             CLinearTimeDelta ltdIdle = ltaNow - d->last_time;
             if (Can_Idle(d->player))
             {
@@ -1233,7 +1349,7 @@ void check_idle(void)
                 shutdownsock(d, R_TIMEOUT);
             }
         }
-        else
+        else if (0 < mudconf.conn_timeout)
         {
             CLinearTimeDelta ltdIdle = ltaNow - d->connected_at;
             if (ltdIdle.ReturnSeconds() > mudconf.conn_timeout)
@@ -1298,14 +1414,14 @@ static const char *trimmed_name(dbref player, int *pvw)
 {
     static char cbuff[MBUF_SIZE];
 
-    size_t nName = ANSI_TruncateToField(
-                     Moniker(player),
-                     sizeof(cbuff),
-                     cbuff,
-                     MAX_TRIMMED_NAME_LENGTH,
-                     pvw,
-                     ANSI_ENDGOAL_NORMAL
-                  );
+    ANSI_TruncateToField(
+        Moniker(player),
+        sizeof(cbuff),
+        cbuff,
+        MAX_TRIMMED_NAME_LENGTH,
+        pvw,
+        ANSI_ENDGOAL_NORMAL
+    );
     return cbuff;
 }
 
@@ -1531,14 +1647,17 @@ static void dump_users(DESC *e, char *match, int key)
             //
             size_t nOnFor = 25 - nFill - vwNameField;
 
+            const char *pTimeStamp1 = time_format_1(ltdConnected.ReturnSeconds(), nOnFor);
+            const char *pTimeStamp2 = time_format_2(ltdLastTime.ReturnSeconds());
+
             if (  (e->flags & DS_CONNECTED)
                && Wizard_Who(e->player)
                && key == CMD_WHO)
             {
                 sprintf(buf, "%s%s%s %4s%-3s#%-6d%5d%3s%s\r\n",
                     pNameField, aFill,
-                    time_format_1(ltdConnected.ReturnSeconds(), nOnFor),
-                    time_format_2(ltdLastTime.ReturnSeconds()),
+                    pTimeStamp1,
+                    pTimeStamp2,
                     flist,
                     ((d->flags & DS_CONNECTED) ? Location(d->player) : -1),
                     d->command_count,
@@ -1547,10 +1666,10 @@ static void dump_users(DESC *e, char *match, int key)
             }
             else if (key == CMD_SESSION)
             {
-                sprintf(buf, "%s%s%s %4s%5d%5d%6d%10d%6d%6d%10d\r\n",
+                sprintf(buf, "%s%s%s %4s%5u%5d%6d%10d%6d%6d%10d\r\n",
                     pNameField, aFill,
-                    time_format_1(ltdConnected.ReturnSeconds(), nOnFor),
-                    time_format_2(ltdLastTime.ReturnSeconds()),
+                    pTimeStamp1,
+                    pTimeStamp2,
                     d->descriptor,
                     d->input_size, d->input_lost,
                     d->input_tot,
@@ -1562,8 +1681,8 @@ static void dump_users(DESC *e, char *match, int key)
             {
                 sprintf(buf, "%s%s%s %4s%-3s%s\r\n",
                     pNameField, aFill,
-                    time_format_1(ltdConnected.ReturnSeconds(), nOnFor),
-                    time_format_2(ltdLastTime.ReturnSeconds()),
+                    pTimeStamp1,
+                    pTimeStamp2,
                     flist,
                     d->doing);
             }
@@ -1571,8 +1690,8 @@ static void dump_users(DESC *e, char *match, int key)
             {
                 sprintf(buf, "%s%s%s %4s  %s\r\n",
                     pNameField, aFill,
-                    time_format_1(ltdConnected.ReturnSeconds(), nOnFor),
-                    time_format_2(ltdLastTime.ReturnSeconds()),
+                    pTimeStamp1,
+                    pTimeStamp2,
                     d->doing);
             }
             queue_string(e, buf);
@@ -1633,7 +1752,7 @@ static void dump_info(DESC *arg_desc)
     queue_write(arg_desc, "### End INFO\r\n");
 }
 
-char *MakeCanonicalDoing(char *pDoing, int *pnValidDoing, bool *pbValidDoing)
+static char *MakeCanonicalDoing(char *pDoing, int *pnValidDoing, bool *pbValidDoing)
 {
     *pnValidDoing = 0;
     *pbValidDoing = false;
@@ -1669,6 +1788,9 @@ char *MakeCanonicalDoing(char *pDoing, int *pnValidDoing, bool *pbValidDoing)
 //
 void do_doing(dbref executor, dbref caller, dbref enactor, int key, char *arg)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     // Make sure there can be no embedded newlines from %r
     //
     static char *Empty = "";
@@ -1798,7 +1920,7 @@ static void failconn(const char *logcode, const char *logtype, const char *logre
 {
     STARTLOG(LOG_LOGIN | LOG_SECURITY, logcode, "RJCT");
     char *buff = alloc_mbuf("failconn.LOG");
-    sprintf(buff, "[%d/%s] %s rejected to ", d->descriptor, d->addr, logtype);
+    sprintf(buff, "[%u/%s] %s rejected to ", d->descriptor, d->addr, logtype);
     log_text(buff);
     free_mbuf(buff);
     if (player != NOTHING)
@@ -1928,7 +2050,7 @@ static bool check_connect(DESC *d, char *msg)
             queue_write(d, connect_fail);
             STARTLOG(LOG_LOGIN | LOG_SECURITY, "CON", "BAD");
             buff = alloc_lbuf("check_conn.LOG.bad");
-            sprintf(buff, "[%d/%s] Failed connect to '%s'", d->descriptor, d->addr, user);
+            sprintf(buff, "[%u/%s] Failed connect to '%s'", d->descriptor, d->addr, user);
             log_text(buff);
             free_lbuf(buff);
             ENDLOG;
@@ -1984,7 +2106,7 @@ static bool check_connect(DESC *d, char *msg)
             //
             STARTLOG(LOG_LOGIN, "CON", "LOGIN");
             buff = alloc_mbuf("check_conn.LOG.login");
-            sprintf(buff, "[%d/%s] Connected to ", d->descriptor, d->addr);
+            sprintf(buff, "[%u/%s] Connected to ", d->descriptor, d->addr);
             log_text(buff);
             log_name_and_loc(player);
             free_mbuf(buff);
@@ -2026,6 +2148,14 @@ static bool check_connect(DESC *d, char *msg)
             }
             announce_connect(player, d);
 
+            DESC* dtemp;
+            int num_con = 0;
+            DESC_ITER_PLAYER(player, dtemp)
+            {
+                num_con++;
+            }
+            local_connect(player, 0, num_con);
+
             // If stuck in an @prog, show the prompt.
             //
             if (d->program_data != NULL)
@@ -2096,7 +2226,7 @@ static bool check_connect(DESC *d, char *msg)
                 queue_write(d, "\r\n");
                 STARTLOG(LOG_SECURITY | LOG_PCREATES, "CON", "BAD");
                 buff = alloc_lbuf("check_conn.LOG.badcrea");
-                sprintf(buff, "[%d/%s] Create of '%s' failed", d->descriptor, d->addr, user);
+                sprintf(buff, "[%u/%s] Create of '%s' failed", d->descriptor, d->addr, user);
                 log_text(buff);
                 free_lbuf(buff);
                 ENDLOG;
@@ -2106,7 +2236,7 @@ static bool check_connect(DESC *d, char *msg)
                 AddToPublicChannel(player);
                 STARTLOG(LOG_LOGIN | LOG_PCREATES, "CON", "CREA");
                 buff = alloc_mbuf("check_conn.LOG.create");
-                sprintf(buff, "[%d/%s] Created ", d->descriptor, d->addr);
+                sprintf(buff, "[%u/%s] Created ", d->descriptor, d->addr);
                 log_text(buff);
                 log_name(player);
                 free_mbuf(buff);
@@ -2117,6 +2247,11 @@ static bool check_connect(DESC *d, char *msg)
                 d->player = player;
                 fcache_dump(d, FC_CREA_NEW);
                 announce_connect(player, d);
+
+                // Since it is on the create call, assume connection count
+                // is 0 and indicate the connect is a new character.
+                //
+                local_connect(player, 1, 0);
             }
         }
     }
@@ -2126,7 +2261,7 @@ static bool check_connect(DESC *d, char *msg)
         STARTLOG(LOG_LOGIN | LOG_SECURITY, "CON", "BAD");
         buff = alloc_mbuf("check_conn.LOG.bad");
         msg[150] = '\0';
-        sprintf(buff, "[%d/%s] Failed connect: '%s'", d->descriptor, d->addr, msg);
+        sprintf(buff, "[%u/%s] Failed connect: '%s'", d->descriptor, d->addr, msg);
         log_text(buff);
         free_mbuf(buff);
         ENDLOG;
@@ -2139,11 +2274,131 @@ static bool check_connect(DESC *d, char *msg)
     return true;
 }
 
-bool do_command(DESC *d, char *command)
+static void do_logged_out_internal(DESC *d, int key, char *arg)
+{
+    switch (key)
+    {
+    case CMD_QUIT:
+
+        shutdownsock(d, R_QUIT);
+        break;
+
+    case CMD_LOGOUT:
+
+        shutdownsock(d, R_LOGOUT);
+        break;
+
+    case CMD_WHO:
+    case CMD_DOING:
+    case CMD_SESSION:
+
+        dump_users(d, arg, key);
+        break;
+
+    case CMD_PREFIX:
+
+        set_userstring(&d->output_prefix, arg);
+        break;
+
+    case CMD_SUFFIX:
+
+        set_userstring(&d->output_suffix, arg);
+        break;
+
+    case CMD_INFO:
+
+        dump_info(d);
+        break;
+
+    case CMD_PUEBLOCLIENT:
+
+        // Set the descriptor's flag.
+        //
+        d->flags |= DS_PUEBLOCLIENT;
+
+        queue_string(d, mudconf.pueblo_msg);
+        queue_write_LEN(d, "\r\n", 2);
+        break;
+
+    default:
+
+        {
+            char buf[LBUF_SIZE * 2];
+            STARTLOG(LOG_BUGS, "BUG", "PARSE");
+            sprintf(buf, "Logged-out command with no handler: '%s'", mudstate.debug_cmd);
+            log_text(buf);
+            ENDLOG;
+        }
+    }
+}
+
+void do_command(DESC *d, char *command)
 {
     char *cmdsave = mudstate.debug_cmd;
     mudstate.debug_cmd = (char *)"< do_command >";
 
+    if (d->flags & DS_CONNECTED)
+    {
+        // Normal logged-in command processing.
+        //
+        d->command_count++;
+        if (d->output_prefix)
+        {
+            queue_string(d, d->output_prefix);
+            queue_write_LEN(d, "\r\n", 2);
+        }
+        mudstate.curr_executor = d->player;
+        mudstate.curr_enactor = d->player;
+        for (int i = 0; i < MAX_GLOBAL_REGS; i++)
+        {
+            mudstate.global_regs[i][0] = '\0';
+            mudstate.glob_reg_len[i] = 0;
+        }
+
+        CLinearTimeAbsolute ltaBegin;
+        ltaBegin.GetUTC();
+        MuxAlarm.Set(mudconf.max_cmdsecs);
+
+        char *log_cmdbuf = process_command(d->player, d->player, d->player,
+            true, command, (char **)NULL, 0);
+
+        CLinearTimeAbsolute ltaEnd;
+        ltaEnd.GetUTC();
+        if (MuxAlarm.bAlarmed)
+        {
+            notify(d->player, "GAME: Expensive activity abbreviated.");
+            halt_que(d->player, NOTHING);
+            s_Flags(d->player, FLAG_WORD1, Flags(d->player) | HALT);
+        }
+        MuxAlarm.Clear();
+
+        CLinearTimeDelta ltd = ltaEnd - ltaBegin;
+        if (ltd > mudconf.rpt_cmdsecs)
+        {
+            STARTLOG(LOG_PROBLEMS, "CMD", "CPU");
+            log_name_and_loc(d->player);
+            char *logbuf = alloc_lbuf("do_command.LOG.cpu");
+            sprintf(logbuf, " queued command taking %s secs: ",
+                ltd.ReturnSecondsString(4));
+            log_text(logbuf);
+            free_lbuf(logbuf);
+            log_text(log_cmdbuf);
+            ENDLOG;
+        }
+
+        mudstate.curr_cmd = (char *) "";
+        if (d->output_suffix)
+        {
+            queue_string(d, d->output_suffix);
+            queue_write_LEN(d, "\r\n", 2);
+        }
+        mudstate.debug_cmd = cmdsave;
+        return;
+    }
+
+    // Login screen (logged-out) command processing.
+    //
+
     // Split off the command from the arguments.
     //
     char *arg = command;
@@ -2157,77 +2412,24 @@ bool do_command(DESC *d, char *command)
         *arg++ = '\0';
     }
 
-    // Look up the command.  If we don't find it, turn it over to the normal
-    // logged-in command processor or to create/connect.
+    // Look up the command in the logged-out command table.
     //
-    NAMETAB *cp = NULL;
-    if (!(d->flags & DS_CONNECTED))
-    {
-        cp = (NAMETAB *)hashfindLEN(command, strlen(command), &mudstate.logout_cmd_htab);
-    }
+    NAMETAB *cp = (NAMETAB *)hashfindLEN(command, strlen(command), &mudstate.logout_cmd_htab);
     if (cp == NULL)
     {
+        // Not in the logged-out command table, so maybe a connect attempt.
+        //
         if (*arg)
         {
             // Restore nullified space
             //
             *--arg = ' ';
         }
-        if (d->flags & DS_CONNECTED)
-        {
-            d->command_count++;
-            if (d->output_prefix)
-            {
-                queue_string(d, d->output_prefix);
-                queue_write_LEN(d, "\r\n", 2);
-            }
-            mudstate.curr_executor = d->player;
-            mudstate.curr_enactor = d->player;
-            for (int i = 0; i < MAX_GLOBAL_REGS; i++)
-            {
-                mudstate.global_regs[i][0] = '\0';
-                mudstate.glob_reg_len[i] = 0;
-            }
-
-            CLinearTimeAbsolute ltaBegin;
-            ltaBegin.GetUTC();
-
-            char *log_cmdbuf = process_command(d->player, d->player, d->player,
-                true, command, (char **)NULL, 0);
-
-            CLinearTimeAbsolute ltaEnd;
-            ltaEnd.GetUTC();
-
-            CLinearTimeDelta ltd = ltaEnd - ltaBegin;
-            if (ltd > mudconf.max_cmdsecs)
-            {
-                STARTLOG(LOG_PROBLEMS, "CMD", "CPU");
-                log_name_and_loc(d->player);
-                char *logbuf = alloc_lbuf("do_command.LOG.cpu");
-                sprintf(logbuf, " queued command taking %s secs: ",
-                    ltd.ReturnSecondsString(4));
-                log_text(logbuf);
-                free_lbuf(logbuf);
-                log_text(log_cmdbuf);
-                ENDLOG;
-            }
-
-            mudstate.curr_cmd = (char *) "";
-            if (d->output_suffix)
-            {
-                queue_string(d, d->output_suffix);
-                queue_write_LEN(d, "\r\n", 2);
-            }
-            mudstate.debug_cmd = cmdsave;
-            return true;
-        }
-        else
-        {
-            mudstate.curr_executor = NOTHING;
-            mudstate.curr_enactor = NOTHING;
-            mudstate.debug_cmd = cmdsave;
-            return check_connect(d, command);
-        }
+        mudstate.curr_executor = NOTHING;
+        mudstate.curr_enactor = NOTHING;
+        mudstate.debug_cmd = cmdsave;
+        check_connect(d, command);
+        return;
     }
 
     // The command was in the logged-out command table. Perform
@@ -2243,87 +2445,21 @@ bool do_command(DESC *d, char *command)
             queue_write_LEN(d, "\r\n", 2);
         }
     }
-    if (  (  (d->flags & DS_CONNECTED)
-          && !check_access(d->player, cp->perm))
-       || ( !(d->flags & DS_CONNECTED)
-          && cp->perm & CA_PLAYER))
+    if (cp->perm != CA_PUBLIC)
     {
         queue_write(d, "Permission denied.\r\n");
     }
     else
     {
         mudstate.debug_cmd = cp->name;
-        switch (cp->flag & CMD_MASK)
-        {
-        case CMD_QUIT:
-
-            shutdownsock(d, R_QUIT);
-            mudstate.debug_cmd = cmdsave;
-            return false;
-
-        case CMD_LOGOUT:
-
-            shutdownsock(d, R_LOGOUT);
-            break;
-
-        case CMD_WHO:
-
-            dump_users(d, arg, CMD_WHO);
-            break;
-
-        case CMD_DOING:
-
-            dump_users(d, arg, CMD_DOING);
-            break;
-
-        case CMD_SESSION:
-
-            dump_users(d, arg, CMD_SESSION);
-            break;
-
-        case CMD_PREFIX:
-
-            set_userstring(&d->output_prefix, arg);
-            break;
-
-        case CMD_SUFFIX:
-
-            set_userstring(&d->output_suffix, arg);
-            break;
-
-        case CMD_INFO:
-
-            dump_info(d);
-            break;
-
-        case CMD_PUEBLOCLIENT:
-
-            // Set the descriptor's flag.
-            //
-            d->flags |= DS_PUEBLOCLIENT;
-
-            // If we're already connected, set the player's flag.
-            //
-            if (d->flags & DS_CONNECTED)
-            {
-                s_Html(d->player);
-            }
-            queue_string(d, mudconf.pueblo_msg);
-            queue_write_LEN(d, "\r\n", 2);
-            break;
-
-        default:
-
-            {
-                char buf[LBUF_SIZE * 2];
-                STARTLOG(LOG_BUGS, "BUG", "PARSE");
-                sprintf(buf, "Prefix command with no handler: '%s'", command);
-                log_text(buf);
-                ENDLOG;
-            }
-        }
+        do_logged_out_internal(d, cp->flag & CMD_MASK, arg);
     }
-    if (!(cp->flag & CMD_NOxFIX))
+    // QUIT or LOGOUT will close the connection and cause the
+    // descriptor to be freed!
+    //
+    if (  ((cp->flag & CMD_MASK) != CMD_QUIT)
+       && ((cp->flag & CMD_MASK) != CMD_LOGOUT)
+       && !(cp->flag & CMD_NOxFIX))
     {
         if (d->output_suffix)
         {
@@ -2332,32 +2468,31 @@ bool do_command(DESC *d, char *command)
         }
     }
     mudstate.debug_cmd = cmdsave;
-    return true;
 }
 
 void logged_out1(dbref executor, dbref caller, dbref enactor, int key, char *arg)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
+    // PUEBLOCLIENT affects all the player's connections.
+    //
     if (key == CMD_PUEBLOCLIENT)
     {
         DESC *d;
         DESC_ITER_PLAYER(executor, d)
         {
-            // Set the descriptor's flag.
-            //
-            d->flags |= DS_PUEBLOCLIENT;
-
-            // If we're already connected, set the player's flag.
-            //
-            if (d->flags & DS_CONNECTED)
-            {
-                s_Html(d->player);
-            }
-            queue_string(d, mudconf.pueblo_msg);
-            queue_write_LEN(d, "\r\n", 2);
+            do_logged_out_internal(d, key, arg);
         }
+        // Set the player's flag.
+        //
+        s_Html(executor);
         return;
     }
 
+    // Other logged-out commands affect only the player's most recently
+    // used connection.
+    //
     DESC *d;
     DESC *dLatest = NULL;
     DESC_ITER_PLAYER(executor, d)
@@ -2368,44 +2503,9 @@ void logged_out1(dbref executor, dbref caller, dbref enactor, int key, char *arg
             dLatest = d;
         }
     }
-    if (dLatest == NULL)
+    if (dLatest != NULL)
     {
-        return;
-    }
-
-    switch (key)
-    {
-    case CMD_QUIT:
-        shutdownsock(dLatest, R_QUIT);
-        break;
-
-    case CMD_LOGOUT:
-        shutdownsock(dLatest, R_LOGOUT);
-        break;
-
-    case CMD_WHO:
-        dump_users(dLatest, arg, CMD_WHO);
-        break;
-
-    case CMD_DOING:
-        dump_users(dLatest, arg, CMD_DOING);
-        break;
-
-    case CMD_SESSION:
-        dump_users(dLatest, arg, CMD_SESSION);
-        break;
-
-    case CMD_PREFIX:
-        set_userstring(&dLatest->output_prefix, arg);
-        break;
-
-    case CMD_SUFFIX:
-        set_userstring(&dLatest->output_suffix, arg);
-        break;
-
-    case CMD_INFO:
-        dump_info(dLatest);
-        break;
+        do_logged_out_internal(dLatest, key, arg);
     }
 }
 
@@ -2416,6 +2516,8 @@ void logged_out0(dbref executor, dbref caller, dbref enactor, int key)
 
 void Task_ProcessCommand(void *arg_voidptr, int arg_iInteger)
 {
+    UNUSED_PARAMETER(arg_iInteger);
+
     DESC *d = (DESC *)arg_voidptr;
     if (d)
     {
@@ -2436,8 +2538,8 @@ void Task_ProcessCommand(void *arg_voidptr, int arg_iInteger)
                 {
                     d->input_tail = NULL;
                 }
-                d->input_size -= (strlen(t->cmd) + 1);
-                d->last_time = mudstate.now;
+                d->input_size -= strlen(t->cmd);
+                d->last_time.GetUTC();
                 if (d->program_data != NULL)
                 {
                     handle_prog(d, t->cmd);
@@ -2452,10 +2554,10 @@ void Task_ProcessCommand(void *arg_voidptr, int arg_iInteger)
             {
                 // Don't bother looking for more quota until at least this much time has past.
                 //
-                CLinearTimeDelta ltd;
-                ltd.SetMilliseconds(mudconf.timeslice);
-                CLinearTimeAbsolute lsaWhen = mudstate.now + ltd;
-                scheduler.DeferTask(lsaWhen, PRIORITY_SYSTEM, Task_ProcessCommand, d, 0);
+                CLinearTimeAbsolute lsaWhen;
+                lsaWhen.GetUTC();
+
+                scheduler.DeferTask(lsaWhen + mudconf.timeslice, PRIORITY_SYSTEM, Task_ProcessCommand, d, 0);
             }
         }
     }
@@ -2636,6 +2738,12 @@ dbref find_connected_name(dbref player, char *name)
 
 FUNCTION(fun_doing)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nfargs);
+    UNUSED_PARAMETER(cargs);
+    UNUSED_PARAMETER(ncargs);
+
     if (is_rational(fargs[0]))
     {
         SOCKET s = mux_atol(fargs[0]);
@@ -2643,7 +2751,7 @@ FUNCTION(fun_doing)
         DESC *d;
         DESC_ITER_CONN(d)
         {
-            if (d->descriptor == s) 
+            if (d->descriptor == s)
             {
                 bFound = true;
                 break;
@@ -2668,7 +2776,7 @@ FUNCTION(fun_doing)
             safe_str("#-1 PLAYER DOES NOT EXIST", buff, bufc);
             return;
         }
-    
+
         if (  Wizard_Who(executor)
            || !Hidden(victim))
         {
@@ -2691,12 +2799,18 @@ FUNCTION(fun_doing)
 // ---------------------------------------------------------------------------
 FUNCTION(fun_host)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nfargs);
+    UNUSED_PARAMETER(cargs);
+    UNUSED_PARAMETER(ncargs);
+
     if (!Wizard_Who(executor))
     {
         safe_noperm(buff, bufc);
         return;
     }
-     
+
     bool isPort = is_rational(fargs[0]);
     bool bFound = false;
     DESC *d;
@@ -2705,13 +2819,13 @@ FUNCTION(fun_host)
         SOCKET s = mux_atol(fargs[0]);
         DESC_ITER_CONN(d)
         {
-            if (d->descriptor == s) 
+            if (d->descriptor == s)
             {
                 bFound = true;
                 break;
             }
         }
-    } 
+    }
     else
     {
         dbref victim = lookup_player(executor, fargs[0], true);
@@ -2731,7 +2845,7 @@ FUNCTION(fun_host)
     }
     if (bFound)
     {
-        char *hostname = ((d->username[0] != '\0') ? 
+        char *hostname = ((d->username[0] != '\0') ?
             tprintf("%s@%s", d->username, d->addr) : d->addr);
         safe_str(hostname, buff, bufc);
         return;
@@ -2748,11 +2862,27 @@ FUNCTION(fun_host)
 
 FUNCTION(fun_poll)
 {
+    UNUSED_PARAMETER(executor);
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(fargs);
+    UNUSED_PARAMETER(nfargs);
+    UNUSED_PARAMETER(cargs);
+    UNUSED_PARAMETER(ncargs);
+
     safe_str(mudstate.doing_hdr, buff, bufc);
 }
 
 FUNCTION(fun_motd)
 {
+    UNUSED_PARAMETER(executor);
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(fargs);
+    UNUSED_PARAMETER(nfargs);
+    UNUSED_PARAMETER(cargs);
+    UNUSED_PARAMETER(ncargs);
+
     safe_str(mudconf.motd_msg, buff, bufc);
 }
 
@@ -2780,7 +2910,7 @@ int fetch_cmds(dbref target)
     }
 }
 
-void ParseConnectionInfoString(char *pConnInfo, char *pFields[5])
+static void ParseConnectionInfoString(char *pConnInfo, char *pFields[5])
 {
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, pConnInfo);
