@@ -1,6 +1,6 @@
 // object.cpp -- Low-level object manipulation routines.
 //
-// $Id: object.cpp,v 1.12 2004/06/19 21:48:37 sdennis Exp $
+// $Id: object.cpp,v 1.17 2006/01/07 17:36:11 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -32,17 +32,17 @@ static void Log_pointer_err(dbref prior, dbref obj, dbref loc, dbref ref,
 {
     STARTLOG(LOG_PROBLEMS, "OBJ", "DAMAG");
     log_type_and_name(obj);
-    if (loc != NOTHING) 
+    if (loc != NOTHING)
     {
         log_text(" in ");
         log_type_and_name(loc);
     }
     log_text(": ");
-    if (prior == NOTHING) 
+    if (prior == NOTHING)
     {
         log_text(reftype);
-    } 
-    else 
+    }
+    else
     {
         log_text("Next pointer");
     }
@@ -58,7 +58,7 @@ static void Log_header_err(dbref obj, dbref loc, dbref val, bool is_object,
 {
     STARTLOG(LOG_PROBLEMS, "OBJ", "DAMAG");
     log_type_and_name(obj);
-    if (loc != NOTHING) 
+    if (loc != NOTHING)
     {
         log_text(" in ");
         log_type_and_name(loc);
@@ -83,7 +83,7 @@ static void Log_simple_err(dbref obj, dbref loc, const char *errtype)
 {
     STARTLOG(LOG_PROBLEMS, "OBJ", "DAMAG");
     log_type_and_name(obj);
-    if (loc != NOTHING) 
+    if (loc != NOTHING)
     {
         log_text(" in ");
         log_type_and_name(loc);
@@ -370,6 +370,10 @@ dbref create_obj(dbref player, int objtype, const char *name, int cost)
     free_lbuf(pValidName);
     db[obj].cpu_time_used.Set100ns(0);
 
+    db[obj].tThrottleExpired.Set100ns(0);
+    s_ThAttrib(obj, 0);
+    s_ThMail(obj, 0);
+
     CLinearTimeAbsolute ltaNow;
     ltaNow.GetLocal();
     buff = ltaNow.ReturnDateString(7);
@@ -391,15 +395,13 @@ dbref create_obj(dbref player, int objtype, const char *name, int cost)
     return obj;
 }
 
-extern void stack_clr(dbref obj);
-
 /*
  * ---------------------------------------------------------------------------
  * * destroy_obj: Destroy an object.  Assumes it has already been removed from
  * * all lists and has no contents or exits.
  */
 
-void destroy_bad_obj(dbref obj)
+static void destroy_bad_obj(dbref obj)
 {
     if (!mudstate.bStandAlone)
     {
@@ -543,6 +545,8 @@ void destroy_obj(dbref obj)
     s_Pennies(obj, 0);
     s_Parent(obj, NOTHING);
     s_Zone(obj, NOTHING);
+
+    local_data_free(obj);
 }
 
 /*
@@ -574,11 +578,11 @@ void divest_object(dbref thing)
 {
     dbref curr, temp;
 
-    SAFE_DOLIST(curr, temp, Contents(thing)) 
+    SAFE_DOLIST(curr, temp, Contents(thing))
     {
         if (  !Controls(thing, curr)
            && Has_location(curr)
-           && Key(curr)) 
+           && Key(curr))
         {
             if (  !Good_obj(Home(curr))
                || Going(Home(curr)))
@@ -618,7 +622,7 @@ void empty_obj(dbref obj)
         }
         else
         {
-            s_Location(targ, NOTHING); 
+            s_Location(targ, NOTHING);
             s_Next(targ, NOTHING);
             if (  !Good_obj(Home(targ))
                || Going(Home(targ))
@@ -808,7 +812,7 @@ static void check_dead_refs(void)
         // Check the owner.
         //
         owner = Owner(i);
-        if (!Good_obj(owner)) 
+        if (!Good_obj(owner))
         {
             if (isPlayer(i))
             {
@@ -828,10 +832,10 @@ static void check_dead_refs(void)
                 halt_que(NOTHING, i);
             }
             s_Halted(i);
-        } 
-        else if (check_type & DBCK_FULL) 
+        }
+        else if (check_type & DBCK_FULL)
         {
-            if (Going(owner)) 
+            if (Going(owner))
             {
                 if (isPlayer(i))
                 {
@@ -851,7 +855,7 @@ static void check_dead_refs(void)
                     halt_que(NOTHING, i);
                 }
                 s_Halted(i);
-            } 
+            }
             else if (!OwnsOthers(owner))
             {
                 if (isPlayer(i))
@@ -873,9 +877,9 @@ static void check_dead_refs(void)
         // Check the parent
         //
         targ = Parent(i);
-        if (Good_obj(targ)) 
+        if (Good_obj(targ))
         {
-            if (Going(targ)) 
+            if (Going(targ))
             {
                 s_Parent(i, NOTHING);
                 if (!mudstate.bStandAlone)
@@ -893,8 +897,8 @@ static void check_dead_refs(void)
                         "is invalid.  Cleared.");
                 }
             }
-        } 
-        else if (targ != NOTHING) 
+        }
+        else if (targ != NOTHING)
         {
             Log_header_err(i, Location(i), targ, true,
                 "Parent", "is invalid.  Cleared.");
@@ -937,26 +941,26 @@ static void check_dead_refs(void)
         //
         fp = fwdlist_get(i);
         dirty = false;
-        if (fp) 
+        if (fp)
         {
-            for (j = 0; j < fp->count; j++) 
+            for (j = 0; j < fp->count; j++)
             {
                 targ = fp->data[j];
                 if (  Good_obj(targ)
-                   && Going(targ)) 
+                   && Going(targ))
                 {
                     fp->data[j] = NOTHING;
                     dirty = true;
-                } 
-                else if (  !Good_obj(targ) 
-                        && targ != NOTHING) 
+                }
+                else if (  !Good_obj(targ)
+                        && targ != NOTHING)
                 {
                     fp->data[j] = NOTHING;
                     dirty = true;
                 }
             }
         }
-        if (dirty) 
+        if (dirty)
         {
             str = alloc_lbuf("purge_going");
             (void)fwdlist_rewrite(fp, str);
@@ -965,17 +969,17 @@ static void check_dead_refs(void)
             free_lbuf(str);
         }
 
-        if (check_type & DBCK_FULL) 
+        if (check_type & DBCK_FULL)
         {
             // Check for wizards
             //
-            if (Wizard(i)) 
+            if (Wizard(i))
             {
-                if (isPlayer(i)) 
+                if (isPlayer(i))
                 {
                     Log_simple_err(i, NOTHING, "Player is a WIZARD.");
                 }
-                if (!Wizard(Owner(i))) 
+                if (!Wizard(Owner(i)))
                 {
                     Log_header_err(i, NOTHING, Owner(i), true,
                                "Owner", "of a WIZARD object is not a wizard");
@@ -1009,14 +1013,14 @@ static void check_dead_refs(void)
 
             // Check for self-referential Next().
             //
-            if (Next(i) == i) 
+            if (Next(i) == i)
             {
                 Log_simple_err(i, NOTHING,
                      "Next points to self.  Next cleared.");
                 s_Next(i, NOTHING);
             }
 
-            if (check_type & DBCK_FULL) 
+            if (check_type & DBCK_FULL)
             {
                 // Check wealth.
                 //
@@ -1611,7 +1615,7 @@ static void check_contents_chains(void)
            && Has_location(i))
         {
             Log_simple_err(i, Location(i), "Orphaned object, moved home.");
-            s_Location(i, NOTHING); 
+            s_Location(i, NOTHING);
             s_Next(i, NOTHING);
             if (  !Good_obj(Home(i))
                || Going(Home(i)))
@@ -1697,6 +1701,9 @@ static void check_floating(void)
 
 void do_dbck(dbref executor, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     check_type = key;
     check_dead_refs();
     check_exit_chains();
@@ -1709,6 +1716,11 @@ void do_dbck(dbref executor, dbref caller, dbref enactor, int key)
     }
     purge_going();
     make_freelist();
+
+    // Allow the local extensions to do data checks.
+    //
+    local_dbck();
+
     if (  !mudstate.bStandAlone
        && executor != NOTHING
        && !Quiet(executor))
