@@ -1,6 +1,6 @@
 // predicates.cpp
 //
-// $Id: predicates.cpp,v 1.47 2004/04/01 22:00:42 sdennis Exp $
+// $Id: predicates.cpp,v 1.77 2006/01/07 08:50:53 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -15,8 +15,9 @@
 #include "command.h"
 #include "interface.h"
 #include "powers.h"
-
-extern bool do_command(DESC *, char *);
+#ifdef REALITY_LVLS
+#include "levels.h"
+#endif /* REALITY_LVLS */
 
 char * DCL_CDECL tprintf(const char *fmt,...)
 {
@@ -32,8 +33,8 @@ void DCL_CDECL safe_tprintf_str(char *str, char **bp, const char *fmt,...)
 {
     va_list ap;
     va_start(ap, fmt);
-    int nAvailable = LBUF_SIZE - (*bp - str);
-    int len = mux_vsnprintf(*bp, nAvailable, fmt, ap);
+    size_t nAvailable = LBUF_SIZE - (*bp - str);
+    size_t len = mux_vsnprintf(*bp, (int)nAvailable, fmt, ap);
     va_end(ap);
     *bp += len;
 }
@@ -159,12 +160,20 @@ bool can_see(dbref player, dbref thing, bool can_see_loc)
     //
     if (can_see_loc)
     {
+#ifdef REALITY_LVLS
+       return ((!Dark(thing) && IsReal(player, thing)) ||
+#else
         return (!Dark(thing) ||
+#endif /* REALITY_LVLS */
             (mudconf.see_own_dark && MyopicExam(player, thing)));
     }
     else
     {
+#ifdef REALITY_LVLS
+        return ((Light(thing) && !Dark(thing) && IsReal(player, thing)) ||
+#else
         return ((Light(thing) && !Dark(thing)) ||
+#endif /* REALITY_LVLS */
             (mudconf.see_own_dark && MyopicExam(player, thing)));
     }
 }
@@ -249,7 +258,7 @@ bool payfor(dbref who, int cost)
 {
     if (  Wizard(who)
        || Wizard(Owner(who))
-       || Free_Money(who) 
+       || Free_Money(who)
        || Free_Money(Owner(who)))
     {
         return true;
@@ -383,7 +392,7 @@ char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid)
     safe_mb_str(pStripped, Buf, &pBuf);
     *pBuf = '\0';
 
-    int nBuf = pBuf - Buf;
+    size_t nBuf = pBuf - Buf;
     pBuf = Buf;
 
     bool bHaveDisplay = false;
@@ -480,7 +489,7 @@ bool ValidatePlayerName(const char *pName)
     {
         return false;
     }
-    unsigned int nName = strlen(pName);
+    size_t nName = strlen(pName);
 
     // Verify that name is not empty, but not too long, either.
     //
@@ -503,11 +512,11 @@ bool ValidatePlayerName(const char *pName)
     if (  mudstate.bStandAlone
        || mudconf.name_spaces)
     {
-        mux_PlayerNameSet[' '] = 1;
+        mux_PlayerNameSet[(unsigned char)' '] = 1;
     }
     else
     {
-        mux_PlayerNameSet[' '] = 0;
+        mux_PlayerNameSet[(unsigned char)' '] = 0;
     }
 
     // Only printable characters besides ARG_DELIMITER, AND_TOKEN,
@@ -603,10 +612,9 @@ void handle_ears(dbref thing, bool could_hear, bool can_hear)
     static const char *poss[5] =
     {"", "its", "her", "his", "their"};
 
-    if (  !could_hear
-       && can_hear)
+    if (could_hear != can_hear)
     {
-        buff = alloc_lbuf("handle_ears.grow");
+        buff = alloc_lbuf("handle_ears");
         strcpy(buff, Name(thing));
         if (isExit(thing))
         {
@@ -617,29 +625,22 @@ void handle_ears(dbref thing, bool could_hear, bool can_hear)
             *bp = '\0';
         }
         gender = get_gender(thing);
-        notify_check(thing, thing, tprintf("%s grow%s ears and can now hear.",
-            buff, (gender == 4) ? "" : "s"),
-            (MSG_ME | MSG_NBR | MSG_LOC | MSG_INV));
-        free_lbuf(buff);
-    }
-    else if (  could_hear
-            && !can_hear)
-    {
-        buff = alloc_lbuf("handle_ears.lose");
-        strcpy(buff, Name(thing));
-        if (isExit(thing))
+
+        if (can_hear)
         {
-            for (bp = buff; *bp && *bp != ';'; bp++)
-            {
-                ; // Nothing.
-            }
-            *bp = '\0';
+            notify_check(thing, thing,
+                         tprintf("%s grow%s ears and can now hear.",
+                                 buff, (gender == 4) ? "" : "s"),
+                         (MSG_ME | MSG_NBR | MSG_LOC | MSG_INV));
+        }
+        else
+        {
+            notify_check(thing, thing,
+                         tprintf("%s lose%s %s ears and become%s deaf.",
+                                 buff, (gender == 4) ? "" : "s",
+                                 poss[gender], (gender == 4) ? "" : "s"),
+                         (MSG_ME | MSG_NBR | MSG_LOC | MSG_INV));
         }
-        gender = get_gender(thing);
-        notify_check(thing, thing,
-            tprintf("%s lose%s %s ears and become%s deaf.", buff,
-            (gender == 4) ? "" : "s", poss[gender], (gender == 4) ? "" : "s"),
-            (MSG_ME | MSG_NBR | MSG_LOC | MSG_INV));
         free_lbuf(buff);
     }
 }
@@ -737,6 +738,8 @@ void do_if
     int   ncargs
 )
 {
+    UNUSED_PARAMETER(key);
+
     if (  !expr
        || nargs <= 0)
     {
@@ -770,6 +773,10 @@ void do_addcommand
     char *command
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     // Validate command name.
     //
     char *pName = NULL;
@@ -791,14 +798,14 @@ void do_addcommand
     // Validate object/attribute.
     //
     dbref thing;
-    ATTR *attr;
-    if (  !parse_attrib(player, command, &thing, &attr)
-       || !attr)
+    ATTR *pattr;
+    if (  !parse_attrib(player, command, &thing, &pattr)
+       || !pattr)
     {
         notify(player, "No such attribute.");
         return;
     }
-    if (!See_attr(player, thing, attr))
+    if (!See_attr(player, thing, pattr))
     {
         notify(player, NOPERM_MESSAGE);
         return;
@@ -818,7 +825,7 @@ void do_addcommand
         for (nextp = old->addent; nextp != NULL; nextp = nextp->next)
         {
             if (  nextp->thing == thing
-               && nextp->atr == attr->number)
+               && nextp->atr == pattr->number)
             {
                 notify(player, tprintf("%s already added.", pName));
                 return;
@@ -828,9 +835,9 @@ void do_addcommand
         // Otherwise, add another (thing,atr) to the list.
         //
         add = (ADDENT *)MEMALLOC(sizeof(ADDENT));
-        (void)ISOUTOFMEMORY(add);
+        ISOUTOFMEMORY(add);
         add->thing = thing;
-        add->atr = attr->number;
+        add->atr = pattr->number;
         add->name = StringClone(pName);
         add->next = old->addent;
         old->addent = add;
@@ -846,7 +853,7 @@ void do_addcommand
         }
 
         cmd = (CMDENT *)MEMALLOC(sizeof(CMDENT));
-        (void)ISOUTOFMEMORY(cmd);
+        ISOUTOFMEMORY(cmd);
         cmd->cmdname = StringClone(pName);
         cmd->switches = NULL;
         cmd->perms = 0;
@@ -862,9 +869,9 @@ void do_addcommand
         }
         cmd->hookmask = 0;
         add = (ADDENT *)MEMALLOC(sizeof(ADDENT));
-        (void)ISOUTOFMEMORY(add);
+        ISOUTOFMEMORY(add);
         add->thing = thing;
-        add->atr = attr->number;
+        add->atr = pattr->number;
         add->name = StringClone(pName);
         add->next = NULL;
         cmd->addent = add;
@@ -894,6 +901,10 @@ void do_addcommand
 void do_listcommands(dbref player, dbref caller, dbref enactor, int key,
                      char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     CMDENT *old;
     ADDENT *nextp;
     bool didit = false;
@@ -976,6 +987,11 @@ void do_delcommand
     char *command
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     if (!*name)
     {
         notify(player, "Sorry.");
@@ -984,21 +1000,21 @@ void do_delcommand
 
     dbref thing = NOTHING;
     int atr = NOTHING;
-    ATTR *attr;
+    ATTR *pattr;
     if (*command)
     {
-        if (  !parse_attrib(player, command, &thing, &attr)
-           || !attr)
+        if (  !parse_attrib(player, command, &thing, &pattr)
+           || !pattr)
         {
             notify(player, "No such attribute.");
             return;
         }
-        if (!See_attr(player, thing, attr))
+        if (!See_attr(player, thing, pattr))
         {
             notify(player, NOPERM_MESSAGE);
             return;
         }
-        atr = attr->number;
+        atr = pattr->number;
     }
 
     // Let's make this case insensitive...
@@ -1014,7 +1030,7 @@ void do_delcommand
        && (old->callseq & CS_ADDED))
     {
         char *p__Name = tprintf("__%s", name);
-        unsigned int n__Name = strlen(p__Name);
+        size_t n__Name = strlen(p__Name);
 
         if (command[0] == '\0')
         {
@@ -1132,18 +1148,30 @@ void do_delcommand
 
 void handle_prog(DESC *d, char *message)
 {
-
     // Allow the player to pipe a command while in interactive mode.
     //
     if (*message == '|')
     {
         do_command(d, message + 1);
 
-        // Use telnet protocol's GOAHEAD command to show prompt
-        //
         if (d->program_data != NULL)
         {
-            queue_string(d, tprintf("%s>%s \377\371", ANSI_HILITE, ANSI_NORMAL));
+            queue_string(d, tprintf("%s>%s ", ANSI_HILITE, ANSI_NORMAL));
+
+            if (OPTION_YES == UsState(d, TELNET_EOR))
+            {
+                // Use telnet protocol's EOR command to show prompt.
+                //
+                const char aEOR[2] = { NVT_IAC, NVT_EOR };
+                queue_write_LEN(d, aEOR, sizeof(aEOR));
+            }
+            else if (OPTION_YES != UsState(d, TELNET_SGA))
+            {
+                // Use telnet protocol's GOAHEAD command to show prompt.
+                //
+                const char aGoAhead[2] = { NVT_IAC, NVT_GA };
+                queue_write_LEN(d, aGoAhead, sizeof(aGoAhead));
+            }
         }
         return;
     }
@@ -1184,6 +1212,10 @@ void handle_prog(DESC *d, char *message)
 
 void do_quitprog(dbref player, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref doer;
 
     if (*name)
@@ -1269,6 +1301,11 @@ void do_prog
     char *command
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     if (  !name
        || !*name)
     {
@@ -1313,13 +1350,25 @@ void do_prog
     if (ap)
     {
         dbref aowner;
-        int aflags;
-        char *pBuffer = atr_get(thing, ap->number, &aowner, &aflags);
-        if (*pBuffer)
+        int   aflags;
+        int   lev;
+        dbref parent;
+        char *pBuffer = NULL;
+        bool bFound = false;
+        ITER_PARENTS(thing, parent, lev)
+        {
+            pBuffer = atr_get(parent, ap->number, &aowner, &aflags);
+            if (pBuffer[0])
+            {
+                bFound = true;
+                break;
+            }
+            free_lbuf(pBuffer);
+        }
+        if (bFound)
         {
             if (  (   God(player)
                   || !God(thing))
-               && ap
                && See_attr(player, thing, ap))
             {
                 atr_add_raw(doer, A_PROGCMD, pBuffer);
@@ -1357,7 +1406,7 @@ void do_prog
     }
 
     PROG *program = (PROG *)MEMALLOC(sizeof(PROG));
-    (void)ISOUTOFMEMORY(program);
+    ISOUTOFMEMORY(program);
     program->wait_enactor = player;
     for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
@@ -1371,45 +1420,76 @@ void do_prog
     {
         d->program_data = program;
 
-        // Use telnet protocol's GOAHEAD command to show prompt.
-        //
-        queue_string(d, tprintf("%s>%s \377\371", ANSI_HILITE, ANSI_NORMAL));
+        queue_string(d, tprintf("%s>%s ", ANSI_HILITE, ANSI_NORMAL));
+
+        if (OPTION_YES == UsState(d, TELNET_EOR))
+        {
+            // Use telnet protocol's EOR command to show prompt.
+            //
+            const char aEOR[2] = { NVT_IAC, NVT_EOR };
+            queue_write_LEN(d, aEOR, sizeof(aEOR));
+        }
+        else if (OPTION_YES != UsState(d, TELNET_SGA))
+        {
+            // Use telnet protocol's GOAHEAD command to show prompt.
+            //
+            const char aGoAhead[2] = { NVT_IAC, NVT_GA };
+            queue_write_LEN(d, aGoAhead, sizeof(aGoAhead));
+        }
     }
 }
 
 /* ---------------------------------------------------------------------------
  * do_restart: Restarts the game.
  */
-void do_restart(dbref player, dbref caller, dbref enactor, int key)
+void do_restart(dbref executor, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
+    if (!Can_SiteAdmin(executor))
+    {
+        notify(executor, NOPERM_MESSAGE);
+        return;
+    }
+
     bool bDenied = false;
 #ifndef WIN32
     if (mudstate.dumping)
     {
-        notify(player, "Dumping. Please try again later.");
+        notify(executor, "Dumping. Please try again later.");
         bDenied = true;
     }
 #endif // !WIN32
+
+
     if (!mudstate.bCanRestart)
     {
-        notify(player, "Server just started. Please try again in a few seconds.");
+        notify(executor, "Server just started. Please try again in a few seconds.");
         bDenied = true;
     }
     if (bDenied)
     {
         STARTLOG(LOG_ALWAYS, "WIZ", "RSTRT");
         log_text("Restart requested but not executed by ");
-        log_name(player);
+        log_name(executor);
         ENDLOG;
         return;
     }
 
-    raw_broadcast(0, "GAME: Restart by %s, please wait.", Name(Owner(player)));
+    raw_broadcast(0, "GAME: Restart by %s, please wait.", Name(Owner(executor)));
     STARTLOG(LOG_ALWAYS, "WIZ", "RSTRT");
     log_text("Restart by ");
-    log_name(player);
+    log_name(executor);
     ENDLOG;
 
+    local_presync_database();
+
+#ifndef MEMORY_BASED
+    al_store();
+#endif
+    pcache_sync();
     dump_database_internal(DUMP_I_RESTART);
     SYNC;
     CLOSE;
@@ -1429,9 +1509,9 @@ void do_restart(dbref player, dbref caller, dbref enactor, int key)
     Log.StopLogging();
 
 #ifdef GAME_DOOFERMUX
-    execl("bin/netmux", mudconf.mud_name, "-c", mudconf.config_file, NULL);
+    execl("bin/netmux", mudconf.mud_name, "-c", mudconf.config_file, "-p", mudconf.pid_file, NULL);
 #else
-    execl("bin/netmux", "netmux", "-c", mudconf.config_file, NULL);
+    execl("bin/netmux", "netmux", "-c", mudconf.config_file, "-p", mudconf.pid_file, NULL);
 #endif // GAME_DOOFERMUX
 #endif // !WIN32
 }
@@ -1446,6 +1526,10 @@ void do_restart(dbref player, dbref caller, dbref enactor, int key)
 
 void do_backup(dbref player, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     notify(player, "This feature is not yet available on Win32-hosted MUX.");
 }
 
@@ -1466,19 +1550,19 @@ void do_backup(dbref player, dbref caller, dbref enactor, int key)
     log_name(player);
     ENDLOG;
 
-#if 1
-    // Invoking _backupflat.sh without an argument prompts the backup script
-    // to use dbconvert itself.
-    //
-    dump_database_internal(DUMP_I_NORMAL);
-    system(tprintf("./_backupflat.sh 1>&2"));
-#else // 1
+#ifdef MEMORY_BASED
     // Invoking _backupflat.sh with an argument prompts the backup script
     // to use it as the flatfile.
     //
     dump_database_internal(DUMP_I_FLAT);
     system(tprintf("./_backupflat.sh %s.FLAT 1>&2", mudconf.indb));
-#endif // 1
+#else // MEMORY_BASED
+    // Invoking _backupflat.sh without an argument prompts the backup script
+    // to use dbconvert itself.
+    //
+    dump_database_internal(DUMP_I_NORMAL);
+    system(tprintf("./_backupflat.sh 1>&2"));
+#endif // MEMORY_BASED
     raw_broadcast(0, "GAME: Backup finished.");
 }
 #endif // WIN32
@@ -1489,6 +1573,10 @@ void do_backup(dbref player, dbref caller, dbref enactor, int key)
 
 void do_comment(dbref player, dbref caller, dbref enactor, int key)
 {
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
 }
 
 static dbref promote_dflt(dbref old, dbref new0)
@@ -1604,8 +1692,8 @@ dbref match_possessed(dbref player, dbref thing, char *target, dbref dflt, bool
         // If we don't control it and it is either dark or opaque, skip past.
         //
         bool control = Controls(player, result1);
-        if (  (  Dark(result1) 
-              || Opaque(result1)) 
+        if (  (  Dark(result1)
+              || Opaque(result1))
            && !control)
         {
             dflt = promote_dflt(dflt, NOTHING);
@@ -1699,15 +1787,18 @@ void parse_range(char **name, dbref *low_bound, dbref *high_bound)
 
 bool parse_thing_slash(dbref player, char *thing, char **after, dbref *it)
 {
-    char *str;
-
     // Get name up to '/'.
     //
-    for (str = thing; *str && (*str != '/'); str++) ;
+    char *str = thing;
+    while (  *str != '\0'
+          && *str != '/')
+    {
+        str++;
+    }
 
     // If no '/' in string, return failure.
     //
-    if (!*str)
+    if (*str == '\0')
     {
         *after = NULL;
         *it = NOTHING;
@@ -1724,11 +1815,9 @@ bool parse_thing_slash(dbref player, char *thing, char **after, dbref *it)
 
     // Return status of search.
     //
-    return (Good_obj(*it));
+    return Good_obj(*it);
 }
 
-extern NAMETAB lock_sw[];
-
 bool get_obj_and_lock(dbref player, char *what, dbref *it, ATTR **attr, char *errmsg, char **bufc)
 {
     char *str, *tbuf;
@@ -1740,8 +1829,7 @@ bool get_obj_and_lock(dbref player, char *what, dbref *it, ATTR **attr, char *er
     {
         // <obj>/<lock> syntax, use the named lock.
         //
-        anum = search_nametab(player, lock_sw, str);
-        if (anum < 0)
+        if (!search_nametab(player, lock_sw, str, &anum))
         {
             free_lbuf(tbuf);
             safe_str("#-1 LOCK NOT FOUND", errmsg, bufc);
@@ -1873,6 +1961,9 @@ bool bCanSetAttr(dbref executor, dbref target, ATTR *tattr)
     dbref aowner;
     int aflags;
     if (  (tattr->flags & mDeny)
+#ifdef FIRANMUX
+       || Immutable(target)
+#endif
        || (  atr_get_info(target, tattr->number, &aowner, &aflags)
           && (aflags & mDeny)))
     {
@@ -2002,7 +2093,7 @@ bool locatable(dbref player, dbref it, dbref enactor)
     if (  Examinable(player, it)
        || Find_Unfindable(player)
        || loc_it == player
-       || (  loc_it != NOTHING 
+       || (  loc_it != NOTHING
           && (  Examinable(player, loc_it)
              || loc_it == where_is(player)))
        || Wizard(enactor)
@@ -2051,6 +2142,12 @@ bool nearby(dbref player, dbref thing)
     {
         return false;
     }
+    if (  Can_Hide(thing)
+       && Hidden(thing)
+       && !See_Hidden(player))
+    {
+        return false;
+    }
     dbref thing_loc = where_is(thing);
     if (thing_loc == player)
     {
@@ -2083,6 +2180,14 @@ bool exit_visible(dbref exit, dbref player, int key)
     }
 #endif // WOD_REALMS
 
+#ifdef REALITY_LVLS
+    if (!mudstate.bStandAlone)
+    {
+    if (!IsReal(player, exit))
+       return 0;
+    }
+#endif /* REALITY_LVLS */
+
     // Exam exit's location
     //
     if (  (key & VE_LOC_XAM)
@@ -2154,6 +2259,11 @@ bool exit_displayable(dbref exit, dbref player, int key)
 void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
             const char *odef, int awhat, char *args[], int nargs)
 {
+    if (MuxAlarm.bAlarmed)
+    {
+        return;
+    }
+
     char *d, *buff, *act, *charges, *bp, *str;
     dbref loc, aowner;
     int num, aflags;
@@ -2235,13 +2345,21 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
             *bp = '\0';
             if (*buff)
             {
+#ifdef REALITY_LVLS
+                notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), buff));
+#else
                 notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), buff));
+#endif /* REALITY_LVLS */
             }
             free_lbuf(buff);
         }
         else if (odef)
         {
+#ifdef REALITY_LVLS
+            notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+#else
             notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+#endif /* REALITY_LVLS */
         }
         free_lbuf(d);
     }
@@ -2250,7 +2368,11 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
        && Has_location(player)
        && Good_obj(loc = Location(player)))
     {
+#ifdef REALITY_LVLS
+        notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+#else
         notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+#endif /* REALITY_LVLS */
     }
 
     // If we preserved the state of the global registers, restore them.
@@ -2264,7 +2386,11 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
 
     // do the action attribute.
     //
+#ifdef REALITY_LVLS
+    if (awhat > 0 && IsReal(thing, player))
+#else
     if (awhat > 0)
+#endif /* REALITY_LVLS */
     {
         if (*(act = atr_pget(thing, awhat, &aowner, &aflags)))
         {
@@ -2308,6 +2434,9 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
 void do_verb(dbref executor, dbref caller, dbref enactor, int key,
              char *victim_str, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(key);
+
     // Look for the victim.
     //
     if (  !victim_str
@@ -2436,7 +2565,7 @@ void do_verb(dbref executor, dbref caller, dbref enactor, int key,
         }
         if (  !ap
            || !bCanReadAttr(executor, victim, ap, false)
-           || (  ap->number == A_DESC 
+           || (  ap->number == A_DESC
               && !mudconf.read_rem_desc
               && !Examinable(executor, victim)
               && !nearby(executor, victim)))
@@ -2478,7 +2607,7 @@ void do_verb(dbref executor, dbref caller, dbref enactor, int key,
 // --------------------------------------------------------------------------
 // OutOfMemory: handle an out of memory condition.
 //
-bool OutOfMemory(const char *SourceFile, unsigned int LineNo)
+void OutOfMemory(const char *SourceFile, unsigned int LineNo)
 {
     Log.tinyprintf("%s(%u): Out of memory." ENDLINE, SourceFile, LineNo);
     Log.Flush();
@@ -2491,7 +2620,6 @@ bool OutOfMemory(const char *SourceFile, unsigned int LineNo)
     {
         abort();
     }
-    return true;
 }
 
 // --------------------------------------------------------------------------
