@@ -1,6 +1,6 @@
 // set.cpp -- Commands which set parameters.
 //
-// $Id: set.cpp,v 1.20 2004/04/01 22:00:42 sdennis Exp $
+// $Id: set.cpp,v 1.30 2006/01/07 08:23:38 sdennis Exp $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -9,10 +9,9 @@
 
 #include "ansi.h"
 #include "attrs.h"
+#include "command.h"
 #include "powers.h"
 
-extern NAMETAB indiv_attraccess_nametab[];
-
 void set_modified(dbref thing)
 {
     CLinearTimeAbsolute ltaNow;
@@ -60,6 +59,11 @@ void do_chzone
     char *newobj
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     if (!mudconf.have_zones)
     {
         notify(executor, "Zones disabled.");
@@ -155,6 +159,10 @@ void do_name
     char *newname
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref thing = match_controlled(executor, name);
     if (thing == NOTHING)
     {
@@ -262,6 +270,11 @@ void do_alias
     char *alias
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     dbref thing = match_controlled(executor, name);
     if (thing == NOTHING)
     {
@@ -369,6 +382,11 @@ void do_forwardlist
     char *newlist
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     dbref thing = match_controlled(executor, target);
     if (thing == NOTHING)
     {
@@ -424,6 +442,10 @@ void do_lock
     char *keytext
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     dbref thing;
     ATTR *ap;
 
@@ -507,6 +529,9 @@ void do_lock
 
 void do_unlock(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref thing;
     ATTR *ap;
 
@@ -565,6 +590,10 @@ void do_unlock(dbref executor, dbref caller, dbref enactor, int key, char *name)
 
 void do_unlink(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref exit;
 
     init_match(executor, name, TYPE_EXIT);
@@ -637,6 +666,11 @@ void do_chown
     char *newown
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     dbref nOwnerOrig, nOwnerNew, thing;
     bool bDoit;
     ATTR *ap;
@@ -878,10 +912,10 @@ static void set_attr_internal(dbref player, dbref thing, int attrnum, char *attr
 {
     dbref aowner;
     int aflags;
-    ATTR *attr = atr_num(attrnum);
+    ATTR *pattr = atr_num(attrnum);
     atr_pget_info(thing, attrnum, &aowner, &aflags);
-    if (  attr
-       && bCanSetAttr(player, thing, attr)) 
+    if (  pattr
+       && bCanSetAttr(player, thing, pattr))
     {
         bool could_hear = Hearer(thing);
         atr_add(thing, attrnum, attrtext, Owner(player), aflags);
@@ -892,8 +926,8 @@ static void set_attr_internal(dbref player, dbref thing, int attrnum, char *attr
         {
             notify_quiet(player, "Set.");
         }
-    } 
-    else 
+    }
+    else
     {
         notify_quiet(player, NOPERM_MESSAGE);
     }
@@ -910,21 +944,25 @@ void do_set
     char *flagname
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     dbref thing, aowner;
     int aflags;
-    ATTR *attr;
+    ATTR *pattr;
 
     // See if we have the <obj>/<attr> form, which is how you set
     // attribute flags.
     //
-    if (parse_attrib(executor, name, &thing, &attr))
+    if (parse_attrib(executor, name, &thing, &pattr))
     {
-        if (  attr
-           && See_attr(executor, thing, attr))
+        if (  pattr
+           && See_attr(executor, thing, pattr))
         {
             // You must specify a flag name.
             //
-            if (  !flagname 
+            if (  !flagname
                || flagname[0] == '\0')
             {
                 notify_quiet(executor, "I don't know what you want to set!");
@@ -942,8 +980,8 @@ void do_set
 
             // Make sure player specified a valid attribute flag.
             //
-            int flagvalue = search_nametab(executor, indiv_attraccess_nametab, flagname);
-            if (flagvalue < 0)
+            int flagvalue;
+            if (!search_nametab(executor, indiv_attraccess_nametab, flagname, &flagvalue))
             {
                 notify_quiet(executor, "You can't set that!");
                 return;
@@ -951,7 +989,7 @@ void do_set
 
             // Make sure the object has the attribute present.
             //
-            if (!atr_get_info(thing, attr->number, &aowner, &aflags))
+            if (!atr_get_info(thing, pattr->number, &aowner, &aflags))
             {
                 notify_quiet(executor, "Attribute not present on object.");
                 return;
@@ -959,7 +997,7 @@ void do_set
 
             // Make sure we can write to the attribute.
             //
-            if (!bCanSetAttr(executor, thing, attr))
+            if (!bCanSetAttr(executor, thing, pattr))
             {
                 notify_quiet(executor, NOPERM_MESSAGE);
                 return;
@@ -976,11 +1014,13 @@ void do_set
                 aflags |= flagvalue;
             }
             bool could_hear = Hearer(thing);
-            atr_set_flags(thing, attr->number, aflags);
+
+            atr_set_flags(thing, pattr->number, aflags);
 
             // Tell the player about it.
             //
             handle_ears(thing, could_hear, Hearer(thing));
+
             if (  !(key & SET_QUIET)
                && !Quiet(executor)
                && !Quiet(thing))
@@ -1017,19 +1057,19 @@ void do_set
     if (*p)
     {
         *p++ = 0;
-        int atr = mkattr(flagname);
+        int atr = mkattr(executor, flagname);
         if (atr <= 0)
         {
             notify_quiet(executor, "Couldn't create attribute.");
             return;
         }
-        attr = atr_num(atr);
-        if (!attr)
+        pattr = atr_num(atr);
+        if (!pattr)
         {
             notify_quiet(executor, NOPERM_MESSAGE);
             return;
         }
-        if (!bCanSetAttr(executor, thing, attr))
+        if (!bCanSetAttr(executor, thing, pattr))
         {
             notify_quiet(executor, NOPERM_MESSAGE);
             return;
@@ -1040,21 +1080,21 @@ void do_set
         //
         if (*p == '_')
         {
-            ATTR *attr2;
+            ATTR *pattr2;
             dbref thing2;
 
             strcpy(buff, p + 1);
-            if (!( parse_attrib(executor, p + 1, &thing2, &attr2)
-                && attr2))
+            if (!( parse_attrib(executor, p + 1, &thing2, &pattr2)
+                && pattr2))
             {
                 notify_quiet(executor, "No match.");
                 free_lbuf(buff);
                 return;
             }
             p = buff;
-            atr_pget_str(buff, thing2, attr2->number, &aowner, &aflags);
+            atr_pget_str(buff, thing2, pattr2->number, &aowner, &aflags);
 
-            if (!See_attr(executor, thing2, attr2))
+            if (!See_attr(executor, thing2, pattr2))
             {
                 notify_quiet(executor, NOPERM_MESSAGE);
                 free_lbuf(buff);
@@ -1085,6 +1125,10 @@ void do_power
     char *flag
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     if (  !flag
        || !*flag)
     {
@@ -1113,6 +1157,10 @@ void do_setattr
     char *attrtext
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     init_match(executor, name, NOTYPE);
     match_everything(MAT_EXIT_PARENTS);
     dbref thing = noisy_match_result();
@@ -1127,6 +1175,8 @@ void do_setattr
 void do_cpattr(dbref executor, dbref caller, dbref enactor, int key,
                char *oldpair, char *newpair[], int nargs)
 {
+    UNUSED_PARAMETER(key);
+
     int i;
     char *oldthing, *oldattr, *newthing, *newattr;
 
@@ -1180,6 +1230,10 @@ void do_cpattr(dbref executor, dbref caller, dbref enactor, int key,
 void do_mvattr(dbref executor, dbref caller, dbref enactor, int key,
                char *what, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     // Make sure we have something to do.
     //
     if (nargs < 2)
@@ -1227,7 +1281,7 @@ void do_mvattr(dbref executor, dbref caller, dbref enactor, int key,
     int  nCopied = 0;
     for (int i = 1; i < nargs; i++)
     {
-        int anum = mkattr(args[i]);
+        int anum = mkattr(executor, args[i]);
         if (anum <= 0)
         {
             notify_quiet(executor, tprintf("%s: That's not a good name for an attribute.", args[i]));
@@ -1326,13 +1380,14 @@ bool parse_attrib(dbref player, char *str, dbref *thing, ATTR **attr)
     //
     char *buff = alloc_lbuf("parse_attrib");
     strcpy(buff, str);
-    bool retval = parse_thing_slash(player, buff, &str, thing);
+    char *AttrName;
+    bool retval = parse_thing_slash(player, buff, &AttrName, thing);
 
     // Get the named attribute from the object if we can.
     //
     if (retval)
     {
-        tattr = atr_str(str);
+        tattr = atr_str(AttrName);
     }
 
     free_lbuf(buff);
@@ -1340,10 +1395,9 @@ bool parse_attrib(dbref player, char *str, dbref *thing, ATTR **attr)
     return retval;
 }
 
-
 static void find_wild_attrs(dbref player, dbref thing, char *str, bool check_exclude, bool hash_insert, bool get_locks)
 {
-    ATTR *attr;
+    ATTR *pattr;
     char *as;
     dbref aowner;
     int ca, ok, aflags;
@@ -1353,17 +1407,17 @@ static void find_wild_attrs(dbref player, dbref thing, char *str, bool check_exc
     atr_push();
     for (ca = atr_head(thing, &as); ca; ca = atr_next(&as))
     {
-        attr = atr_num(ca);
+        pattr = atr_num(ca);
 
         // Discard bad attributes and ones we've seen before.
         //
-        if (!attr)
+        if (!pattr)
         {
             continue;
         }
 
         if (  check_exclude
-           && (  (attr->flags & AF_PRIVATE)
+           && (  (pattr->flags & AF_PRIVATE)
               || hashfindLEN(&ca, sizeof(ca), &mudstate.parent_htab)))
         {
             continue;
@@ -1381,28 +1435,29 @@ static void find_wild_attrs(dbref player, dbref thing, char *str, bool check_exc
 
         if (get_locks)
         {
-            ok = bCanReadAttr(player, thing, attr, false);
+            ok = bCanReadAttr(player, thing, pattr, false);
         }
         else
         {
-            ok = See_attr(player, thing, attr);
+            ok = See_attr(player, thing, pattr);
         }
 
         mudstate.wild_invk_ctr = 0;
         if (  ok
-           && quick_wild(str, attr->name))
+           && quick_wild(str, pattr->name))
         {
             olist_add(ca);
             if (hash_insert)
             {
-                hashaddLEN(&ca, sizeof(ca), attr, &mudstate.parent_htab);
+                hashaddLEN(&ca, sizeof(ca), pattr, &mudstate.parent_htab);
             }
         }
     }
     atr_pop();
 }
 
-bool parse_attrib_wild(dbref player, char *str, dbref *thing, bool check_parents, bool get_locks, bool df_star)
+bool parse_attrib_wild(dbref player, char *str, dbref *thing,
+    bool check_parents, bool get_locks, bool df_star)
 {
     if (!str)
     {
@@ -1514,7 +1569,7 @@ void edit_string(char *src, char **dst, char *from, char *to)
     }
 }
 
-void edit_string_ansi(char *src, char **dst, char **returnstr, char *from, char *to)
+static void edit_string_ansi(char *src, char **dst, char **returnstr, char *from, char *to)
 {
     char *cp, *rp;
 
@@ -1583,8 +1638,12 @@ void edit_string_ansi(char *src, char **dst, char **returnstr, char *from, char
 void do_edit(dbref executor, dbref caller, dbref enactor, int key, char *it,
              char *args[], int nargs)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref thing, aowner;
-    int attr, aflags;
+    int atr, aflags;
     bool bGotOne;
     char *from, *to, *result, *returnstr, *atext;
     ATTR *ap;
@@ -1617,9 +1676,9 @@ void do_edit(dbref executor, dbref caller, dbref enactor, int key, char *it,
     atext = alloc_lbuf("do_edit.atext");
     bool could_hear = Hearer(thing);
 
-    for (attr = olist_first(); attr != NOTHING; attr = olist_next())
+    for (atr = olist_first(); atr != NOTHING; atr = olist_next())
     {
-        ap = atr_num(attr);
+        ap = atr_num(atr);
         if (ap)
         {
             // Get the attr and make sure we can modify it.
@@ -1667,6 +1726,10 @@ void do_edit(dbref executor, dbref caller, dbref enactor, int key, char *it,
 
 void do_wipe(dbref executor, dbref caller, dbref enactor, int key, char *it)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref thing;
 
     olist_push();
@@ -1686,13 +1749,13 @@ void do_wipe(dbref executor, dbref caller, dbref enactor, int key, char *it)
 
     // Iterate through matching attributes, zapping the writable ones
     //
-    int attr;
+    int atr;
     ATTR *ap;
     bool bGotOne = false, could_hear = Hearer(thing);
 
-    for (attr = olist_first(); attr != NOTHING; attr = olist_next())
+    for (atr = olist_first(); atr != NOTHING; atr = olist_next())
     {
-        ap = atr_num(attr);
+        ap = atr_num(atr);
         if (ap)
         {
             // Get the attr and make sure we can modify it.
@@ -1727,11 +1790,14 @@ void do_wipe(dbref executor, dbref caller, dbref enactor, int key, char *it)
 void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
                 char *object, char *argv[], int nargs)
 {
-    dbref thing;
-    ATTR *attr;
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
 
-    if (!( parse_attrib(executor, object, &thing, &attr)
-        && attr))
+    dbref thing;
+    ATTR *pattr;
+
+    if (!( parse_attrib(executor, object, &thing, &pattr)
+        && pattr))
     {
         notify_quiet(executor, "No match.");
         return;
@@ -1741,7 +1807,7 @@ void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
         notify_quiet(executor, NOPERM_MESSAGE);
         return;
     }
-    did_it(executor, thing, 0, NULL, 0, NULL, attr->number, argv, nargs);
+    did_it(executor, thing, 0, NULL, 0, NULL, pattr->number, argv, nargs);
 
     // TODO: Be more descriptive as to what was triggered?
     //
@@ -1754,6 +1820,10 @@ void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
 
 void do_use(dbref executor, dbref caller, dbref enactor, int key, char *object)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     char *df_use, *df_ouse, *temp;
     dbref thing, aowner;
     int aflags;
@@ -1830,6 +1900,9 @@ void do_setvattr
     char *arg2
 )
 {
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     char *s;
     int anum;
 
@@ -1853,7 +1926,7 @@ void do_setvattr
 
     // Get or make attribute
     //
-    anum = mkattr(arg1);
+    anum = mkattr(executor, arg1);
 
     if (anum <= 0)
     {
