@@ -1,6 +1,6 @@
 // speech.cpp -- Commands which involve speaking.
 //
-// $Id: speech.cpp,v 1.13 2004/04/08 04:36:09 sdennis Exp $
+// $Id: speech.cpp,v 1.22 2006/01/07 08:10:56 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -9,8 +9,12 @@
 #include "externs.h"
 
 #include "attrs.h"
+#include "command.h"
 #include "interface.h"
 #include "powers.h"
+#ifdef REALITY_LVLS
+#include "levels.h"
+#endif
 
 char *modSpeech(dbref player, char *message, bool bWhich, char *command)
 {
@@ -19,7 +23,8 @@ char *modSpeech(dbref player, char *message, bool bWhich, char *command)
     char *mod = atr_get(player, bWhich ? A_SPEECHMOD : A_SAYSTRING,
         &aowner, &aflags);
 
-    if (mod[0] == '\0')
+    if (  mod[0] == '\0'
+       || MuxAlarm.bAlarmed)
     {
         free_lbuf(mod);
         return NULL;
@@ -31,7 +36,7 @@ char *modSpeech(dbref player, char *message, bool bWhich, char *command)
     char *args[2];
     args[0] = message;
     args[1] = command;
-    mux_exec(new_message, &t_ptr, player, player, player, 
+    mux_exec(new_message, &t_ptr, player, player, player,
         EV_FCHECK | EV_EVAL | EV_TOP, &mod, args, 2);
     free_lbuf(mod_orig);
     return new_message;
@@ -50,7 +55,7 @@ static int idle_timeout_val(dbref player)
     return idle_timeout;
 }
 
-bool sp_ok(dbref player)
+static bool sp_ok(dbref player)
 {
     if (  Gagged(player)
        && !Wizard(player))
@@ -78,7 +83,7 @@ bool sp_ok(dbref player)
     return true;
 }
 
-void wall_broadcast(int target, dbref player, char *message)
+static void wall_broadcast(int target, dbref player, char *message)
 {
     DESC *d;
     DESC_ITER_CONN(d)
@@ -131,6 +136,8 @@ static const char *admin_msg = "Admin: ";
 void do_think(dbref executor, dbref caller, dbref enactor, int key,
     char *message)
 {
+    UNUSED_PARAMETER(key);
+
     char *str, *buf, *bp;
 
     buf = bp = alloc_lbuf("do_think");
@@ -144,11 +151,14 @@ void do_think(dbref executor, dbref caller, dbref enactor, int key,
 
 void do_say(dbref executor, dbref caller, dbref enactor, int key, char *message)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     // Make sure speaker is somewhere if speaking in a place
     //
     dbref loc = where_is(executor);
     if ( !(  Good_obj(loc)
-            && sp_ok(executor)))
+            & sp_ok(executor)))
     {
         return;
     }
@@ -237,26 +247,38 @@ void do_say(dbref executor, dbref caller, dbref enactor, int key, char *message)
         {
             notify_saypose(executor, tprintf("%s %s \"%s\"",
                 Moniker(executor), saystring, message));
-            notify_except(loc, executor, executor, tprintf("%s %s \"%s\"",
-                Moniker(executor), saystring, message), MSG_SAYPOSE);
+#ifdef REALITY_LVLS
+            notify_except_rlevel(loc, executor, executor, tprintf("%s %s \"%s\"", Moniker(executor), saystring, message), MSG_SAYPOSE);
+#else
+            notify_except(loc, executor, executor, tprintf("%s %s \"%s\"", Moniker(executor), saystring, message), MSG_SAYPOSE);
+#endif
             free_lbuf(saystring);
         }
         else
         {
             notify_saypose(executor, tprintf("You say, \"%s\"", message));
-            notify_except(loc, executor, executor, tprintf("%s says, \"%s\"",
-                Moniker(executor), message), MSG_SAYPOSE);
+#ifdef REALITY_LVLS
+            notify_except_rlevel(loc, executor, executor, tprintf("%s says, \"%s\"", Moniker(executor), message), MSG_SAYPOSE);
+#else
+            notify_except(loc, executor, executor, tprintf("%s says, \"%s\"", Moniker(executor), message), MSG_SAYPOSE);
+#endif
         }
         break;
 
     case SAY_POSE:
-        notify_all_from_inside_saypose(loc, executor, tprintf("%s %s",
-            Moniker(executor), message));
+#ifdef REALITY_LVLS
+        notify_except_rlevel(loc, executor, -1, tprintf("%s %s", Moniker(executor), message), MSG_SAYPOSE);
+#else
+        notify_all_from_inside_saypose(loc, executor, tprintf("%s %s", Moniker(executor), message));
+#endif
         break;
 
     case SAY_POSE_NOSPC:
-        notify_all_from_inside_saypose(loc, executor, tprintf("%s%s",
-            Moniker(executor), message));
+#ifdef REALITY_LVLS
+        notify_except_rlevel(loc, executor, -1, tprintf("%s%s", Moniker(executor), message), MSG_SAYPOSE);
+#else
+        notify_all_from_inside_saypose(loc, executor, tprintf("%s%s", Moniker(executor), message));
+#endif
         break;
 
     case SAY_EMIT:
@@ -270,7 +292,12 @@ void do_say(dbref executor, dbref caller, dbref enactor, int key, char *message)
             }
             else
             {
+
+#ifdef REALITY_LVLS
+                notify_except_rlevel(loc, executor, -1, message, SAY_EMIT);
+#else
                 notify_all_from_inside(loc, executor, message);
+#endif
             }
         }
         if (say_flags & SAY_ROOM)
@@ -299,7 +326,11 @@ void do_say(dbref executor, dbref caller, dbref enactor, int key, char *message)
             }
             if (isRoom(loc))
             {
+#ifdef REALITY_LVLS
+                notify_except_rlevel(loc, executor, -1, message, -1);
+#else
                 notify_all_from_inside(loc, executor, message);
+#endif
             }
         }
         break;
@@ -314,6 +345,9 @@ void do_say(dbref executor, dbref caller, dbref enactor, int key, char *message)
 void do_shout(dbref executor, dbref caller, dbref enactor, int key,
     char *message)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     char *p;
     char *buf2, *bp;
     int say_flags = key & (SAY_NOTAG | SAY_HERE | SAY_ROOM | SAY_HTML);
@@ -511,6 +545,11 @@ void do_shout(dbref executor, dbref caller, dbref enactor, int key,
 static void page_return(dbref player, dbref target, const char *tag,
     int anum, const char *dflt)
 {
+    if (MuxAlarm.bAlarmed)
+    {
+        return;
+    }
+
     dbref aowner;
     int aflags;
     char *str, *str2, *buf, *bp;
@@ -619,6 +658,10 @@ void do_page
     char *arg2
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     int   nPlayers = 0;
     dbref aPlayers[(LBUF_SIZE+1)/2];
 
@@ -639,7 +682,7 @@ void do_page
             {
                 *q = '\0';
             }
-    
+
             // Decode space-delimited or comma-delimited recipients.
             //
             MUX_STRTOK_STATE tts;
@@ -774,7 +817,6 @@ void do_page
         char *pBuff = alloc_lbuf("do_page.lastpage");
         char *pBufc = pBuff;
         ItemToList_Init(&itl, pBuff, &pBufc);
-        int i;
         for (i = 0; i < nPlayers; i++)
         {
             if (  Good_obj(aPlayers[i])
@@ -959,7 +1001,9 @@ void do_page
         if (target != NOTHING)
         {
             notify_with_cause_ooc(target, executor, omessage);
-            if (fetch_idle(target) >= idle_timeout_val(target))
+            int target_idle = fetch_idle(target);
+            int target_idle_timeout_val = idle_timeout_val(target);
+            if (target_idle >= target_idle_timeout_val)
             {
                 page_return(executor, target, "Idle", A_IDLE, NULL);
             }
@@ -981,7 +1025,7 @@ void do_page
  * do_pemit: Messages to specific players, or to all but specific players.
  */
 
-void whisper_pose(dbref player, dbref target, char *message, bool bSpace)
+static void whisper_pose(dbref player, dbref target, char *message, bool bSpace)
 {
     char *newMessage = modSpeech(player, message, true, "whisper");
     if (newMessage)
@@ -990,9 +1034,9 @@ void whisper_pose(dbref player, dbref target, char *message, bool bSpace)
     }
     char *buff = alloc_lbuf("do_pemit.whisper.pose");
     strcpy(buff, Moniker(player));
-    notify(player, tprintf("%s senses \"%s%s%s\"", Moniker(target), buff, 
+    notify(player, tprintf("%s senses \"%s%s%s\"", Moniker(target), buff,
         bSpace ? " " : "", message));
-    notify_with_cause(target, player, tprintf("You sense %s%s%s", buff, 
+    notify_with_cause(target, player, tprintf("You sense %s%s%s", buff,
         bSpace ? " " : "", message));
     free_lbuf(buff);
     if (newMessage)
@@ -1326,6 +1370,9 @@ void do_pemit
     char *message
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     if (nargs < 2)
     {
         return;
