@@ -130,10 +130,15 @@
 
 #include "autoconf.h"
 #include "config.h"
+#include "externs.h"
 #include "stringutil.h"
 
-#ifdef HAVE_FPU_CONTROL_H
+#if   defined(HAVE_FPU_CONTROL_H)
 #include <fpu_control.h>
+#elif defined(IEEEFP_H_USEABLE)
+#include <ieeefp.h>
+#elif defined(HAVE_FENV_H)
+#include <fenv.h>
 #endif
 
 #if defined(WORDS_BIGENDIAN)
@@ -421,7 +426,7 @@ static Bigint *Balloc(int k)
         x = 1 << k;
 #ifdef Omit_Private_Memory
         rv = (Bigint *)MEMALLOC(sizeof(Bigint) + (x-1)*sizeof(ULong));
-        (void)ISOUTOFMEMORY(rv);
+        ISOUTOFMEMORY(rv);
 #else
         len = (sizeof(Bigint) + (x-1)*sizeof(ULong) + sizeof(double) - 1)
             /sizeof(double);
@@ -433,7 +438,7 @@ static Bigint *Balloc(int k)
         else
         {
             rv = (Bigint*)MEMALLOC(len*sizeof(double));
-            (void)ISOUTOFMEMORY(rv);
+            ISOUTOFMEMORY(rv);
         }
 #endif
         rv->k = k;
@@ -631,7 +636,7 @@ static int lo0bits(ULong *y)
     {
         k++;
         x >>= 1;
-        if (!x & 1)
+        if (!(x & 1))
         {
             return 32;
         }
@@ -2573,7 +2578,7 @@ static char *nrv_alloc(char *s, char **rve, int n)
 
 /* freedtoa(s) must be used to free values s returned by dtoa.
  */
-void freedtoa(char *s)
+static void freedtoa(char *s)
 {
     Bigint *b = (Bigint *)((int *)s - 1);
     b->maxwds = 1 << (b->k = *(int*)b);
@@ -3467,18 +3472,136 @@ ret1:
     return s0;
 }
 
-void FLOAT_Initialize(void)
-{
-#if defined(HAVE_FPU_CONTROL_H) && defined(_FPU_GETCW) \
- && defined(_FPU_EXTENDED) && defined(_FPU_DOUBLE)
-    fpu_control_t oldcw, newcw;  
-    _FPU_GETCW(oldcw);
-    newcw = (oldcw & ~_FPU_EXTENDED) | _FPU_DOUBLE;
-    _FPU_SETCW(newcw);
+#if defined(HAVE_FPU_CONTROL_H) \
+ && defined(_FPU_GETCW) \
+ && defined(_FPU_SETCW)
+
+fpu_control_t maskoff = 0
+#if defined(_FPU_EXTENDED)
+    | _FPU_EXTENDED
 #endif
-#ifdef WIN32
-    // Set Floating-Point precision.
+#if defined(_FPU_SINGLE)
+    | _FPU_SINGLE
+#endif
+    ;
+
+fpu_control_t maskon = 0
+#if defined(_FPU_DOUBLE)
+    | _FPU_DOUBLE
+#endif
+    ;
+
+fpu_control_t origcw;
+
+void mux_FPInit(void)
+{
+    _FPU_GETCW(origcw);
+}
+
+void mux_FPSet(void)
+{
+    // Set double-precision.
+    //
+    fpu_control_t newcw;
+    newcw = (origcw & ~maskoff) | maskon;
+    _FPU_SETCW(newcw);
+}
+
+void mux_FPRestore(void)
+{
+    _FPU_SETCW(origcw);
+}
+
+#elif defined(IEEEFP_H_USEABLE)
+
+fp_rnd_t   orig_rnd;
+fp_prec_t orig_prec;
+
+void mux_FPInit(void)
+{
+    orig_rnd  = fpgetround();
+    orig_prec = fpgetprec();
+}
+
+void mux_FPSet(void)
+{
+    // Set double-precision.
+    //
+    fpsetprec(FP_PD);
+}
+
+void mux_FPRestore(void)
+{
+    fpsetprec(orig_prec);
+}
+
+#elif defined(WIN32)
+
+static unsigned origcw;
+
+void mux_FPInit(void)
+{
+    origcw = _controlfp(0, 0);
+}
+
+void mux_FPSet(void)
+{
+    // Set double-precision.
     //
     _controlfp(_PC_53, _MCW_PC);
-#endif // WIN32
+}
+
+void mux_FPRestore(void)
+{
+    const unsigned int maskall = 0xFFFFFFFF;
+
+    _controlfp(origcw, maskall);
+}
+
+#elif defined(HAVE_FENV_H) \
+   && defined(HAVE_FESETPREC) \
+   && defined(HAVE_FEGETPREC) \
+   && defined(FE_DBLPREC)
+
+int origcw;
+
+void mux_FPInit(void)
+{
+    origcw = fegetprec();
+}
+
+void mux_FPSet(void)
+{
+    // Set double-precision.
+    //
+    fesetprec(FE_DBLPREC);
+}
+
+void mux_FPRestore(void)
+{
+    fesetprec(origcw);
+}
+
+#else
+
+#warning "No method of floating-point control was found, using dummy functions"
+
+void mux_FPInit(void)
+{
+}
+
+void mux_FPSet(void)
+{
+}
+
+void mux_FPRestore(void)
+{
+}
+
+#endif
+
+void FLOAT_Initialize(void)
+{
+    mux_FPInit();
+    mux_FPSet();
 }
