@@ -1,6 +1,6 @@
 // wiz.cpp -- Wizard-only commands.
 //
-// $Id: wiz.cpp,v 1.10 2004/03/08 04:37:40 sdennis Exp $
+// $Id: wiz.cpp,v 1.16 2006/01/07 01:24:27 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -13,52 +13,21 @@
 #include "file_c.h"
 #include "powers.h"
 
-void do_teleport
+static void do_teleport_single
 (
     dbref executor,
     dbref caller,
     dbref enactor,
     int   key,
-    int   nargs,
-    char *arg1,
-    char *arg2
+    dbref victim,
+    char *to
 )
 {
-    dbref victim, destination, loc;
-    char *to;
-    int hush = 0;
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
 
-    if (  (  Fixed(executor)
-          || Fixed(Owner(executor)))
-       && !Tel_Anywhere(executor))
-    {
-        notify(executor, mudconf.fixed_tel_msg);
-        return;
-    }
-
-    // Get victim.
-    //
-    if (nargs == 1)
-    {
-        victim = executor;
-        to = arg1;
-    }
-    else if (nargs == 2)
-    {
-        init_match(executor, arg1, NOTYPE);
-        match_everything(0);
-        victim = noisy_match_result();
-
-        if (victim == NOTHING)
-        {
-            return;
-        }
-        to = arg2;
-    }
-    else
-    {
-        return;
-    }
+    dbref loc;
+    int   hush = 0;
 
     // Validate type of victim.
     //
@@ -72,9 +41,7 @@ void do_teleport
     // Fail if we don't control the victim or the victim's location.
     //
     if (  !Controls(executor, victim)
-       && (  (  isExit(victim)
-             && !Controls(executor, Home(victim)))
-          || !Controls(executor, Location(victim)))
+       && !Controls(executor, isExit(victim) ? Exits(victim) : Location(victim))
        && !Tel_Anything(executor))
     {
         notify_quiet(executor, NOPERM_MESSAGE);
@@ -100,7 +67,7 @@ void do_teleport
     //
     init_match(executor, to, NOTYPE);
     match_everything(0);
-    destination = match_result();
+    dbref destination = match_result();
 
     switch (destination)
     {
@@ -226,6 +193,73 @@ void do_teleport
     }
 }
 
+void do_teleport
+(
+    dbref executor,
+    dbref caller,
+    dbref enactor,
+    int   key,
+    int   nargs,
+    char *arg1,
+    char *arg2
+)
+{
+    if (  (  Fixed(executor)
+          || Fixed(Owner(executor)))
+       && !Tel_Anywhere(executor))
+    {
+        notify(executor, mudconf.fixed_tel_msg);
+        return;
+    }
+
+    // Get victim.
+    //
+    if (nargs == 1)
+    {
+        // Teleport executor to given destination.
+        //
+        do_teleport_single(executor, caller, enactor, key, executor, arg1);
+    }
+    else if (nargs == 2)
+    {
+        // Teleport 3rd part(y/ies) to given destination.
+        //
+        if (key & TELEPORT_LIST)
+        {
+            // We have a space-delimited list of victims.
+            //
+            char *p;
+            MUX_STRTOK_STATE tts;
+            mux_strtok_src(&tts, arg1);
+            mux_strtok_ctl(&tts, " ");
+            for (p = mux_strtok_parse(&tts); p; p = mux_strtok_parse(&tts))
+            {
+                init_match(executor, p, NOTYPE);
+                match_everything(0);
+                dbref victim = noisy_match_result();
+
+                if (Good_obj(victim))
+                {
+                    do_teleport_single(executor, caller, enactor, key, victim,
+                        arg2);
+                }
+            }
+        }
+        else
+        {
+            init_match(executor, arg1, NOTYPE);
+            match_everything(0);
+            dbref victim = noisy_match_result();
+
+            if (Good_obj(victim))
+            {
+                do_teleport_single(executor, caller, enactor, key, victim,
+                    arg2);
+            }
+        }
+    }
+}
+
 // ---------------------------------------------------------------------------
 // do_force_prefixed: Interlude to do_force for the # command
 //
@@ -253,6 +287,9 @@ void do_force_prefixed( dbref executor, dbref caller, dbref enactor, int key,
 void do_force( dbref executor, dbref caller, dbref enactor, int key,
                char *what, char *command, char *args[], int nargs )
 {
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref victim = match_controlled(executor, what);
     if (victim != NOTHING)
     {
@@ -278,6 +315,9 @@ void do_toad
     char *newowner
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     dbref victim, recipient, loc, aowner;
     char *buf;
     int count, aflags;
@@ -396,6 +436,11 @@ void do_newpassword
     char *password
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(nargs);
+
     dbref victim = lookup_player(executor, name, false);
     if (victim == NOTHING)
     {
@@ -456,6 +501,9 @@ void do_newpassword
 
 void do_boot(dbref executor, dbref caller, dbref enactor, int key, char *name)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     if (!Can_Boot(executor))
     {
         notify(executor, NOPERM_MESSAGE);
@@ -538,6 +586,11 @@ void do_boot(dbref executor, dbref caller, dbref enactor, int key, char *name)
 //
 void do_poor(dbref executor, dbref caller, dbref enactor, int key, char *arg1)
 {
+    UNUSED_PARAMETER(executor);
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     if (!is_rational(arg1))
     {
         return;
@@ -564,6 +617,10 @@ void do_poor(dbref executor, dbref caller, dbref enactor, int key, char *arg1)
 //
 void do_cut(dbref executor, dbref caller, dbref enactor, int key, char *thing)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(key);
+
     dbref object = match_controlled(executor, thing);
     if (Good_obj(object))
     {
@@ -577,6 +634,9 @@ void do_cut(dbref executor, dbref caller, dbref enactor, int key, char *thing)
 //
 void do_motd(dbref executor, dbref caller, dbref enactor, int key, char *message)
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+
     bool is_brief = false;
 
     if (key & MOTD_BRIEF)
@@ -692,12 +752,13 @@ NAMETAB enable_names[] =
 
 void do_global(dbref executor, dbref caller, dbref enactor, int key, char *flag)
 {
-    int flagvalue;
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
 
     // Set or clear the indicated flag.
     //
-    flagvalue = search_nametab(executor, enable_names, flag);
-    if (flagvalue < 0)
+    int flagvalue;
+    if (!search_nametab(executor, enable_names, flag, &flagvalue))
     {
         notify_quiet(executor, "I don't know about that flag.");
     }
